local waitTable = {};
local waitFrame = nil;

local debug = true;
-- functions
function cui_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function cui_info(value, name)
    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ffCleanUI:|r "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ffCleanUI:|r "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

function CleanUI_TakeScreenshot()   
    CleanUI_StartDelay(1.2, Screenshot);
end

function CleanUI_RandomPet()
    SummonRandomCritter();
end

function CleanUI_RandomMount(flying, ground, swimming)
    if (IsMounted()) then
        if (IsFlying()) then
            if (IsControlKeyDown()) then
                Dismount();
            end
        else
            Dismount();
        end
    else
        CleanUI_SearchMount(flying, ground, swimming);
    end
end

function CleanUI_SearchMount(flying, ground, swimming)    
    local countFlyable = 0;
    if (flying) then
        countFlyable = #flying;
    end
    
    local countGround = 0;
    if (ground) then
        countGround = #ground;
    end
    
    local countSwimming = 0;
    if (swimming) then
        countSwimming = #swimming;
    end
    
    local type = CleanUI_GetMountType(countFlyable, countGround, countSwimming);
    
    if (type == 1) then
        -- flyable
        CleanUI_FindRandomMount(flying);
    elseif (type == 3) then
        -- under water
        CleanUI_FindRandomMount(swimming);
    else
        -- 0 = non flyable area, 2 = no licence
        -- non flyable
        CleanUI_FindRandomMount(ground);
    end
end

function CleanUI_FindRandomMount(mounts) 
                
    local count = #mounts;
    
    if (count > 0) then    
        local rnd = math.random(1, count);
        local name = mounts[rnd];
            
        CleanUI_MountRufen(name);
    end
end

function CleanUI_MountRufen(name)   
    for i=1, GetNumCompanions("MOUNT"), 1 do
        creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
                
        if (creatureName == name) then
            CallCompanion("MOUNT", i);
            break;
        end
    end
end

function CleanUI_GetMountType(countFlyable, countGround, countSwimming)
    
    local zone = GetZoneText();
    local subzone = GetSubZoneText();
            
    if (IsAltKeyDown() or countFlyable == 0) then
        return 0;
    end
    
    if ( not WorldMapFrame:IsShown() ) then
        SetMapToCurrentZone();
    end
    
    if (GetCurrentMapAreaID() == WORLDMAP_WINTERGRASP_ID) then
        local nextBattleTime = select(5, GetWorldPVPAreaInfo(1));
        
        if (nextBattleTime == nil) then
            return 0;
        end
    end
    
    local continent = GetCurrentMapContinent();
    local areaid = GetCurrentMapAreaID();

    -- stliche Knigreiche = 2
    -- Abyssische Tiefen == 614
    -- Schimmernde Weiten == 615
    -- Tang'tharwald == 610
    -- Vashj'ir == 613
    if (countSwimming > 0 and IsSwimming() and continent == 2 and (areaid == 613 or areaid == 614 or areaid == 615 or areaid == 610)) then
        return 3; -- under water
    end
    
    if (IsFlyableArea() == nil) then
        return 0;
    end
    
    -- 1 = Kalimdor
    -- 2 = Azeroth
    -- 3 = Scherbenwelt
    -- 4 = Northrend
    -- 6 = Pandaria
    
    -- ColdWeatherFlying = 54197
    -- FlightMastersLicense = 90267
    -- Pandaria Flight License = 115913
    
    local licence = nil;
    
    if (continent == 1 or continent == 2) then
        licence = 90267;
    elseif (continent == 4) then
        licence = 54197;
    elseif (continent == 6) then
        licence = 115913;
    end

    if (licence) then
        -- check licence    
        local hasLicence = CleanUI_HasFlightLicense(licence);
        
        if (hasLicence) then
            return 1;
        else
            return 2;
        end
    else
        if (continent == 3) then
            return 1;
        end
    end
    
    return 2;
end

function CleanUI_HasFlightLicense(spellid)

    local i = 1;
    
    local spellName = GetSpellInfo(spellid);
    
    while true do
        local foundName, foundSubName = GetSpellBookItemName(i, BOOKTYPE_SPELL);
        
        if (not foundName) then
            return false;
        end
        
        if (spellName == foundName and IsSpellKnown(spellid)) then
            return true;
        end
       
       i = i + 1;
    end
    
    return false;
end

function CleanUI_TakeScreenshotWithoutUI()    
    if ( UIParent:IsShown() ) then
        securecall("CloseMenus");
        securecall("CloseAllWindows");
        UIParent:Hide();
        SetUIVisibility(false);
    end 
    
    --TakeScreenshot();
    
    local screenshotFrame = _G["CUI_SCREENSHOT_FRAME"];
    if (not screenshotFrame) then
        screenshotFrame = CreateFrame("Frame", "CUI_SCREENSHOT_FRAME"); 
    end
    
    screenshotFrame.last = 0.0;
    screenshotFrame.shot = 1.0;
    screenshotFrame.throttle = 3.0;
    screenshotFrame.foto = nil;
        
    screenshotFrame:SetScript('OnUpdate', function(self, elapsed)
        self.last = self.last + elapsed;
        
        if (self.last >= self.shot and not self.foto) then
            TakeScreenshot();
            self.foto = 1;
        end
        
        if (self.last >= self.throttle) then            
            self:SetScript('OnUpdate', nil);      
            UIParent:Show();
            SetUIVisibility(true); 
        end
    end)
    
    screenshotFrame:Show();
end

function CleanUI_StartDelay(delay, func, ...)
    if(type(delay)~="number" or type(func)~="function") then
        return false;
    end
    
    if(waitFrame == nil) then
        waitFrame = CreateFrame("Frame","CUI_WaitFrame", UIParent);
        waitFrame:SetScript("onUpdate",function (self,elapse)
            local count = #waitTable;
            local i = 1;
            
            while(i<=count) do
                local waitRecord = tremove(waitTable,i);
                local d = tremove(waitRecord,1);
                local f = tremove(waitRecord,1);
                local p = tremove(waitRecord,1);
                
                if(d>elapse) then
                    tinsert(waitTable,i,{d-elapse,f,p});
                    i = i + 1;
                else
                    count = count - 1;
                    f(unpack(p));
                end
            end
        end);
    end
    
    tinsert(waitTable,{delay,func,{...}});
    
    return true;
end

function CleanUI_MakeFrameMoveable(frame)  
    frame:SetMovable(true); 
    frame:SetUserPlaced(true); 
    frame:SetClampedToScreen(true);
    frame:SetClampRectInsets(1, -1, -1, 1);
    
    frame:HookScript("OnMouseDown", CleanUI_AnyFrameOnMouseDown);
    frame:HookScript("OnMouseUp", CleanUI_AnyFrameOnMouseUp);
end

function CleanUI_AnyFrameOnMouseDown(self, button)
    if (button == "LeftButton") then
        self:StartMoving();
    end
end

function CleanUI_AnyFrameOnMouseUp(self, button)
    self:StopMovingOrSizing();
end

function cui_rgb2hex(red, green, blue)

    if (not red) then
        red = 1.0;
    end

    if (not green) then
        green = 1.0;
    end

    if (not blue) then
        blue = 1.0;
    end

    local r = string.format("%x", red * 255);
    local g = string.format("%x", green * 255);
    local b = string.format("%x", blue * 255);
    
    if (strlen(r) < 2) then
        r = "0"..r;
    end
    if (strlen(g) < 2) then
        g = "0"..g;
    end
    if (strlen(b) < 2) then
        b = "0"..b;
    end
    
    local color = r..g..b;
    
    return color;
end

function CleanUI_ReplaceAllTextures(texturename, frame, withTexture)

    if (CleanUIData.AUTO_CHANGE_BORDER_STYLE ~= 1) then
        return;
    end

    for i, child in ipairs({frame:GetChildren()}) do
    
        if (child and child:IsObjectType("Frame")) then
            CleanUI_ReplaceTextureWith(texturename, child, withTexture);
            
            if (child:IsObjectType("frame")) then           
                CleanUI_ReplaceAllTextures(texturename, child, withTexture);
            end
        end
    end
end

function CleanUI_ReplaceTextureWith(texturename, frame, withTexture)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            if (texture) then
                if (string.find(texture, texturename, 1, true)) then
                    child:SetTexture(withTexture);
                end
            end
        end
    end
end

function CleanUI_RemoveTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
    
        if (child and child:IsObjectType("texture")) then
            child:SetTexture(nil);
        end
    end
end

function CleanUI_GetRegionOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildWithName(frame, name)
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            return child;
        end
    end
end

function CleanUI_GetChildWithNameAtPos(frame, name, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_ListChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");
            
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
            child:Show();
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");
            child:Show();
            
            if (not child:IsObjectType("button")) then
                CleanUI_ShowTextures(child);
            end
        end
    end
end

function CleanUI_ListTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then            
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            child:Show();
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then  
            child:Show();          
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_GC()
    local before = collectgarbage("count");
    collectgarbage();
    local after = collectgarbage("count");
    
    local freed = before - after;
    
    local msg = "collectgarbage:\n"..
        "memory freed: |c00FFFF66"..string.format("%.2f", freed).." kb|r.\n"..
        "current memory use: |c00FFFF66"..string.format("%.2f", after).." kb|r.\n";
        
    DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 1.0, 1, 10);
end

function CleanUI_WoWBuildInfo()
    local version, build, date, tocversion = GetBuildInfo();
    cui_debug(version, "version");
    cui_debug(build, "build");
    cui_debug(date, "date");
    cui_debug(tocversion, "tocversion");
end

-- hook
function CleanUI_HookChangeStatusBar(texture)    
end

function CleanUI_XXXXX(message)
end