local headerText;

function CleanUI_InitAuctionFrame()  
    CleanUI_RegisterAuctionFrameEvents();
end
    
function CleanUI_RegisterAuctionFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED");
    
    frame:HookScript("OnEvent", CleanUI_AuctionFrameOnEvent); 
end

function CleanUI_AuctionFrameHookEvents()
    hooksecurefunc("AuctionFrameTab_OnClick", CleanUI_AuctionFrameTab_OnClick);
    hooksecurefunc("AuctionFrameBrowse_Update", CleanUI_AuctionFrameBrowse_Update);    
end

function CleanUI_AuctionFrameTab_OnClick(self, button, down, index)
    CleanUI_RemoveTextures(AuctionFrame);  
    CleanUI_SetBackdrop(AuctionFrame);
    
    local index = self:GetID();
    
    if (index == 1) then
        headerText:SetText(BROWSE_AUCTIONS);
    elseif (index == 2) then
        headerText:SetText(BID);
    elseif (index == 3) then
        headerText:SetFormattedText(AUCTION_TITLE, UnitName("player"));
    end
end

function CleanUI_AuctionFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_AuctionUI" ) then   
            CleanUI_AuctionFrameHookEvents();
            CleanUI_CleanAuctionFrame();
        end
    end
end
    
function CleanUI_CleanAuctionFrame()
    CleanUI_RemoveTextures(AuctionFrame);  
    CleanUI_SetBackdrop(AuctionFrame); 
    AuctionFrame:SetHeight(442);
    
    AuctionPortraitTexture:Hide();
        
    CleanUI_ChangeCloseButton(AuctionFrame);
    
    _, headerText = CleanUI_CreateHeaderFrame(AuctionFrame, "CUI_AuctionFrameHeaderBorderframe", 250, nil, BROWSE_AUCTIONS);
    BrowseTitle:Hide();
    BidTitle:Hide();
    AuctionsTitle:Hide();
    
    AuctionFrameTab1:SetPoint("TOPLEFT", AuctionFrame, "BOTTOMLEFT", 15, 2);
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["AuctionFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
    
    local filterButton;
    for i=1, 20 do
        filterButton = _G["AuctionFilterButton"..i];
        
        if (filterButton) then
            CleanUI_ChangeFilterButton(filterButton);
        end
    end
    
    CleanUI_RemoveTextures(BrowsePrevPageButton);        
    CleanUI_ChangeBackButton(BrowsePrevPageButton);
        
    CleanUI_RemoveTextures(BrowseNextPageButton);       
    CleanUI_ChangeNextButton(BrowseNextPageButton);
    
    CleanUI_RemoveTextures(BrowseScrollFrame);    
    CleanUI_RemoveTextures(BrowseFilterScrollFrame);
    CleanUI_ChangeButton(BrowseBidButton);
    CleanUI_ChangeButton(BrowseBuyoutButton);
    CleanUI_ChangeButtonRed(BrowseCloseButton);
    CleanUI_ChangeButton(BrowseSearchButton);
    CleanUI_ChangeButton(BrowseResetButton);
    
    CleanUI_ChangeDropDown(BrowseDropDown); 
    
    BrowseNameText:SetPoint("TOPLEFT", 30, -20);
    
    BrowseLevelText:ClearAllPoints();
    BrowseLevelText:SetPoint("TOPLEFT", 180, -20);
    
    CleanUI_ChangeEditBox(BrowseName, 32, -10, 5);
    BrowseName:SetPoint("TOPLEFT", BrowseNameText, "BOTTOMLEFT", 3, 0);
    CleanUI_ChangeEditBox(BrowseMinLevel, 33, -14, 5);
    CleanUI_ChangeEditBox(BrowseMaxLevel, 33, -14, 5);
    BrowseMinLevel:SetPoint("TOPLEFT", BrowseLevelText, "BOTTOMLEFT", 3, 0);
    
    BrowseSearchButton:ClearAllPoints();
    BrowseSearchButton:SetPoint("TOPLEFT", 30, -58);
    BrowseResetButton:ClearAllPoints();
    BrowseResetButton:SetPoint("LEFT", BrowseSearchButton, "RIGHT", 2, 0);
    
    BrowseBidPrice:SetPoint("BOTTOM", 25, 23);
    BrowseBidText:SetPoint("RIGHT", BrowseBidPrice, "LEFT", -10, -7);
    CleanUI_ChangeEditBox(BrowseBidPriceGold, 32, -12, 10);
    CleanUI_ChangeEditBox(BrowseBidPriceSilver, 32, -12, 0);
    CleanUI_ChangeEditBox(BrowseBidPriceCopper, 32, -12, 0);
    
    CleanUI_ChangeFilterButton(BrowseQualitySort, true); 
    CleanUI_ChangeFilterButton(BrowseLevelSort, true); 
    CleanUI_ChangeFilterButton(BrowseDurationSort, true); 
    CleanUI_ChangeFilterButton(BrowseHighBidderSort, true); 
    CleanUI_ChangeFilterButton(BrowseCurrentBidSort, true); 
    
    CleanUI_RemoveTextures(BidScrollFrame);
    CleanUI_ChangeButton(BidBidButton);
    CleanUI_ChangeButton(BidBuyoutButton);
    CleanUI_ChangeButtonRed(BidCloseButton);
    
    BidBidPrice:SetPoint("BOTTOM", 25, 23);
    BidBidText:SetPoint("RIGHT", BidBidPrice, "LEFT", -10, -7);
    CleanUI_ChangeEditBox(BidBidPriceGold, 32, -12, 10);
    CleanUI_ChangeEditBox(BidBidPriceSilver, 32, -12, 0);
    CleanUI_ChangeEditBox(BidBidPriceCopper, 32, -12, 0);
    
    CleanUI_ChangeFilterButton(BidQualitySort, true); 
    CleanUI_ChangeFilterButton(BidLevelSort, true); 
    CleanUI_ChangeFilterButton(BidDurationSort, true); 
    CleanUI_ChangeFilterButton(BidBuyoutSort, true); 
    CleanUI_ChangeFilterButton(BidStatusSort, true); 
    CleanUI_ChangeFilterButton(BidBidSort, true);  
    
    CleanUI_RemoveTextures(AuctionsScrollFrame);
    CleanUI_ChangeButton(AuctionsCreateAuctionButton);
    CleanUI_ChangeButton(AuctionsCancelAuctionButton);
    CleanUI_ChangeButtonRed(AuctionsCloseButton);
    
    CleanUI_ChangeFilterButton(AuctionsQualitySort, true); 
    CleanUI_ChangeFilterButton(AuctionsDurationSort, true); 
    CleanUI_ChangeFilterButton(AuctionsHighBidderSort, true); 
    CleanUI_ChangeFilterButton(AuctionsBidSort, true); 
    
    CleanUI_ChangeDropDown(PriceDropDown); 
    CleanUI_ChangeDropDown(DurationDropDown); 
    
    CleanUI_ChangeEditBox(StartPriceGold, 32, -12, 10);
    CleanUI_ChangeEditBox(StartPriceSilver, 32, -12, 0);
    CleanUI_ChangeEditBox(StartPriceCopper, 32, -12, 0);
    
    CleanUI_ChangeEditBox(BuyoutPriceGold, 32, -12, 10);
    CleanUI_ChangeEditBox(BuyoutPriceSilver, 32, -12, 0);
    CleanUI_ChangeEditBox(BuyoutPriceCopper, 32, -12, 0);
    
    DurationDropDown:ClearAllPoints();
    DurationDropDown:SetPoint("BOTTOMLEFT", 67, 80);
    
    CleanUI_RemoveTextures(SideDressUpFrame);  
    CleanUI_SetBackdrop(SideDressUpFrame); 
    CleanUI_RemoveTextures(SideDressUpModelCloseButton); 
    CleanUI_ChangeCloseButton(SideDressUpFrame, "SideDressUpModelCloseButton");
    CleanUI_ChangeButton(SideDressUpModelResetButton);
end

function CleanUI_AuctionFrameBrowse_Update()
	local offset = FauxScrollFrame_GetOffset(BrowseScrollFrame);
	
	local button, index, name, texture, count, quality, canUse, level, levelColHeader, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, owner, ownerFullName, saleStatus, itemId, hasAllInfo;
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType;
	local stats;
	
	for i=1, NUM_BROWSE_TO_DISPLAY do
		index = offset + i + (NUM_AUCTION_ITEMS_PER_PAGE * AuctionFrameBrowse.page);
		button = _G["BrowseButton"..i];
		
		if (not button.additionalInfo) then			    
		    button.additionalInfo = button:CreateFontString(button:GetName().."AdditionalInfo", "ARTWORK");
			button.additionalInfo:SetFontObject("CuiFont_VerySmall");
		    button.additionalInfo:SetWidth(200);
		    button.additionalInfo:SetHeight(10);
		    button.additionalInfo:SetTextColor(1.0, 1.0, 1.0, 1.0);
		    button.additionalInfo:SetJustifyH("LEFT");
		    button.additionalInfo:SetText("");			
    		button.additionalInfo:SetPoint("BOTTOMLEFT", button, "BOTTOMLEFT", 43, 5);
		end
		
		if (button:IsVisible()) then
			name, texture, count, quality, canUse, level, levelColHeader, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, owner, ownerFullName, saleStatus, itemId, hasAllInfo =  GetAuctionItemInfo("list", offset + i);
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType = GetItemInfo(itemId);
			
			stats = CleanUI_GetGemStats(itemId);			
			button.additionalInfo:SetText(stats or "");	
		end
	end
end

local tip = CreateFrame("GameTooltip", "GemStatsTip");
local line1 = tip:CreateFontString();
local line2 = tip:CreateFontString();
local line3 = tip:CreateFontString();

tip:AddFontStrings(tip:CreateFontString(), tip:CreateFontString());
tip:AddFontStrings(line1, tip:CreateFontString());
tip:AddFontStrings(line2, tip:CreateFontString());
tip:AddFontStrings(line3, tip:CreateFontString());

local GEM_S = "%+[0-9]+.*";

function CleanUI_GetGemStats(id)
    tip:SetOwner(WorldFrame, "ANCHOR_NONE");
    tip:SetItemByID(id);

    if(tip:IsShown()) then
        return string.match(line1:GetText() or '', GEM_S) or string.match(line2:GetText() or '', GEM_S) or string.match(line3:GetText() or '', GEM_S);
    end
end
