local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local initialized = false;

local playerData = {};
local enemyData = {};

function CleanUI_InitBattlePets()
    CleanUI_CleanPetBattleUI();
    
    CleanUI_InitPetBattleInfoPanel();
    CleanUI_BattlePetsHookEvents();
    CleanUI_RegisterBattlePetsEvents();
end
    
function CleanUI_RegisterBattlePetsEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("PET_BATTLE_PET_CHANGED"); 
    frame:RegisterEvent("PET_BATTLE_OPENING_DONE"); 
    frame:RegisterEvent("PET_BATTLE_CLOSE"); 
    
    frame:HookScript("OnEvent", CleanUI_BattlePetsOnEvent); 
    
    hooksecurefunc("PetBattleFrame_UpdateActionBarLayout", CleanUI_PetBattleFrame_UpdateActionBarLayout);
    hooksecurefunc("PetBattleFrame_UpdatePassButtonAndTimer", CleanUI_PetBattleFrame_UpdatePassButtonAndTimer);    
    
    CleanUIBattlePetsInfo:Hide();
end

function CleanUI_BattlePetsOnEvent(self, event, ...)    
    if (event == "PET_BATTLE_PET_CHANGED") then
        CleanUI_PetBattleFrame_UpdateBattlePetInfo();
    end
    
    if (event == "PET_BATTLE_OPENING_DONE") then
        CleanUI_PetBattleFrame_LoadBattlePetInfo();
        CleanUI_PetBattleFrame_UpdateBattlePetInfo();
        CleanUI_ShowPetBattleChatInfo();
        CleanUIBattlePetsInfo:Show();
    end
    
    if (event == "PET_BATTLE_CLOSE") then
        CleanUI_PetBattleFrame_Close();
        CleanUIBattlePetsInfo:Hide();
    end
end

function CleanUI_BattlePetsHookEvents()    
    GameTooltip:HookScript("OnTooltipSetItem", CleanUI_AddTooltipInfo);
    ItemRefTooltip:HookScript("OnTooltipSetItem", CleanUI_AddTooltipInfo);

    GameTooltip:HookScript("OnTooltipSetUnit", function(self)
        local name, unit = GameTooltip:GetUnit();
        if unit and UnitIsWildBattlePet(unit) then
            CleanUI_AddTooltipInfo(self, name);
        end
    end);
end

function CleanUI_AddTooltipInfo(tooltip, name)
    local info, levelinfo = CleanUI_PetBattleFrame_TamedInfo(name);
    tooltip:AddLine(levelinfo, 1.0, 1.0, 1.0);
  
    tooltip:Show();
end

function CleanUI_PetBattleFrame_LoadBattlePetInfo()
    CleanUI_PetBattleFrame_ScanData(LE_BATTLE_PET_ALLY, playerData);
    CleanUI_PetBattleFrame_ScanData(LE_BATTLE_PET_ENEMY, enemyData);
end

function CleanUI_PetBattleFrame_ScanData(petOwner, data)
    local num = C_PetBattles.GetNumPets(petOwner);
    
    local species, petName, petIcon, petType, petCreatureID, petSourceText, petDescription, petIsWild, petCanBattle, petTradable, petUnique, petLevel, petQuality, abilityId;
    local maxAbilities;
    
    for i = 1, num do
        species = C_PetBattles.GetPetSpeciesID(petOwner, i);
        petName, petIcon, petType, petCreatureID, petSourceText, petDescription, petIsWild, petCanBattle, petTradable, petUnique = C_PetJournal.GetPetInfoBySpeciesID(species);
        petLevel = C_PetBattles.GetLevel(petOwner, i);
        petQuality = C_PetBattles.GetBreedQuality(petOwner,i)
        
        data[i] = {
            name = petName,
            icon = petIcon,
            type = petType,
            level = petLevel,
            quality = petQuality,
            unique = petUnique,
            abilities = {},
        };
        
        maxAbilities = 1;
        if (petLevel >= 4) then
            maxAbilities = 3;
        elseif (petLevel >= 2) then
            maxAbilities = 2;
        end
        
        for j = 1, maxAbilities do
            abilityId = C_PetBattles.GetAbilityInfo(petOwner, i, j);
            table.insert(data[i].abilities, abilityId);
        end
    end
end

function CleanUI_ShowPetBattleChatInfo()
    DEFAULT_CHAT_FRAME:AddMessage("Pet-Info:", 1.0, 1.0, 1.0, 1, 10);    
        
    for i, pet in ipairs(enemyData) do
        DEFAULT_CHAT_FRAME:AddMessage(i..": "..ITEM_QUALITY_COLORS[pet.quality-1].hex..pet.name.."|r ("..pet.level..")", 1.0, 1.0, 1.0, 1, 10);
    
        if (pet.unique) then
            DEFAULT_CHAT_FRAME:AddMessage("   - "..ITEM_UNIQUE, 1.0, 1.0, 1.0, 1, 10);
        end  
        
        DEFAULT_CHAT_FRAME:AddMessage("   - "..CleanUI_PetBattleFrame_TamedInfo(pet.name), 1.0, 1.0, 1.0, 1, 10);        
    end
end
    
function CleanUI_CleanPetBattleUI()        
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI");
    end
    
	CleanUI_RemoveTextures(PetBattleFrame);
	CleanUI_RemoveTextures(PetBattleFrame.BottomFrame);
    CleanUI_SetBackdrop(PetBattleFrame.BottomFrame);
	
    CleanUI_RemoveTextures(PetBattleFrame.BottomFrame.xpBar);
    PetBattleFrame.BottomFrame.xpBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR); 
    PetBattleFrame.BottomFrame.xpBar:SetWidth(PetBattleFrame.BottomFrame:GetWidth()-8);
    PetBattleFrame.BottomFrame.xpBar:SetPoint("BOTTOM", PetBattleFrame.BottomFrame, "TOP", 0, -15);
    
	CleanUI_RemoveTextures(PetBattleFrame.BottomFrame.TurnTimer);
    
    PetBattleFrame.BottomFrame.FlowFrame:Hide();
    PetBattleFrame.BottomFrame.MicroButtonFrame:Hide();    
    
    CleanUI_ChangeButton(PetBattleFrame.BottomFrame.TurnTimer.SkipButton);
    
    PetBattleFrame.WeatherFrame:ClearAllPoints(); 
    PetBattleFrame.WeatherFrame:SetPoint("TOP", PetBattleFrame, "TOP", 0, -112);
    
	local borderframe = _G["CUIPetBattleFrameBorderFrame"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUIPetBattleFrameBorderFrame", PetBattleFrame);
	end
	borderframe:SetPoint("TOPLEFT", PetBattleFrame, "TOP", -515, 2);
	borderframe:SetPoint("BOTTOMRIGHT", PetBattleFrame, "TOP", 515, -90);
    borderframe:SetFrameStrata("BACKGROUND");

	CleanUI_SetBackdrop(borderframe);	
	PetTrackerAbilityAction1:SetScale(0.8);
	PetTrackerAbilityAction2:SetScale(0.8);
	PetTrackerAbilityAction3:SetScale(0.8);
	
	PetTrackerAbilityAction1:ClearAllPoints();
	PetTrackerAbilityAction1:SetPoint("TOPLEFT", PetBattleFrame, "TOP", 100, -58);
	PetTrackerAbilityAction2:ClearAllPoints();
	PetTrackerAbilityAction2:SetPoint("LEFT", PetTrackerAbilityAction1, "RIGHT", 4, 0);
	PetTrackerAbilityAction3:ClearAllPoints();
	PetTrackerAbilityAction3:SetPoint("LEFT", PetTrackerAbilityAction2, "RIGHT", 4, 0);
end

function CleanUI_PetBattleFrame_UpdateActionBarLayout()

    PetBattleFrame.BottomFrame.xpBar:SetSize(PetBattleFrame.BottomFrame:GetWidth()-8, 14);
    PetBattleFrame.BottomFrame.xpBar:SetPoint("BOTTOM", PetBattleFrame.BottomFrame, "TOP", 0, -18);
    
	if (initialized) then
		return;
	end
    
    if (MASQUE and buttongroup) then
    	buttongroup:AddButton(PetTrackerAbilityAction1);
    	PetTrackerAbilityAction1.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	buttongroup:AddButton(PetTrackerAbilityAction2);
    	PetTrackerAbilityAction2.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	buttongroup:AddButton(PetTrackerAbilityAction3);
    	PetTrackerAbilityAction3.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	
		for i=1, NUM_BATTLE_PET_ABILITIES do
    		buttongroup:AddButton(PetBattleFrame.BottomFrame.abilityButtons[i]);
    		PetBattleFrame.BottomFrame.abilityButtons[i].Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	end
    	
    	buttongroup:AddButton(PetBattleFrame.BottomFrame.SwitchPetButton);
    	PetBattleFrame.BottomFrame.SwitchPetButton.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	
    	buttongroup:AddButton(PetBattleFrame.BottomFrame.CatchButton);
    	PetBattleFrame.BottomFrame.CatchButton.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    	buttongroup:AddButton(PetBattleFrame.BottomFrame.ForfeitButton);
    	PetBattleFrame.BottomFrame.ForfeitButton.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    end
    
    PetBattleFrame.BottomFrame.TurnTimer.SkipButton:ClearAllPoints(); 
    PetBattleFrame.BottomFrame.TurnTimer.SkipButton:SetPoint("BOTTOM", PetBattleFrame.BottomFrame, "BOTTOM", 35, 2);
    PetBattleFrame.BottomFrame.TurnTimer.SkipButton:SetHeight(16);
	
	initialized = true;
end

function CleanUI_PetBattleFrame_UpdatePassButtonAndTimer(self)
    PetBattleFrame.BottomFrame.TurnTimer.SkipButton:ClearAllPoints(); 
    PetBattleFrame.BottomFrame.TurnTimer.SkipButton:SetPoint("BOTTOM", PetBattleFrame.BottomFrame, "BOTTOM", 35, 0);
end

function CleanUI_InitPetBattleInfoPanel()
    CleanUIBattlePetsInfo:SetParent(PetBattleFrame.BottomFrame);
    CleanUIBattlePetsInfo:ClearAllPoints(); 
    CleanUIBattlePetsInfo:SetPoint("TOPLEFT", PetBattleFrame.BottomFrame.MicroButtonFrame, "TOPLEFT", -17, 10);
end

function CleanUI_PetBattleFrame_Close()
    playerData = {};
    enemyData = {};
end

function CleanUI_PetBattleFrame_UpdateBattlePetInfo()
    local activeAlly = C_PetBattles.GetActivePet(LE_BATTLE_PET_ALLY);
    local activeEnemy = C_PetBattles.GetActivePet(LE_BATTLE_PET_ENEMY);
    
    local data = enemyData[activeEnemy];
    
    if (not data) then
        return;
    end
    
    CleanUIBattlePetsInfo.icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[data.type]);
    CleanUIBattlePetsInfo.name:SetText(ITEM_QUALITY_COLORS[data.quality-1].hex..data.name.."|r ("..data.level..")");
    
    if (data.unique) then
        CleanUIBattlePetsInfo.unique:SetText(ITEM_UNIQUE);
    else
        CleanUIBattlePetsInfo.unique:SetText("");
    end
    
    CleanUIBattlePetsInfo.tamed:SetText(CleanUI_PetBattleFrame_TamedInfo(data.name));
end

function CleanUI_PetBattleFrame_TamedInfo(checkname)
    local info = CUI_BATTLEPETS_NOT_TAMED;
    local levelinfo = "";
    
    local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle;
    local health, maxHealth, power, speed, rarity;
    
    local isWild = false;    
    local numPets, numOwned = C_PetJournal.GetNumPets(isWild);
    
    local tamed = nil;
    local count = 0;
    local ownedData = {};

    for index=1, numPets do
        petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, creatureID, sourceText, description, isWildPet, canBattle = C_PetJournal.GetPetInfoByIndex(index, isWild);
        
        if (petID) then
            health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(petID);
            
            if (isOwned) then            
                if (name == checkname) then
                    tamed = 1;
                    count = count + 1;
                    ownedData[count] = {level = level, rarity = rarity};
                end
            end
        end
    end
    
    if (tamed) then
        levelinfo = "@Level: ";
        local level, rarity;
        
        for index=1, count do  
            level = ownedData[index].level;
            rarity = ownedData[index].rarity;
            levelinfo = levelinfo..ITEM_QUALITY_COLORS[rarity-1].hex..level.."|r";
            if (count > index) then
                levelinfo = levelinfo..", ";
            end
        end
        
        info = CUI_BATTLEPETS_TAMED.." "..levelinfo;
    end
    
    return info, levelinfo;
end