

function CleanUI_InitCalendarFrame()    
    CleanUI_CalendarFrameHookEvents();
    CleanUI_RegisterCalendarFrameEvents();
end
    
function CleanUI_RegisterCalendarFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_CalendarFrameOnEvent); 
end

function CleanUI_CalendarFrameHookEvents()
    --hooksecurefunc("foo", bar);
end

function CleanUI_CalendarFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_Calendar" ) then 
            CleanUI_CleanCalendarFrame();
        end
    end
end
    
function CleanUI_CleanCalendarFrame()
    -- CalendarFrame
    CleanUI_RemoveTextures(CalendarFrame);  
    CleanUI_SetBackdrop(CalendarFrame); 
    
    CleanUI_ChangeCloseButton(CalendarFrame, nil, CalendarCloseButton); 
    
    CleanUI_CreateHeaderFrame(CalendarFrame, "CUI_CalendarFrameHeaderBorderframe", 250, CalendarMonthName);
    CalendarMonthBackground:SetPoint("TOP", 0, 16);
    CalendarYearBackground:SetPoint("TOP", CalendarMonthBackground, "BOTTOM", 0, 10);
    
    CleanUI_RemoveTextures(CalendarPrevMonthButton);        
    CleanUI_ChangeBackButton(CalendarPrevMonthButton);
        
    CleanUI_RemoveTextures(CalendarNextMonthButton);       
    CleanUI_ChangeNextButton(CalendarNextMonthButton);
    
    CleanUI_RemoveTextures(CalendarFilterFrame); 
    
    
    -- CalendarViewHolidayFrame
    CleanUI_RemoveTextures(CalendarViewHolidayFrame);  
    CleanUI_SetBackdrop(CalendarViewHolidayFrame); 
    
    CleanUI_CreateHeaderFrame(CalendarViewHolidayFrame, "CUI_CalendarViewHolidayFrameHeaderBorderframe", 250, CalendarViewHolidayTitleFrameText);
    CalendarViewHolidayTitleFrame:Hide();
    
    CleanUI_RemoveTextures(CalendarViewHolidayCloseButton);      
    CleanUI_ChangeCloseButton(CalendarViewHolidayFrame, nil, CalendarViewHolidayCloseButton); 
    
    CleanUI_RemoveTextures(CalendarViewHolidayScrollFrame);  
    
    -- CalendarViewRaidFrame
    CleanUI_RemoveTextures(CalendarViewRaidFrame);  
    CleanUI_SetBackdrop(CalendarViewRaidFrame); 
    
    CleanUI_CreateHeaderFrame(CalendarViewRaidFrame, "CUI_CalendarViewRaidFrameHeaderBorderframe", 250, CalendarViewRaidTitleFrameText);
    CalendarViewRaidTitleFrame:Hide();
    
    CleanUI_RemoveTextures(CalendarViewRaidCloseButton);      
    CleanUI_ChangeCloseButton(CalendarViewRaidFrame, nil, CalendarViewRaidCloseButton); 
    
    CleanUI_RemoveTextures(CalendarViewRaidScrollFrame);  
    
    -- CalendarViewEventFrame
    CleanUI_RemoveTextures(CalendarViewEventFrame);  
    CleanUI_SetBackdrop(CalendarViewEventFrame); 
    
    CleanUI_RemoveTextures(CalendarViewEventInviteListSection);  
    
    CleanUI_CreateHeaderFrame(CalendarViewEventFrame, "CUI_CalendarViewEventFrameHeaderBorderframe", 250, CalendarViewEventTitleFrameText);
    CalendarViewEventTitleFrame:Hide();
    
    CleanUI_RemoveTextures(CalendarViewEventCloseButton);      
    CleanUI_ChangeCloseButton(CalendarViewEventFrame, nil, CalendarViewEventCloseButton); 
    
    CleanUI_ChangeButton(CalendarViewEventAcceptButton);
    CleanUI_ChangeButton(CalendarViewEventTentativeButton);
    CleanUI_ChangeButton(CalendarViewEventRemoveButton);
    CleanUI_ChangeButton(CalendarViewEventDeclineButton);
    
    CalendarViewEventInviteListScrollFrameScrollBarBG:Hide();
    CalendarViewEventInviteListScrollFrameScrollBarTop:Hide();
    CalendarViewEventInviteListScrollFrameScrollBarBottom:Hide();
    CalendarViewEventInviteListScrollFrameScrollBarMiddle:Hide();
    
    -- CalendarCreateEventFrame
    CleanUI_RemoveTextures(CalendarCreateEventFrame);  
    CleanUI_SetBackdrop(CalendarCreateEventFrame); 
    
    CleanUI_RemoveTextures(CalendarCreateEventInviteListSection);  
    
    CleanUI_CreateHeaderFrame(CalendarCreateEventFrame, "CUI_CalendarCreateEventFrameHeaderBorderframe", 250, CalendarCreateEventTitleFrameText);
    CalendarCreateEventTitleFrame:Hide();
    
    CleanUI_RemoveTextures(CalendarCreateEventCloseButton);      
    CleanUI_ChangeCloseButton(CalendarCreateEventFrame, nil, CalendarCreateEventCloseButton); 
    
    CleanUI_ChangeButton(CalendarCreateEventInviteButton);
    CleanUI_ChangeButton(CalendarCreateEventMassInviteButton);
    CleanUI_ChangeButton(CalendarCreateEventRaidInviteButton);
    CleanUI_ChangeButton(CalendarCreateEventCreateButton);
    
    CleanUI_ChangeEditBox(CalendarCreateEventInviteEdit, 32, -12, 4);
    
    CalendarCreateEventInviteButton:ClearAllPoints();
    CalendarCreateEventInviteButton:SetPoint("BOTTOMRIGHT", CalendarCreateEventCreateButton, "BOTTOMRIGHT", 0, 27);
    CalendarCreateEventInviteButton:SetWidth(100);
    
    CleanUI_ChangeEditBox(CalendarCreateEventTitleEdit, 32, -12, 4);
    CalendarCreateEventTitleEdit:SetPoint("TOPLEFT", 25, -79);
    
    CleanUI_ChangeDropDown(CalendarCreateEventTypeDropDown); 
    CleanUI_ChangeDropDown(CalendarCreateEventHourDropDown); 
    CleanUI_ChangeDropDown(CalendarCreateEventMinuteDropDown); 
    
    CalendarCreateEventInviteListScrollFrameScrollBarBG:Hide();
    CalendarCreateEventInviteListScrollFrameScrollBarTop:Hide();
    CalendarCreateEventInviteListScrollFrameScrollBarBottom:Hide();
    CalendarCreateEventInviteListScrollFrameScrollBarMiddle:Hide();
    
    CleanUI_RemoveTextures(CalendarClassTotalsButton);
    
    -- modal overlay
    CleanUI_RemoveTextures(CalendarEventPickerFrame);  
    CleanUI_SetBackdrop(CalendarEventPickerFrame); 
    
    CleanUI_ChangeButton(CalendarEventPickerCloseButton);
    
    CleanUI_RemoveTextures(CalendarEventPickerTitleFrame); 
    CleanUI_CreateHeaderFrame(CalendarEventPickerFrame, "CUI_CalendarCreateEventPickerFrameHeaderBorderframe", 175, CalendarEventPickerTitleFrameText);
    
    
    -- class tabs
    local button, background;
    local iconOffset = 0.03;
    for i, class in ipairs(CLASS_SORT_ORDER) do
        button = _G["CalendarClassButton"..i];
        background = _G["CalendarClassButton"..i.."Background"];
        
        if (not background) then
            background = CreateFrame("Frame", "CalendarClassButton"..i.."Background", button);
            background:SetPoint("TOPLEFT", button, "TOPLEFT", -3, 3);
            background:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 3, -3);
            background:SetFrameStrata("BACKGROUND");
            CleanUI_SetBackdrop(background, false); 
        end
        
        CleanUI_RemoveTextures(button);
        
        button:SetFrameStrata("LOW");
        
        -- set texture
        buttonIcon = button:GetNormalTexture();
        buttonIcon:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
        
        buttonIcon:SetDrawLayer("BORDER");
        tcoords = CLASS_ICON_TCOORDS[class];
        
        buttonIcon:SetTexCoord(tcoords[1] + iconOffset, tcoords[2] - iconOffset, tcoords[3] + iconOffset, tcoords[4] - iconOffset);
    end
end
