
local SLOT_BUTTONS = {};
SLOT_BUTTONS[1] = {name="HeadSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[2] = {name="NeckSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[3] = {name="ShoulderSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[4] = {name="BackSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[5] = {name="ChestSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[6] = {name="WristSlot", gemPos="RIGHT", hasItemLevel=1};
SLOT_BUTTONS[7] = {name="ShirtSlot", gemPos="RIGHT"};
SLOT_BUTTONS[8] = {name="TabardSlot", gemPos="RIGHT"};
SLOT_BUTTONS[9] = {name="HandsSlot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[10] = {name="WaistSlot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[11] = {name="LegsSlot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[12] = {name="FeetSlot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[13] = {name="Finger0Slot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[14] = {name="Finger1Slot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[15] = {name="Trinket0Slot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[16] = {name="Trinket1Slot", gemPos="LEFT", hasItemLevel=1};
SLOT_BUTTONS[17] = {name="MainHandSlot", gemPos="TOP", hasItemLevel=1};
SLOT_BUTTONS[18] = {name="SecondaryHandSlot", gemPos="TOP", hasItemLevel=1};

CUI_ITEM_SOCKETING = ITEM_SOCKETING;
ITEM_SOCKETING = " ";

function CleanUI_InitCharacterFrame()
	local buttonname, button, itemButtonBorder, texture;
    for i, data in ipairs(SLOT_BUTTONS) do  
        buttonname = "Character"..data.name;
        button = _G[buttonname];
        
        button.data = data;
        
        itemButtonBorder = _G[buttonname.."NormalTexture"];        
        itemButtonBorder:SetTexture();
        
        texture = button:CreateTexture(buttonname.."CUI_Border", "ARTWORK");
        texture:SetAllPoints(itemButtonBorder);        
        
        button.gem1 = CreateFrame("button", buttonname.."CUI_Gem1", button, "ItemButtonTemplate");
        button.gem1:SetSize(12, 12);
        button.gem1.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        button.gem1:SetNormalTexture(nil);
        button.gem1:HookScript("OnEnter", CleanUI_ShowGemTooltip);
        button.gem1:HookScript("OnLeave", CleanUI_HideGemTooltip);
        button.gem1:Hide();
        
        button.gem2 = CreateFrame("button", buttonname.."CUI_Gem2", button, "ItemButtonTemplate");
        button.gem2:SetSize(12, 12); 
        button.gem2.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        button.gem2:SetNormalTexture(nil);
        button.gem2:HookScript("OnEnter", CleanUI_ShowGemTooltip);
        button.gem2:HookScript("OnLeave", CleanUI_HideGemTooltip);
        button.gem2:Hide();
        
        button.gem3 = CreateFrame("button", buttonname.."CUI_Gem3", button, "ItemButtonTemplate");
        button.gem3:SetSize(12, 12); 
        button.gem3.icon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        button.gem3:SetNormalTexture(nil);
        button.gem3:HookScript("OnEnter", CleanUI_ShowGemTooltip);
        button.gem3:HookScript("OnLeave", CleanUI_HideGemTooltip);
        button.gem3:Hide();
        
        if (button.data.gemPos == "RIGHT") then
        	button.gem1:SetPoint("TOPLEFT", button, "TOPRIGHT", 0, -1);
        	button.gem2:SetPoint("LEFT", button, "RIGHT", 0, 0);
        	button.gem3:SetPoint("BOTTOMLEFT", button, "BOTTOMRIGHT", 0, 1);
        end
        
        if (button.data.gemPos == "LEFT") then
        	button.gem1:SetPoint("TOPRIGHT", button, "TOPLEFT", 0, -1);
        	button.gem2:SetPoint("RIGHT", button, "LEFT", 0, 0);
        	button.gem3:SetPoint("BOTTOMRIGHT", button, "BOTTOMLEFT", 0, 1);
        end
        
        if (button.data.gemPos == "TOP") then
        	button.gem1:SetPoint("BOTTOMLEFT", button, "TOPLEFT", 1, 0);
        	button.gem2:SetPoint("BOTTOM", button, "TOP", 0, 0);
        	button.gem3:SetPoint("BOTTOMRIGHT", button, "TOPRIGHT", -1, 0);
        end
    end
    
    CleanUI_CharacterFrameHookEvents();
    CleanUI_RegisterCharacterFrameEvents();
    
    -- Frames stylen
    CleanUI_CleanCharacterFrame();
    CleanUI_CleanDressUpFrame();
    CleanUI_CleanGearManagerDialog();
end

function CleanUI_ShowGemTooltip(self)
    local id = self.gemid;
    
    if (id) then
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	    GameTooltip:SetItemByID(id);
	    GameTooltip:Show();
	end
end

function CleanUI_HideGemTooltip()
	GameTooltip:FadeOut();
end
    
function CleanUI_RegisterCharacterFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_CharacterFrameOnEvent); 
end

function CleanUI_CharacterFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_InspectUI" ) then
            CleanUI_CleanInspectFrame();
        end
        if ( addonName == "Blizzard_ItemSocketingUI" ) then
    		hooksecurefunc("ItemSocketingSocketButton_Disable", CleanUI_ItemSocketingSocketButton_Disable);
    		hooksecurefunc("ItemSocketingSocketButton_Enable", CleanUI_ItemSocketingSocketButton_Enable);
            CleanUI_CleanItemSocketingFrame();
        end
    end
end

function CleanUI_CharacterFrameHookEvents()
    hooksecurefunc("ToggleCharacter", CleanUI_ToggleCharacter);
    hooksecurefunc("ReputationFrame_Update", CleanUI_ReputationFrame_Update);
    hooksecurefunc("PaperDollItemSlotButton_Update", CleanUI_PaperDollItemSlotButton_Update);
end
    
function CleanUI_CleanCharacterFrame()
    CleanUI_RemoveTextures(CharacterFrame);  
    CleanUI_SetBackdrop(CharacterFrame);  
    
    CharacterFramePortrait:Hide();    
        
    CleanUI_ChangeCloseButton(CharacterFrame);
    
    CleanUI_CreateHeaderFrame(CharacterFrame, "CUI_CharacterFrameHeaderBorderframe", 250, CharacterFrameTitleText);
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["CharacterFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
    
    local raceIcon = PaperDollFrame:CreateTexture("PaperDollFrameRaceIcon", "ARTWORK");
    raceIcon:SetSize(110, 110);
    raceIcon:SetPoint("TOPRIGHT",CharacterFrameInset ,"TOPRIGHT" , -55, -5);
    raceIcon:SetTexture(CleanUI_GetRaceTexturePath("player"));
    
    local classIcon = PaperDollFrame:CreateTexture("PaperDollFrameClassIcon", "ARTWORK");
    classIcon:SetSize(110, 110);
    classIcon:SetPoint("TOPLEFT",CharacterFrameInset ,"TOPLEFT" , 55, -5);
    classIcon:SetTexture(CleanUI_GetClassTexturePath("player"));
end

function CleanUI_CleanGearManagerDialog()
    CleanUI_RemoveTextures(GearManagerDialogPopup);  
    CleanUI_SetBackdrop(GearManagerDialogPopup);  
    
    CleanUI_ChangeButton(GearManagerDialogPopupOkay); 
    CleanUI_ChangeButton(GearManagerDialogPopupCancel); 
    
    CleanUI_ChangeEditBox(GearManagerDialogPopupEditBox, 30, -12, -75);
    
    CleanUI_RemoveTextures(GearManagerDialogPopupScrollFrame);
    
    GearManagerDialogPopupCancel:SetPoint("BOTTOMRIGHT", -5, 8);
    
    for i=1, 15 do
    	CleanUI_RemoveTextures(_G["GearManagerDialogPopupButton"..i]); 
    	_G["GearManagerDialogPopupButton"..i.."Icon"]:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    end
end
    
function CleanUI_PaperDollItemSlotButton_Update(button)
    CleanUI_ItemSlotButton_Update(button, "player");
end
    
function CleanUI_InspectPaperDollItemSlotButton_Update(button)
    local unit = InspectFrame.unit;
    CleanUI_ItemSlotButton_Update(button, unit);
end
    
function CleanUI_CleanInspectFrame()
    -- hook events
    hooksecurefunc("InspectSwitchTabs", CleanUI_CleanInspectSubFrames);
    hooksecurefunc("InspectPaperDollFrame_UpdateButtons", CleanUI_CleanInspectSubFrames);
    hooksecurefunc("InspectPaperDollItemSlotButton_Update", CleanUI_InspectPaperDollItemSlotButton_Update);
    
    -- borders
    for i, data in ipairs(SLOT_BUTTONS) do  
        local buttonname = "Inspect"..data.name;
        local button = _G[buttonname];
        
        local itemButtonBorder = button:GetNormalTexture();        
        itemButtonBorder:SetTexture();
        
        local texture = button:CreateTexture(buttonname.."CUI_Border", OVERLAY);
        texture:SetAllPoints(itemButtonBorder);      
    end
    
    -- change layout
    CleanUI_RemoveTextures(InspectFrame);  
    CleanUI_SetBackdrop(InspectFrame);  
    
    InspectFramePortrait:Hide();    
        
    CleanUI_ChangeCloseButton(InspectFrame);
    
    CleanUI_CreateHeaderFrame(InspectFrame, "CUI_InspectFrameHeaderBorderframe", 250, InspectFrameTitleText);
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["InspectFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
    
    -- race icon
    local raceIcon = InspectPaperDollFrame:CreateTexture("InspectPaperDollFrameRaceIcon", "ARTWORK");
    raceIcon:SetSize(110, 110);
    raceIcon:SetPoint("TOPRIGHT", InspectFrameInset, "TOPRIGHT", -45, 5);
    raceIcon:SetTexture(CleanUI_GetRaceTexturePath(InspectFrame.unit)); 
    
    -- class icon    
    local classIcon = InspectPaperDollFrame:CreateTexture("InspectPaperDollFrameClassIcon", "ARTWORK");
    classIcon:SetSize(110, 110);
    classIcon:SetPoint("TOPLEFT", InspectFrameInset, "TOPLEFT", 45, 5);
    classIcon:SetTexture(CleanUI_GetClassTexturePath(InspectFrame.unit)); 
    
    -- timer
    local inspectItemLevel = InspectFrame:CreateFontString("CUI_InspectFrameItemLevel", "ARTWORK");
    inspectItemLevel:SetFontObject("GameFontNormalSmall");
    inspectItemLevel:SetWidth(200);
    inspectItemLevel:SetHeight(10);
    inspectItemLevel:ClearAllPoints(); 
    inspectItemLevel:SetPoint("TOP", InspectFrameTitleText, "BOTTOM", 0, -35);
    inspectItemLevel:SetJustifyH("CENTER");
    inspectItemLevel:SetText("");    
end
    
function CleanUI_CleanItemSocketingFrame()
    CleanUI_RemoveTextures(ItemSocketingFrame);  
    CleanUI_SetBackdrop(ItemSocketingFrame);  
    CleanUI_RemoveTextures(ItemSocketingFrameInset);  
    
    ItemSocketingFramePortrait:Hide();    
        
    CleanUI_ChangeCloseButton(ItemSocketingFrame);
    
    --local header = CleanUI_GetRegionOfTypeAtPos(ItemSocketingFrame, "FontString", 0);
    --header.Hide();
    
    CleanUI_CreateHeaderFrame(ItemSocketingFrame, "CUI_ItemSocketingFrameHeaderBorderframe", 250, nil, CUI_ITEM_SOCKETING);
    
    ItemSocketingScrollFrameTop:Hide();
    ItemSocketingScrollFrameMiddle:Hide();
    ItemSocketingScrollFrameBottom:Hide();
    ItemSocketingScrollFrameTop:SetTexture(nil);
    ItemSocketingScrollFrameMiddle:SetTexture(nil);
    ItemSocketingScrollFrameBottom:SetTexture(nil);
    
    ItemSocketingScrollFrame:SetPoint("TOPLEFT", 22, -34);
    ItemSocketingScrollFrame:SetHeight(300);
    
    CleanUI_ChangeButton(ItemSocketingSocketButton);   
end

function CleanUI_ItemSocketingSocketButton_Disable()
    CleanUI_ChangeButton(ItemSocketingSocketButton); 
end

function CleanUI_ItemSocketingSocketButton_Enable()
    CleanUI_ChangeButton(ItemSocketingSocketButton); 
end

function CleanUI_GetRaceTexturePath(unit)
    if (not unit) then
        return nil;
    end

    local race, raceFileName = UnitRace(unit);

    -- HACK
    if ( not raceFileName ) then
        raceFileName = "Orc";
    end

    return "interface\\AddOns\\CleanUI\\skins\\races\\"..strlower(raceFileName);
end

function CleanUI_GetClassTexturePath(unit)
    if (not unit) then
        return nil;
    end

    local localizedClass, englishClass, classIndex = UnitClass(unit);

    -- HACK
    if ( not englishClass ) then
        englishClass = "Warrior";
    end

    return "interface\\AddOns\\CleanUI\\skins\\classes\\"..strlower(englishClass);
end
    
function CleanUI_ToggleCharacter()   
    CleanUI_CleanPaperDollFrame();
    CleanUI_CleanPetPaperDollFrame();
    CleanUI_CleanReputationFrame();
    CleanUI_CleanTokenFrame();
end
    
function CleanUI_CleanPaperDollFrame()
    CleanUI_RemoveTextures(CharacterModelFrame);
      
    for i, child in ipairs({PaperDollItemsFrame:GetChildren()}) do
        if (child and child:IsObjectType("Button")) then
            _G[child:GetName().."Frame"]:Hide();
        end
    end
    
    CleanUI_RemoveTextures(CharacterStatsPane);   
     
    PaperDollTitlesPaneScrollBarBG:Hide();
    PaperDollTitlesPaneScrollBarTop:Hide();
    PaperDollTitlesPaneScrollBarBottom:Hide();
    PaperDollTitlesPaneScrollBarMiddle:Hide();
    
    PaperDollEquipmentManagerPaneScrollBarBG:Hide();
    PaperDollEquipmentManagerPaneScrollBarTop:Hide();
    PaperDollEquipmentManagerPaneScrollBarBottom:Hide();
    PaperDollEquipmentManagerPaneScrollBarMiddle:Hide();
    
    CleanUI_RemoveTextures(CharacterFrameInset);
    CleanUI_RemoveTextures(CharacterFrameInsetRight);
    CleanUI_RemoveTextures(PaperDollSidebarTabs);
    
    CharacterFrameInsetRight:SetPoint("BOTTOMRIGHT", CharacterFrame, "BOTTOMRIGHT", -4, 8);
    
    PaperDollSidebarTab1.TabBg:Hide();
    PaperDollSidebarTab2.TabBg:Hide();
    PaperDollSidebarTab3.TabBg:Hide();
    
    CleanUI_CreatePaperDollSidebarTabBorder(PaperDollSidebarTab1);
    CleanUI_CreatePaperDollSidebarTabBorder(PaperDollSidebarTab2);
    CleanUI_CreatePaperDollSidebarTabBorder(PaperDollSidebarTab3);
    
    CleanUI_ChangeButton(PaperDollEquipmentManagerPaneEquipSet);
    CleanUI_ChangeButton(PaperDollEquipmentManagerPaneSaveSet);
    
    CleanUI_RemoveTextures(CharacterMainHandSlot);
    CleanUI_RemoveTextures(CharacterSecondaryHandSlot);

    PaperDollItemSlotButton_Update(CharacterMainHandSlot);
    PaperDollItemSlotButton_Update(CharacterSecondaryHandSlot);
end
    
function CleanUI_CleanInspectSubFrames()
    CleanUI_CleanInspectPaperDollFrame();
end
    
function CleanUI_CleanInspectPaperDollFrame()
    CleanUI_RemoveTextures(InspectModelFrame);
      
    for i, child in ipairs({InspectPaperDollItemsFrame:GetChildren()}) do
        if (child and child:IsObjectType("Button")) then
            _G[child:GetName().."Frame"]:Hide();
        end
    end
    
    CleanUI_RemoveTextures(InspectFrameInset);
    
    CleanUI_RemoveTextures(InspectMainHandSlot);
    CleanUI_RemoveTextures(InspectSecondaryHandSlot);
    
    local unit = InspectFrame.unit;
    
    InspectPaperDollItemSlotButton_Update(InspectMainHandSlot);
    InspectPaperDollItemSlotButton_Update(InspectSecondaryHandSlot);
    
    local raceIcon = _G["InspectPaperDollFrameRaceIcon"];
    raceIcon:SetTexture(CleanUI_GetRaceTexturePath(unit));
    
    local classIcon = _G["InspectPaperDollFrameClassIcon"];
    classIcon:SetTexture(CleanUI_GetClassTexturePath(unit));
    
    local level = CleanUI_GetUnitItemLevel(unit);
    CUI_InspectFrameItemLevel:SetText(level); 
end
    
function CleanUI_CreatePaperDollSidebarTabBorder(tabButton)    
    local borderframe = _G[tabButton:GetName().."Borderframe"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", tabButton:GetName().."Borderframe", tabButton);
    end
    
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -2, 1);
    borderframe:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 4, -5);
    CleanUI_SetBackdrop(borderframe, false);
end
    
function CleanUI_CleanPetPaperDollFrame()
    CleanUI_RemoveTextures(PetPaperDollFrame);
    CleanUI_RemoveTextures(PetModelFrame);
    CleanUI_RemoveTextures(PetModelFrameShadowOverlay);
end
    
function CleanUI_CleanReputationFrame()    
    CleanUI_RemoveTextures(ReputationListScrollFrame);
    
    -- popup
    CleanUI_RemoveTextures(ReputationDetailFrame);    
    CleanUI_SetBackdrop(ReputationDetailFrame);
    
    CleanUI_ChangeCloseButton(ReputationDetailFrame, "ReputationDetailCloseButton");
end
    
function CleanUI_CleanDressUpFrame()

    DressUpFrame:SetSize(360, 424);

    CleanUI_RemoveTextures(DressUpFrame); 
    CleanUI_SetBackdrop(DressUpFrame); 
    
    DressUpFramePortrait:Hide();   
        
    CleanUI_ChangeCloseButton(DressUpFrame);
    
    CleanUI_CreateHeaderFrame(DressUpFrame, "CUIDressUpFrameHeaderBorderframe", 250, DressUpFrameTitleText);
    
    DressUpModel:ClearAllPoints();
    DressUpModel:SetPoint("CENTER", DressUpFrame, "CENTER", 0, 0);
    
    CleanUI_ChangeButton(DressUpFrameResetButton);
    CleanUI_ChangeButtonRed(DressUpFrameCancelButton);
    
    DressUpFrameCancelButton:ClearAllPoints();
    DressUpFrameCancelButton:SetPoint("BOTTOMRIGHT", DressUpFrame, "BOTTOMRIGHT", -6, 4);
    
    DressUpBackgroundTopLeft:Hide();
    DressUpBackgroundTopRight:Hide();
    DressUpBackgroundBotLeft:Hide();
    DressUpBackgroundBotRight:Hide();
end

function CleanUI_ReputationFrame_Update()   
    local statusbar, atWar;
    
    for i=1, 15 do
        _G["ReputationBar"..i.."Background"]:Hide();
        
        statusbar = _G["ReputationBar"..i.."ReputationBar"];
        CleanUI_RemoveTextures(statusbar);
        statusbar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);  
        statusbar:SetWidth(80);  
        
        atWar = _G["ReputationBar"..i.."ReputationBarAtWarHighlight1"];
        atWar:ClearAllPoints();
        atWar:SetPoint("LEFT", statusbar, "RIGHT", 2, 0);
        atWar:SetSize(16, 16);
        atWar:SetTexture("Interface\\Groupframe\\UI-Group-MainAssistIcon");
        atWar:SetBlendMode("BLEND");
        atWar:SetAlpha(1.0);
        atWar:SetTexCoord(0, 1, 0, 1);
    end
end
    
function CleanUI_CleanTokenFrame()    
    -- popup
    CleanUI_RemoveTextures(TokenFramePopup);    
    CleanUI_SetBackdrop(TokenFramePopup);
        
    CleanUI_ChangeCloseButton(TokenFramePopup);
end
    
function CleanUI_ItemSlotButton_Update(button, unit)
	-- item level
    local link = GetInventoryItemLink(unit, button:GetID());
    local itemName, itemLink, itemRarity, itemLevel;
    
    if (link) then
        itemName, itemLink, itemRarity, itemLevel = GetItemInfo(link);
        itemLevel = CleanUI_GetActualItemLevel(link);
    end

    local itemLevelLabel = _G[button:GetName().."ItemLevel"];
    if (not itemLevelLabel) then
	    itemLevelLabel = button:CreateFontString(button:GetName().."ItemLevel", "ARTWORK");
	    itemLevelLabel:SetFontObject("GameFontNormalSmall");
	    itemLevelLabel:SetSize(button:GetWidth(), 10);
	    itemLevelLabel:ClearAllPoints(); 
	    itemLevelLabel:SetPoint("TOPRIGHT", -2, -2);
	    itemLevelLabel:SetJustifyH("RIGHT");
	    itemLevelLabel:SetText("");
    end
    
    if (itemLevel and string.find(button:GetName(), "CharacterBag") ~= 1) then
    	itemLevelLabel:SetText(ORANGE_FONT_COLOR_CODE..itemLevel..FONT_COLOR_CODE_CLOSE);
    	itemLevelLabel:Show();
    else
    	itemLevelLabel:Hide();
    end
    
    -- quality border
    local texture = _G[button:GetName().."CUI_Border"];
    
    if (texture) then
        texture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
        texture:Show();
        
        local r, g, b = 0.5, 0.5, 0.5;
        
        if (itemRarity) then
            r, g, b = GetItemQualityColor(itemRarity);
        end
        
        texture:SetVertexColor(r, g, b);
    end
    
    -- gems
    local gem1, gem2, gem3 = GetInventoryItemGems(button:GetID());
    
    CleanUI_UpdateGemInfo(gem1, button.gem1);
    CleanUI_UpdateGemInfo(gem2, button.gem2);
    CleanUI_UpdateGemInfo(gem3, button.gem3);
end

function CleanUI_UpdateGemInfo(id, button)
	if (not button) then
		return;
	end

    if (id) then
	    button:Show();
	    local itemName, itemLink, _, _, _, _, _, _, _, itemTexture = GetItemInfo(id);
	    button.icon:SetTexture(itemTexture);
	    button.gemid = id;
    else
    	button:Hide();
	    button.gemid = nil;
    end
end

function CleanUI_GetActualItemLevel(link)
    local levelAdjust={ -- 11th item:id field and level adjustment
		["0"]=0,["1"]=8,["373"]=4,["374"]=8,["375"]=4,["376"]=4,["377"]=4,["379"]=4,["380"]=4,
		["445"]=0,["446"]=4,["447"]=8,
		["451"]=0,["452"]=8,
		["453"]=0,["454"]=4,["455"]=8,
		["456"]=0,["457"]=8,
		["458"]=0,["459"]=4,["460"]=8,["461"]=12,["462"]=16,
		["465"]=0,["466"]=4,["467"]=8,
		["468"]=0,["469"]=4,["470"]=8,["471"]=12,["472"]=16,
		["476"]=0,["477"]=4,["478"]=8,
		["479"]=0,["480"]=4,["481"]=8,
		["482"]=0,["483"]=4,["484"]=8,
		["485"]=0,["486"]=4,["487"]=8,
		["488"]=0,["489"]=4,["490"]=8,
		["491"]=0,["492"]=4,["493"]=8,
		["504"]=12,["505"]=16};
		
    local baseLevel = select(4,GetItemInfo(link));
    local upgrade = link:match(":(%d+)\124h%[");
    
    if (baseLevel and upgrade and levelAdjust[upgrade]) then
        return baseLevel + levelAdjust[upgrade];
    else
        return baseLevel;
    end
end

function CleanUI_GetUnitItemLevel(unit)
    local min = 0;
    local total = 0;
    local count = 0;
    
    local slotInfo;

    for i, data in ipairs(SLOT_BUTTONS) do  
		if (data.hasItemLevel) then	
	    	slotInfo = GetInventorySlotInfo(data.name);
	        local itemLevel = CleanUI_GetSlotItemLevel(unit, slotInfo);
	        
	        if (itemLevel >= 0) then
	            if (min == 0 or itemLevel < min) then
	                min = itemLevel;
	            end
	            
	            total = total + itemLevel;
	            count = count + 1;
	        end
	    end
    end
    
    if (count == 0) then
        total = 0;
    else    
        total = total / count;
    end
    
    return "|cffffffff"..string.format(ITEM_LEVEL_AND_MIN, total, min).."|r";
end

function CleanUI_GetSlotItemLevel(unit, slot)
    if (not GetInventoryItemLink(unit, slot)) then
        return -1;
    end
    
    local itemLink = GetInventoryItemLink(unit, slot);
    local itemLevel = CleanUI_GetActualItemLevel(itemLink);
    
    if (not itemLevel) then
        return -1;
    end
    
    return itemLevel;
end