local hoveredButton = nil;

local RAID_FLARE_ICONS = {};
RAID_FLARE_ICONS[1] = "Interface\\AddOns\\CleanUI\\skins\\marker\\blue";
RAID_FLARE_ICONS[2] = "Interface\\AddOns\\CleanUI\\skins\\marker\\green";
RAID_FLARE_ICONS[3] = "Interface\\AddOns\\CleanUI\\skins\\marker\\purple";
RAID_FLARE_ICONS[4] = "Interface\\AddOns\\CleanUI\\skins\\marker\\red";
RAID_FLARE_ICONS[5] = "Interface\\AddOns\\CleanUI\\skins\\marker\\yellow";

function CleanUI_InitChat()
    if (CleanUIData.DISABLE_ALTARROWKEYMODE == 1) then
        CleanUI_DisableAltArrowKeyMode();
    end
    
    if (CleanUIData.CHANGE_CHAT ~= 1) then
        return;
    end
    
    CleanUI_ChangeChat();
    CleanUI_ChatHookEvents();
    CleanUI_RegisterChatEvents();
end
    
function CleanUI_RegisterChatEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("MODIFIER_STATE_CHANGED");
    
    frame:HookScript("OnEvent", CleanUI_ChatOnEvent); 
end

function CleanUI_ChatOnEvent(self, event, ...)
    if (event == "MODIFIER_STATE_CHANGED") then
        CleanUI_UpdateButton(hoveredButton);
    end
end

function CleanUI_ChatHookEvents()
    if (CleanUIData.CHANGE_CHAT == 1) then
        hooksecurefunc("ChatEdit_DeactivateChat", CleanUI_ChatEdit_RestoreEditBoxAlpha);
        hooksecurefunc("ChatEdit_SetLastActiveWindow", CleanUI_ChatEdit_RestoreEditBoxAlpha);
        hooksecurefunc("ChatEdit_UpdateHeader", CleanUI_ChatEdit_UpdateHeader);
    end
end

function CleanUI_UpdateChatFontSize()    
    if (CleanUIData.CHANGE_CHAT == 1) then
        for i = 1, 10 do 
            FCF_SetChatWindowFontSize(nil, _G["ChatFrame"..i], CleanUIData.CHAT_FONT_SIZE);     
        end  
    end
end

function CleanUI_DisableAltArrowKeyMode()
    local i = 1;
    local editbox = "ChatFrame"..i.."EditBox";
    
    while ( _G[editbox] ~= nil ) do
        _G[editbox]:SetAltArrowKeyMode(nil);
        i = i + 1;
        editbox = "ChatFrame"..i.."EditBox";
    end
end

function CleanUI_ChatEdit_RestoreEditBoxAlpha(editBox)
	editBox:SetAlpha(1.0);
end

function CleanUI_ChatEdit_UpdateHeader(editBox)
	local type = editBox:GetAttribute("chatType");
	if ( not type ) then
		return;
	end
	
	local info = ChatTypeInfo[type];
	
	local header = _G[editBox:GetName().."Header"];
	
	if (header) then
        header:ClearAllPoints();
        header:SetPoint("LEFT", editBox, "LEFT", 10, 0);
        
        editBox:SetTextInsets(10 + header:GetWidth(), 6, 0, 0);
        editBox:SetBackdropBorderColor(info.r, info.g, info.b, 1.0);
    end
end

function CleanUI_ChangeChat()
    CleanUI_ChangeChatBackgroundTextures();
    CleanUI_ChangeChatBorders();
    CleanUI_ChangeChatButtonFrame();
    CleanUI_ChangeChatEditBox();
    
    CleanUI_AddChatExtensions();
end

function CleanUI_ChangeChatBackgroundTextures()
	local tab;
    for i = 1, 100 do   
    	tab = _G["ChatFrame"..i.."Tab"];
    	
    	if (not tab) then
    		break;
    	end
    	
    	CleanUI_ChangeChatTab(tab);
    	      
        -- Textures
        _G["ChatFrame"..i.."Background"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."LeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."RightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."BottomTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."TopTexture"]:SetTexture(nil);
        
        _G["ChatFrame"..i.."ButtonFrameBackground"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameLeftTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameRightTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameBottomTexture"]:SetTexture(nil);
        _G["ChatFrame"..i.."ButtonFrameTopTexture"]:SetTexture(nil);
        
        _G["ChatFrame"..i.."EditBoxLeft"]:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-left");
        _G["ChatFrame"..i.."EditBoxMid"]:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-center");
        _G["ChatFrame"..i.."EditBoxRight"]:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-right");
        
        --_G["ChatFrame"..i.."EditBoxFocusLeft"]:SetTexture(nil);
        --_G["ChatFrame"..i.."EditBoxFocusMid"]:SetTexture(nil);
        --_G["ChatFrame"..i.."EditBoxFocusRight"]:SetTexture(nil);
    end
    
    CombatLogQuickButtonFrame_CustomTexture:Hide();
    
    CombatLogQuickButtonFrame_CustomAdditionalFilterButton:SetPoint("TOPRIGHT", CombatLogQuickButtonFrame_Custom, "TOPRIGHT", 6, 2); 
            
    local borderframe = _G["CUI_CombatLogButtonFrameBorder"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUI_CombatLogButtonFrameBorder", CombatLogQuickButtonFrame_Custom);
    end
    
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", CombatLogQuickButtonFrame_Custom, "TOPLEFT", -6, 2); 
    borderframe:SetPoint("BOTTOMRIGHT", CombatLogQuickButtonFrame_Custom, "BOTTOMRIGHT", 4, 0); 
    
    CleanUI_SetBackdrop(borderframe);
end

function CleanUI_ChangeChatTab(tabButton)
    CleanUI_RemoveTextures(tabButton);
    
    tabButton:SetNormalTexture(CUI_TAB_BACKGROUND);
    tabButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    tabButton:GetNormalTexture():ClearAllPoints();
    tabButton:GetNormalTexture():SetPoint("TOPLEFT", tabButton, "BOTTOMLEFT", 0, 25);
    tabButton:GetNormalTexture():SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 0, -3);
    
    tabButton:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-highlight-blue");
    tabButton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    tabButton:GetHighlightTexture():ClearAllPoints();
    tabButton:GetHighlightTexture():SetPoint("TOPLEFT", tabButton, "BOTTOMLEFT", 0, 25);
    tabButton:GetHighlightTexture():SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 0, -3);
    tabButton:GetHighlightTexture():SetBlendMode("MOD");
end

function CleanUI_ChangeChatBorders()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local buttonframe = _G["ChatFrame"..i.."ButtonFrame"];
        local editbox = _G["ChatFrame"..i.."EditBox"];
        
        chatframe:SetFrameStrata("LOW");
        editbox:SetFrameStrata("LOW");
        
        -- Border                
        local borderframe = _G["CUI_Chat"..i];
        if (not borderframe) then
            borderframe = CreateFrame("Frame", "CUI_Chat"..i, chatframe);
        end
        
        borderframe:SetFrameStrata("BACKGROUND");
        
        CleanUI_SetBackdrop(borderframe);
        
        borderframe:ClearAllPoints();
        borderframe:SetPoint("TOPLEFT", chatframe, "TOPLEFT", -6, 2);
        borderframe:SetPoint("BOTTOMRIGHT", chatframe, "BOTTOMRIGHT", 4, -31);
    end
end

function CleanUI_ChangeChatEditBox()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local editbox = _G["ChatFrame"..i.."EditBox"];
    
        editbox:SetHeight(30);
        
        editbox:ClearAllPoints();
        editbox:SetPoint("TOPLEFT", chatframe, "BOTTOMLEFT", -6, 0);
        editbox:SetPoint("TOPRIGHT", chatframe, "BOTTOMRIGHT", 4, 0);
    end
end

function CleanUI_ChangeChatButtonFrame()
    for i = 1, 10 do
        local chatframe = _G["ChatFrame"..i];
        local buttonframe = _G["ChatFrame"..i.."ButtonFrame"];
        
        local buttonFrameBorder = _G["CUI_ChatFrame"..i.."ButtonFrameBorder"];
        if (not buttonFrameBorder) then
            buttonFrameBorder = CreateFrame("Frame", "CUI_ChatFrame"..i.."ButtonFrameBorder", buttonframe);
        end
        buttonFrameBorder:SetPoint("TOPLEFT", -4, 2);        
        buttonFrameBorder:SetPoint("BOTTOMRIGHT", 2, -31);
        buttonFrameBorder:SetFrameStrata("BACKGROUND");

        -- ButtonFrame
        CleanUI_SetBackdrop(buttonFrameBorder);
        
        -- Buttons        
        
        local bottomButton = _G["ChatFrame"..i.."ButtonFrameBottomButton"];
        local downButton = _G["ChatFrame"..i.."ButtonFrameDownButton"];
        local upButton = _G["ChatFrame"..i.."ButtonFrameUpButton"];
        local minimizeButton = _G["ChatFrame"..i.."ButtonFrameMinimizeButton"];
        
        bottomButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom");
        bottomButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom-clicked");
        bottomButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\bottom-disabled");
        
        downButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down");
        downButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down-clicked");
        downButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\down-disabled");
        
        upButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up");
        upButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up-clicked");
        upButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\up-disabled");
        
        minimizeButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize");
        minimizeButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize-clicked");
        minimizeButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\minimize-disabled");
    end
    
    FriendsMicroButton:ClearAllPoints();
    FriendsMicroButton:SetPoint("BOTTOM", ChatFrameMenuButton, "TOP", 0, -150); 
    
    FriendsMicroButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet");
    FriendsMicroButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet-clicked");
    FriendsMicroButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\battlenet-disabled");
    
    ChatFrameMenuButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat");
    ChatFrameMenuButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat-clicked");
    ChatFrameMenuButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\chat-disabled");
end

function CleanUI_AddChatExtensions()
    if (CleanUIData.SHOW_CHAT_ACTIONBAR == 1) then
        CleanUI_AddChatActionBar();
    end
    
    if (CleanUIData.SHOW_CHAT_EMOTICONS == 1) then
        CleanUI_AddChatMenuEmoticons();
    end
end

function CleanUI_AddChatActionBar()        
    local chatActionBarBorder = _G["CUI_ChatActionBarBorder"];
    if (not chatActionBarBorder) then
        chatActionBarBorder = CreateFrame("Frame", "CUI_ChatActionBarBorder", ChatFrame1);
    end        
    chatActionBarBorder:ClearAllPoints();
    chatActionBarBorder:SetPoint("TOPLEFT", ChatFrame1EditBox, "BOTTOMLEFT", 0, 4); 
    chatActionBarBorder:SetPoint("TOPRIGHT", ChatFrame1EditBox, "BOTTOMRIGHT", 0, 4);  
    chatActionBarBorder:SetHeight(26);

    CleanUI_SetBackdrop(chatActionBarBorder);  
    
    -- raid icons
    local raidicons = CreateFrame("Frame", "CUI_ChatRaidIcons", chatActionBarBorder);
    raidicons:SetSize(160, 23);
    CleanUI_ResetRaidIconPosition();   

    for i=1, 8, 1 do    
    
        local posx = -6-(i-1) * 16;
    
        local btn = CreateFrame("Button", "CUI_CHAT_RAIDICON_"..i, raidicons, "UIPanelButtonTemplate");
        
        btn:SetNormalTexture(CUI_RAIDICON_FILE);
        btn:SetPushedTexture(CUI_RAIDICON_FILE);
        btn:GetNormalTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        btn:GetPushedTexture():SetTexCoord(CUI_RAIDICONS[i].left, CUI_RAIDICONS[i].right, CUI_RAIDICONS[i].top, CUI_RAIDICONS[i].bottom);
        CleanUI_RemoveUiButtonTexture(btn);
        
        btn:RegisterForClicks("AnyUp");

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidicons, "RIGHT", posx, 0);
        
        btn:SetAttribute("index", i);
        
        btn:HookScript("OnClick", CleanUI_RaidIconClicked);
    end 
    
    -- rc button
    local rcbutton = CreateFrame("Button", "CUI_CHAT_RCBUTTON", raidicons, "UIPanelButtonTemplate");
    
    rcbutton:SetNormalTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetPushedTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    rcbutton:SetHighlightTexture("interface\\RaidFrame\\ReadyCheck-Ready");
    CleanUI_RemoveUiButtonTexture(rcbutton);
    
    rcbutton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    rcbutton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    
    rcbutton:SetWidth(18);
    rcbutton:SetHeight(18);
    rcbutton:ClearAllPoints();
    rcbutton:SetPoint("LEFT", raidicons, "LEFT", 3, 0);
    rcbutton:HookScript("OnClick", function() DoReadyCheck(); end);
    
    -- raid flares
    local raidflares = CreateFrame("Frame", "CUI_ChatRaidFlares", chatActionBarBorder);
    raidflares:SetSize(90, 23);
    CleanUI_ResetRaidFlarePosition();

    for i=1, 5, 1 do  
        local posx = -6-(i-1) * 16;
    
        local btn = CreateFrame("Button", "CUI_CHAT_RAIDFLARE_"..i, raidflares, "SecureActionButtonTemplate");
        
        btn:SetNormalTexture(RAID_FLARE_ICONS[i]);
        btn:SetPushedTexture(RAID_FLARE_ICONS[i]);
        
        btn:RegisterForClicks("AnyUp");
        btn:SetAttribute("type1", "macro");
        btn:SetAttribute("macrotext1", "/worldmarker "..i);
        btn:SetAttribute("type2", "macro");
        btn:SetAttribute("macrotext2", "/clearworldmarker "..i);

        btn:SetWidth(13);
        btn:SetHeight(13);
        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidflares, "RIGHT", posx, 0);
        
        btn:SetAttribute("index", i);
    end
    
    -- dice button
    local dicebutton = CreateFrame("Button", "CUI_CHAT_DICEBUTTON", chatActionBarBorder, "UIPanelButtonTemplate");
    
    dicebutton:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up");
    dicebutton:SetPushedTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Down");
    dicebutton:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Highlight");
    CleanUI_RemoveUiButtonTexture(dicebutton);
    
    dicebutton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
    dicebutton:GetPushedTexture():SetTexCoord(0, 1, 0, 1);
    dicebutton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    
    dicebutton:SetWidth(18);
    dicebutton:SetHeight(18);
    dicebutton:ClearAllPoints();
    dicebutton:SetPoint("LEFT", chatActionBarBorder, "LEFT", 7, -1);
    dicebutton:HookScript("OnClick", function() RandomRoll(1, 100); end);
    
    -- GZ button
    local gzbutton = CreateFrame("Button", "CUI_CHAT_GZBUTTON", chatActionBarBorder, "UIPanelButtonTemplate");
    gzbutton:SetWidth(30);
    gzbutton:SetHeight(18);
    gzbutton:ClearAllPoints();
    gzbutton:SetPoint("LEFT", chatActionBarBorder, "LEFT", 30, 0);
    gzbutton:SetAttribute("TEXT", "GZ");
    CleanUI_RemoveUiButtonTexture(gzbutton);
    CleanUI_UpdateButton(gzbutton);
    
    gzbutton:SetNormalTexture(nil);
    gzbutton:SetPushedTexture(nil);
    gzbutton:SetDisabledTexture(nil);
    gzbutton:SetHighlightTexture(nil);
    
    gzbutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_GZBUTTON"], CUI_CHAT_GZ); end);
    gzbutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_GZBUTTON"]); end);
    gzbutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_GZBUTTON"]); end);
    
    -- THX button
    local thxbutton = CreateFrame("Button", "CUI_CHAT_THXBUTTON", chatActionBarBorder, "UIPanelButtonTemplate");
    thxbutton:SetWidth(30);
    thxbutton:SetHeight(18);
    thxbutton:ClearAllPoints();
    thxbutton:SetPoint("LEFT", gzbutton, "RIGHT", 2, 0);
    thxbutton:SetAttribute("TEXT", "THX");
    CleanUI_RemoveUiButtonTexture(thxbutton);
    CleanUI_UpdateButton(thxbutton);
    
    thxbutton:SetNormalTexture(nil);
    thxbutton:SetPushedTexture(nil);
    thxbutton:SetDisabledTexture(nil);
    thxbutton:SetHighlightTexture(nil);
    
    thxbutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_THXBUTTON"], CUI_CHAT_THX); end);
    thxbutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_THXBUTTON"]); end);
    thxbutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_THXBUTTON"]); end);
    
    -- HI button
    local hibutton = CreateFrame("Button", "CUI_CHAT_HIBUTTON", chatActionBarBorder, "UIPanelButtonTemplate");
    hibutton:SetWidth(30);
    hibutton:SetHeight(18);
    hibutton:ClearAllPoints();
    hibutton:SetPoint("LEFT", thxbutton, "RIGHT", 2, 0);
    hibutton:SetAttribute("TEXT", "HI");
    CleanUI_RemoveUiButtonTexture(hibutton);
    CleanUI_UpdateButton(hibutton);
    
    hibutton:SetNormalTexture(nil);
    hibutton:SetPushedTexture(nil);
    hibutton:SetDisabledTexture(nil);
    hibutton:SetHighlightTexture(nil);
    
    hibutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_HIBUTTON"], CUI_CHAT_HI); end);
    hibutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_HIBUTTON"]); end);
    hibutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_HIBUTTON"]); end);
    
    -- BYE button
    local byebutton = CreateFrame("Button", "CUI_CHAT_BYEBUTTON", chatActionBarBorder, "UIPanelButtonTemplate");
    byebutton:SetWidth(30);
    byebutton:SetHeight(18);
    byebutton:ClearAllPoints();
    byebutton:SetPoint("LEFT", hibutton, "RIGHT", 2, 0);
    byebutton:SetAttribute("TEXT", "BYE");
    CleanUI_RemoveUiButtonTexture(byebutton);
    CleanUI_UpdateButton(byebutton);
    
    byebutton:SetNormalTexture(nil);
    byebutton:SetPushedTexture(nil);
    byebutton:SetDisabledTexture(nil);
    byebutton:SetHighlightTexture(nil);
    
    byebutton:HookScript("OnClick", function() CleanUI_Great_OnClick(_G["CUI_CHAT_BYEBUTTON"], CUI_CHAT_BYE); end);
    byebutton:HookScript("OnEnter", function() CleanUI_Great_OnEnter(_G["CUI_CHAT_BYEBUTTON"]); end);
    byebutton:HookScript("OnLeave", function() CleanUI_Great_OnLeave(_G["CUI_CHAT_BYEBUTTON"]); end);
       
    -- chat menu
    local chatMenuBorder = _G["CUI_ChatMenuBorder"];
    if (not chatMenuBorder) then
        chatMenuBorder = CreateFrame("Frame", "CUI_ChatMenuBorder", ChatFrame1);
    end        
    chatMenuBorder:ClearAllPoints();
    chatMenuBorder:SetPoint("TOPLEFT", chatActionBarBorder, "TOPLEFT", -31, 0); 
    chatMenuBorder:SetPoint("BOTTOMRIGHT", chatActionBarBorder, "BOTTOMLEFT", 4, 0);  
    chatMenuBorder:SetHeight(23);

    CleanUI_SetBackdrop(chatMenuBorder);  
    local chatMenuButton = CreateFrame("Button", "CUI_CHAT_MENUBUTTON", chatMenuBorder, "UIPanelButtonTemplate"); 
    chatMenuButton:SetWidth(14);
    chatMenuButton:SetHeight(14);
    chatMenuButton:ClearAllPoints();
    chatMenuButton:SetPoint("CENTER", 0, 0);
    chatMenuButton:SetNormalTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up");
    chatMenuButton:SetPushedTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up");
    chatMenuButton:SetHighlightTexture(nil);
    chatMenuButton:HookScript("OnClick", CleanUI_OpenChatMenu);
    
    chatMenuButton.menu = CreateFrame("Frame", "CUI_ChatMenu", chatMenuButton, "UIMenuTemplate"); 
end

function CleanUI_OpenChatMenu(self, button, down)
    CleanUI_InitChatMenu(self.menu);
    self.menu:ClearAllPoints();
    self.menu:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 0, 0);
    self.menu:Show();
end

function CleanUI_InitChatMenu(self)	
	local textShowCloak = GREEN_FONT_COLOR_CODE..SHOW_CLOAK..FONT_COLOR_CODE_CLOSE;	
	if (not ShowingCloak()) then
		textShowCloak = RED_FONT_COLOR_CODE..SHOW_CLOAK..FONT_COLOR_CODE_CLOSE;
	end

	local textShowHelm = GREEN_FONT_COLOR_CODE..SHOW_HELM..FONT_COLOR_CODE_CLOSE;	
	if (not ShowingHelm()) then
		textShowHelm = RED_FONT_COLOR_CODE..SHOW_HELM..FONT_COLOR_CODE_CLOSE;
	end

	local textScriptErrors = GREEN_FONT_COLOR_CODE..SHOW_LUA_ERRORS..FONT_COLOR_CODE_CLOSE;	
	if (GetCVar("scriptErrors") ~= "1") then
		textScriptErrors = RED_FONT_COLOR_CODE..SHOW_LUA_ERRORS..FONT_COLOR_CODE_CLOSE;
	end
	
	local textFramerate = GREEN_FONT_COLOR_CODE..BINDING_NAME_TOGGLEFPS..FONT_COLOR_CODE_CLOSE;	
	if ( not FramerateText:IsShown() ) then
		textFramerate = RED_FONT_COLOR_CODE..BINDING_NAME_TOGGLEFPS..FONT_COLOR_CODE_CLOSE;
	end

    UIMenu_Initialize(self);
    UIMenu_AddButton(self, textShowCloak, nil, function(frame) if (ShowingCloak()) then ShowCloak(nil); else ShowCloak(1); end end);    
    UIMenu_AddButton(self, textShowHelm, nil, function(frame) if (ShowingHelm()) then ShowHelm(nil); else ShowHelm(1); end end);    
    UIMenu_AddButton(self, textScriptErrors, nil, function(frame) CleanUI_ToggleShowLuaErrors(); end);        
    UIMenu_AddButton(self, textFramerate, nil, function(frame) ToggleFramerate(); end);          
    UIMenu_AddButton(self, "DataStore", nil, function(frame) CleanUI_ShowDataStoreUI(); end);   
    UIMenu_AddButton(self, " ");  
    
    -- other addons   
    local countAddons = 0;
    if (CleanUI_RecountInstalled()) then	
		local text = GREEN_FONT_COLOR_CODE.."Recount"..FONT_COLOR_CODE_CLOSE;	
		if ( not Recount.MainWindow:IsShown() ) then
			text = RED_FONT_COLOR_CODE.."Recount"..FONT_COLOR_CODE_CLOSE;
		end
	
        UIMenu_AddButton(self, text, nil, function(frame) CleanUI_ToggleRecount(); end);
        countAddons = countAddons + 1;
    end
    if (CleanUI_SkadaInstalled()) then	
		local text = GREEN_FONT_COLOR_CODE.."Skada"..FONT_COLOR_CODE_CLOSE;	
		if ( not SkadaBarWindowSkada:IsShown() ) then
			text = RED_FONT_COLOR_CODE.."Skada"..FONT_COLOR_CODE_CLOSE;
		end
	
        UIMenu_AddButton(self, text, nil, function(frame) CleanUI_ToggleSkada(); end);
        countAddons = countAddons + 1;
    end
         
    if (countAddons > 0) then
        UIMenu_AddButton(self, " ");  
    end
    
    UIMenu_AddButton(self, CANCEL, nil, function(frame) self:Hide(); end);      
    UIMenu_AutoSize(self);
end

function CleanUI_ToggleShowLuaErrors()
    if ( not GetCVarBool("scriptErrors") ) then
        SetCVar( "scriptErrors", 1 );
    else
        SetCVar( "scriptErrors", nil );
    end
end

function CleanUI_RaidIconSnap()
    local frame = _G["CUI_ChatRaidIcons"];
    
    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();
    
    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();
    
    x = x / scale;  
    y = y / scale;
    
    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    CleanUI_SetBackdrop(frame);  
end

function CleanUI_ResetRaidIconPosition()
    local frame = _G["CUI_ChatRaidIcons"];
    local borderframe = _G["CUI_ChatActionBarBorder"];
    frame:ClearAllPoints();
    frame:SetPoint("TOPRIGHT", borderframe, "TOPRIGHT", 0, -2);   
    
    frame:SetBackdrop(nil);  
end

function CleanUI_RaidIconClicked(self, button, down)

    local index = self:GetAttribute("index");

    if (not UnitExists("target")) then
        CleanUI_ChatInsertText("{rt"..index.."}");
    end
    
    if (button == "LeftButton") then
        SetRaidTarget("target", index);
    elseif (button == "RightButton") then
        SetRaidTarget("target", 0);
    end
end

function CleanUI_RaidFlareSnap()

    local doSnap = true;

    if (IsInRaid()) then
        if ( UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") ) then  
            doSnap = true;
        else
            doSnap = false;  
        end
    end
    
    if (not doSnap) then
        return;
    end
    
    local frame = _G["CUI_ChatRaidFlares"];
    
    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();
    
    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();
    
    x = x / scale;  
    y = y / scale;
    
    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
    
    CleanUI_SetBackdrop(frame);  
end

function CleanUI_ResetRaidFlarePosition()
    local frame = _G["CUI_ChatRaidFlares"];
    local borderframe = _G["CUI_ChatActionBarBorder"];
    frame:ClearAllPoints();
    frame:SetPoint("TOPRIGHT", borderframe, "TOPRIGHT", -160, -2);   
    
    frame:SetBackdrop(nil);   
end

function CleanUI_RemoveUiButtonTexture(button)
    button.Left:Hide();
    button.Middle:Hide();
    button.Right:Hide();
end

function CleanUI_Great_OnClick(self, textarray)
   	local rnd = math.random(1, #textarray);
    local msg = textarray[rnd].text;
    local chatType = "GUILD";

    if (IsAltKeyDown() and self == hoveredButton) then
        if (IsInRaid()) then
            chatType = "RAID";
        elseif (IsInGroup()) then
            chatType = "PARTY";
        end
    else
        chatType = "GUILD";
    end
    
    SendChatMessage(msg, chatType);
end

function CleanUI_Great_OnEnter(self)
    hoveredButton = self;
    CleanUI_UpdateButton(self);
end

function CleanUI_Great_OnLeave(self)
    hoveredButton = nil;
    CleanUI_UpdateButton(self);
end

function CleanUI_UpdateButton(self)
    if (not self) then
        return;
    end

    local text = self:GetAttribute("TEXT");
    local color = {r=0.8, b=0.8, b=0.8};
    
    if (IsAltKeyDown() and self == hoveredButton) then
        if (IsInRaid()) then
            color = ChatTypeInfo["RAID"]; --raid
        elseif (IsInGroup()) then
            color = ChatTypeInfo["PARTY"]; --group
        end
    elseif (not hoveredButton) then
        color = {r=0.8, b=0.8, b=0.8}; --nothing
    else
        color = ChatTypeInfo["GUILD"]; --guild
    end
    
    local hexcolor = cui_rgb2hex(color.r, color.g, color.b)
    self:SetText("|cff"..hexcolor..text.."|r"); --guild
end

function CleanUI_AddChatMenuEmoticons()
	UIMenu_AddButton(ChatMenu, "Emoticons", nil, nil, "CUI_EmoticonMenu");
	UIMenu_AutoSize(ChatMenu);
end

function CleanUI_InitEmoticonMenu(self)
	
	UIMenu_Initialize(self);
	UIMenu_AddButton(self, "White Smiley", nil, function(frame) CleanUI_ChatInsertText("☺") end);
	UIMenu_AddButton(self, "Black Smiley", nil, function(frame) CleanUI_ChatInsertText("☻") end);
	UIMenu_AddButton(self, "Sun", nil, function(frame) CleanUI_ChatInsertText("☼") end);
	UIMenu_AddButton(self, "Black heart", nil, function(frame) CleanUI_ChatInsertText("♥") end);
	UIMenu_AddButton(self, "Diamond", nil, function(frame) CleanUI_ChatInsertText("♦") end);
	UIMenu_AddButton(self, "Clove (Clover/Puppyfeet)", nil, function(frame) CleanUI_ChatInsertText("♣") end);
	UIMenu_AddButton(self, "Spade", nil, function(frame) CleanUI_ChatInsertText("♠") end);
	UIMenu_AddButton(self, "Musical Double Quarter Note", nil, function(frame) CleanUI_ChatInsertText("♫") end);
	UIMenu_AutoSize(self);
end

function CleanUI_ChatInsertText(text)
	ChatMenu:Hide();
	
    if (not DEFAULT_CHAT_FRAME.editBox:IsShown()) then
        DEFAULT_CHAT_FRAME.editBox:Show();
    end
    
	DEFAULT_CHAT_FRAME.editBox:SetFocus()
	DEFAULT_CHAT_FRAME.editBox:Insert(text);
end