local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local actionbuttons = {};

function CleanUI_InitDeBuffs()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI");
    end

    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_DeBuffsOnUpdate);  
    
    frame:RegisterEvent("UNIT_AURA"); 
    
    frame:HookScript("OnEvent", CleanUI_DeBuffsOnEvent); 
    
    CleanUI_CleanDeBuffs();
    
    CleanUI_DeBuffsHookEvents();
end

function CleanUI_DeBuffsOnEvent(self, event, ...)
    if (event == "UNIT_AURA") then
        CleanUI_CleanDeBuffs();
    end
end

function CleanUI_DeBuffsHookEvents()        
    hooksecurefunc("ActionButton_CalculateAction", CleanUI_ActionButton_CalculateAction);
end

function CleanUI_ActionButton_CalculateAction(button)    
    local name = button:GetName();
    
    if (name) then
        local type, globalID, subType = GetActionInfo(button.action);
        local spellname, spellrank = GetSpellInfo(globalID);
        actionbuttons[name] = {spell = spellname, rank = spellrank};
    end
end

function CleanUI_CleanDeBuffs()
    if (MASQUE and buttongroup) then
        if (CleanUIData.MASQUE_BUFFS == 1) then
            CleanUI_SkinBuffs();
        end
    end
end

local UPDATE_WAIT = 0.05;
local _WaitTimeSinceLast = 0;
function CleanUI_DeBuffsOnUpdate(self, elapsed)
    if (_WaitTimeSinceLast <= 0) then
        CleanUI_DeBuffsUpdate();
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_DeBuffsUpdate()

    local button, statusbar, shown;
    local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId;
    local target, modifiedClick;
    
    for buttonname, data in pairs(actionbuttons) do
        button = _G[buttonname];
        shown = nil;
    
        if (CleanUIData.ACTIONBARS_DEBUFFS == 1 and InCombatLockdown()) then
            if (data.spell and data.rank) then              
                target = "target";
                modifiedClick = GetModifiedClick("FOCUSCAST");
                
                -- focus infos
                if (modifiedClick == "ALT" and IsAltKeyDown()) then
                    target = "focus";                
                elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
                    target = "focus";            
                elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
                    target = "focus";
                end
            
                -- check buffs
                name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(target, data.spell, data.rank);

                if (name and unitCaster == "player") then
                    shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime);
                end
                
                if (not shown) then
                    -- check debuffs
                    name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(target, data.spell, data.rank);
                     
                    if (name and unitCaster == "player") then
                        shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime);
                    end
                end
                
                if (not shown) then
                    -- self buffs
                    name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff("player", data.spell, data.rank);
                     
                    if (name and unitCaster == "player") then
                        shown = CleanUI_UpdateDeBuffInfo(button, duration, expirationTime);
                    end
                end
            end
        end
        
        if (not shown) then    
            if (button) then       
                statusbar = CleanUI_GetStatusBar(button);
                statusbar:Hide();
            end
        end
    end
end

function CleanUI_SkinBuffs()
    local button;
        
    for i=1, BUFF_MAX_DISPLAY do
        button = _G["BuffButton"..i];
        if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
        end
    end
    
    for i=1, DEBUFF_MAX_DISPLAY do
        button = _G["DebuffButton"..i];
        if (button) then
            buttongroup:RemoveButton(button);
            buttongroup:AddButton(button);
            
            if (CleanUIData.BIGGER_DEBUFFS) then
                -- resize debuffs (default = 30)
                button:SetWidth(36);
                button:SetHeight(36);
            end
        end
    end
end

function CleanUI_UpdateDeBuffInfo(button, duration, expirationTime)
    if (not button) then
        return nil;
    end
    
    local statusbar = CleanUI_GetStatusBar(button);
    
    statusbar:Show();
    
    local startTime = expirationTime - duration;
    statusbar:SetMinMaxValues(0, expirationTime-startTime);
    statusbar:SetValue(expirationTime-GetTime());
    
    return 1;
end

function CleanUI_GetStatusBar(button)
    local name = button:GetName().."DeBuffBar";
    local statusbar = _G[name];
    
    if (not statusbar) then
        statusbar = CreateFrame("StatusBar", name, button, "CleanUIActionButtonDeBuffBarTemplate");
    end
    
    local width = button:GetWidth() - 2;
    local height = CleanUIData.ACTIONBARS_DEBUFFS_HEIGHT or 2;
    
    statusbar:SetWidth(width);
    statusbar:SetHeight(height);
    statusbar:ClearAllPoints();
    statusbar:SetPoint("BOTTOM", button, "BOTTOM", 0, 1);
    statusbar:SetStatusBarColor(CleanUIData.ACTIONBARS_DEBUFFS_R, CleanUIData.ACTIONBARS_DEBUFFS_G, CleanUIData.ACTIONBARS_DEBUFFS_B);

    return statusbar;
end
