function CleanUI_InitDungeonBrowser()
    CleanUI_DungeonBrowserHookEvents();
    CleanUI_RegisterDungeonBrowserEvents();
    
    -- DungeonBrowser stylen
    CleanUI_CleanDungeonBrowserFrame();
    
    -- RaidBrowser stylen
    CleanUI_CleanRaidBrowserFrame();
end
    
function CleanUI_RegisterDungeonBrowserEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_DungeonBrowserOnEvent); 
end

function CleanUI_DungeonBrowserOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_ChallengesUI" ) then
            CleanUI_CleanChallengesUI();
        end
    end
end

function CleanUI_DungeonBrowserHookEvents()
    hooksecurefunc("RaidFinderQueueFrameSelectionDropDown_Initialize", CleanUI_RaidFinderQueueFrameSelectionDropDown_Initialize);
    hooksecurefunc("FlexRaidFrameSelectionDropDown_Initialize", CleanUI_FlexRaidFrameSelectionDropDown_Initialize);
end

function CleanUI_RaidFinderQueueFrameSelectionDropDown_Initialize(self)
	local listFrame = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL];
	
	local button, text;
	local numEncounters, numCompleted;
	local isAvailable, isAvailableToPlayer;
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i];
		if(i <= listFrame.numButtons) then
			numEncounters, numCompleted = GetLFGDungeonNumEncounters(button.value);
			isAvailable, isAvailableToPlayer = IsLFGDungeonJoinable(button.value);
			
			text = button:GetText().." ("..numCompleted.."/"..numEncounters..")"
			
			if ( isAvailable ) then
				if (numCompleted == 0) then
					text = GREEN_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
				end
			else
				text = RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
			end
			
			button:SetText(text);
			button.dungeonID = button.value;
		end
	end
end

function CleanUI_FlexRaidFrameSelectionDropDown_Initialize(self)
	local listFrame = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL];
	
	local button, text;
	local numEncounters, numCompleted;
	local isAvailable, isAvailableToPlayer;
	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i];
		if(i <= listFrame.numButtons) then
			numEncounters, numCompleted = GetLFGDungeonNumEncounters(button.value);
			isAvailable, isAvailableToPlayer = IsLFGDungeonJoinable(button.value);
			
			text = button:GetText().." ("..numCompleted.."/"..numEncounters..")"
			
			if ( isAvailable ) then
				if (numCompleted == 0) then
					text = GREEN_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
				end
			else
				text = RED_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE;
			end
			
			button:SetText(text);
			button.dungeonID = button.value;
		end
	end
end

function CleanUI_HideEncounterTooltip()
    GameTooltip:FadeOut();   
end

function CleanUI_CleanDungeonBrowserFrame()
    CleanUI_RemoveTextures(PVEFrame);  
    CleanUI_SetBackdrop(PVEFrame);  
    
    PVEFrame:SetHeight(424);
    
    PVEFrame.shadows:Hide(); 
    
    PVEFramePortrait:Hide();    
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["PVEFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
        
    CleanUI_ChangeCloseButton(PVEFrame);
    
    CleanUI_CreateHeaderFrame(PVEFrame, "CUI_PVEFrameHeaderBorderframe", 250, PVEFrameTitleText);
    
    CleanUI_ChangeDropDown(LFDQueueFrameTypeDropDown);

    CleanUI_RemoveTextures(PVEFrameLeftInset);  
    CleanUI_RemoveTextures(LFDParentFrame);    
    CleanUI_RemoveTextures(LFDParentFrameInset);
    CleanUI_RemoveTextures(LFDQueueFrame);
   
    LFDQueueFrameRandomScrollFrameScrollBackground:Hide();
    LFDQueueFrameRandomScrollFrameScrollBackgroundTopLeft:Hide();
    LFDQueueFrameRandomScrollFrameScrollBackgroundBottomRight:Hide();
   
    RaidFinderQueueFrameScrollFrameScrollBackground:Hide();
    RaidFinderQueueFrameScrollFrameScrollBackgroundTopLeft:Hide();
    RaidFinderQueueFrameScrollFrameScrollBackgroundBottomRight:Hide();
   
    ScenarioQueueFrameRandomScrollFrameScrollBackground:Hide();
    ScenarioQueueFrameRandomScrollFrameScrollBackgroundTopLeft:Hide();
    ScenarioQueueFrameRandomScrollFrameScrollBackgroundBottomRight:Hide();
    
    CleanUI_ChangeButton(LFDQueueFrameFindGroupButton);  
    
    CleanUI_ReplaceAllTextures("bluemenu-vert", PVEFrame, nil);
    
    -- RaidFinder
    CleanUI_RemoveTextures(RaidFinderFrame);
    CleanUI_RemoveTextures(RaidFinderQueueFrame);
    CleanUI_RemoveTextures(RaidFinderFrameRoleInset);
    CleanUI_RemoveTextures(RaidFinderFrameBottomInset);
    CleanUI_ChangeDropDown(RaidFinderQueueFrameSelectionDropDown);
    
    CleanUI_ChangeButton(RaidFinderFrameFindRaidButton);  
    
    CleanUI_ChangeButton(RaidFinderQueueFramePartyBackfillBackfillButton);  
    CleanUI_ChangeButton(RaidFinderQueueFramePartyBackfillNoBackfillButton);  
    
    -- Scenario
    CleanUI_RemoveTextures(ScenarioFinderFrame);
    CleanUI_ChangeDropDown(ScenarioQueueFrameTypeDropDown);
    CleanUI_ChangeButton(ScenarioQueueFrameFindGroupButton);  
    
    LFDQueueFrameFindGroupButton:SetPoint("BOTTOMLEFT", 90, 7);
    RaidFinderFrameFindRaidButton:SetPoint("BOTTOMLEFT", 90, 7);
    ScenarioQueueFrameFindGroupButton:SetPoint("BOTTOMLEFT", 90, 7);
    
    -- Flex Raid
    CleanUI_RemoveTextures(FlexRaidFrame);
    CleanUI_RemoveTextures(FlexRaidFrameBottomInset);
    CleanUI_ChangeDropDown(FlexRaidFrameSelectionDropDown);
    CleanUI_ChangeButton(FlexRaidFrameStartRaidButton);  
    FlexRaidFrameStartRaidButton:SetPoint("BOTTOM", 0, 6);
end

function CleanUI_CleanChallengesUI()
    CleanUI_RemoveTextures(ChallengesFrameInset);
    CleanUI_RemoveTextures(ChallengesFrameDetails);
    ChallengesFrameDetails.bg:Hide();
    
    CleanUI_ReplaceAllTextures("bluemenu-vert", ChallengesFrame, nil);
    
    CleanUI_ChangeButton(ChallengesFrameLeaderboard);   
end

function CleanUI_CleanRaidBrowserFrame()
    CleanUI_RemoveTextures(RaidBrowserFrame);  
    CleanUI_SetBackdrop(RaidBrowserFrame);  
    
    CleanUI_SetBackdrop(LFRBrowseFrame);  
    
    RaidBrowserFramePortrait:Hide(); 
    
    local sideTabButton;
    for i=1, 10 do
        sideTabButton = _G["LFRParentFrameSideTab"..i];
        if (not sideTabButton) then
            break;
        end
        
        CleanUI_ChangeRaidBrowserSideTab(sideTabButton);
    end
        
    CleanUI_ChangeCloseButton(RaidBrowserFrame); 
    
    --panels
	CleanUI_RemoveTextures(LFRQueueFrameRoleInset); 
	CleanUI_RemoveTextures(LFRQueueFrameListInset); 
	CleanUI_RemoveTextures(LFRQueueFrameCommentInset); 
	CleanUI_RemoveTextures(LFRBrowseFrameRoleInset); 

	--Buttons
	CleanUI_ChangeButton(LFRQueueFrameFindGroupButton); 
	CleanUI_ChangeButton(LFRQueueFrameAcceptCommentButton); 
	CleanUI_ChangeButton(LFRBrowseFrameSendMessageButton); 
	CleanUI_ChangeButton(LFRBrowseFrameInviteButton); 
	CleanUI_ChangeButton(LFRBrowseFrameRefreshButton); 

	-- DD
	CleanUI_ChangeDropDown(LFRBrowseFrameRaidDropDown); 
	
	-- scroll frames
	LFRBrowseFrameListScrollFrameTop:Hide();
	LFRBrowseFrameListScrollFrameBottom:Hide();
	LFRBrowseFrameListScrollFrameMiddle:Hide();
	
	LFRQueueFrameSpecificListScrollFrameScrollBackgroundTopLeft:Hide();
	LFRQueueFrameSpecificListScrollFrameScrollBackgroundBottomRight:Hide();
end

function CleanUI_ChangeRaidBrowserSideTab(tabButton)
	local texture = tabButton:GetNormalTexture():GetTexture();
	
    CleanUI_RemoveTextures(tabButton);
    
    local borderframe = _G[tabButton:GetName().."Borderframe"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", tabButton:GetName().."Borderframe", tabButton);
    end
    
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -4, 2);
    borderframe:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 2, -2);
    
    borderframe:SetFrameStrata("BACKGROUND");
    
    borderframe:SetBackdrop( { 
        bgFile = CUI_TAB_BACKGROUND_RIGHT, 
        edgeFile = border, tile = true, tileSize = 64, edgeSize = 16, 
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
    
    tabButton:SetNormalTexture(texture);
    tabButton:SetPushedTexture(nil);
    tabButton:SetHighlightTexture(nil);    
    
    tabButton:GetNormalTexture():SetTexCoord(0.1, 0.9, 0.1, 0.9);
    
    local checkedTexture = tabButton:GetCheckedTexture();
    checkedTexture:ClearAllPoints();
    checkedTexture:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -4, 2);
    checkedTexture:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 2, -2);
    tabButton:SetCheckedTexture("Interface\\AddOns\\CleanUI\\skins\\highlight");
end