function CleanUI_InitGuild()
    CleanUI_RegisterGuildEvents();
end
    
function CleanUI_RegisterGuildEvents()         
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_GuildOnEvent); 
end

function CleanUI_GuildOnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		local addonName = ...;
        if ( addonName == "Blizzard_GuildUI" ) then
            -- needs blizzard addon
            CleanUI_StartDelay(2, CleanUI_InitGuildRoster);
            
            -- clean guild frame
            CleanUI_CleanGuildFrame();
        end
        
        if ( addonName == "Blizzard_LookingForGuildUI" ) then            
            -- clean looking for guild frame
            CleanUI_CleanLookingForGuildFrame();
        end
    end
end

function CleanUI_GuildRosterFrame_OnShow(self)
    CleanUI_InitGuildRoster();
end

function CleanUI_InitGuildRoster()
    local button;

    for ii = 1, 14 do
        button = _G["GuildRosterContainerButton"..ii];
        button:SetScript("OnEnter", function() CleanUI_GuildRosterShowTooltip(ii); end);
    end
    
    CleanUI_MakeFrameMoveable(GuildFrame);
end

function CleanUI_CleanGuildFrame()    
    CleanUI_RemoveTextures(GuildFrame);  
    CleanUI_SetBackdrop(GuildFrame);
    CleanUI_RemoveTextures(GuildNewPerksFrame);  
    CleanUI_RemoveTextures(GuildFrameInset);  
    CleanUI_RemoveTextures(GuildFrameBottomInset);  
    CleanUI_RemoveTextures(GuildNewPerksFrame);  
    CleanUI_RemoveTextures(GuildAllPerksFrame);  
    CleanUI_RemoveTextures(GuildPerksToggleButton);
    CleanUI_SetBackdrop(GuildPerksToggleButton);
    
    CleanUI_RemoveTextures(GuildLevelFrame);   
    CleanUI_SetBackdrop(GuildLevelFrame, true, true); 
    
    GuildFactionBarLeft:Hide();
    GuildFactionBarMiddle:Hide();
    GuildFactionBarRight:Hide();
    GuildFactionBarBG:SetTexture(CUI_FRAME_BACKGROUND_OPAQUE);
    GuildFactionBarProgress:SetTexture(CleanUIData.CUI_STATUSBAR);
    
    GuildFramePortrait:Hide();
    GuildFrameTabardEmblem:Show();
    GuildFrame_UpdateTabard();
    
    GuildFrameTabardEmblem:ClearAllPoints();
    GuildFrameTabardEmblem:SetPoint("TOPLEFT", GuildFrame, "TOPLEFT", 5, -12);
        
    CleanUI_ChangeCloseButton(GuildFrame);
    
    CleanUI_CreateHeaderFrame(GuildFrame, "CUI_GuildFrameHeaderBorderframe", 250, GuildFrameTitleText);
    
    GuildLevelFrame:SetSize(36, 28); 
    GuildLevelFrame:ClearAllPoints();
    GuildLevelFrame:SetPoint("RIGHT", CUI_GuildFrameHeaderBorderframe, "LEFT", -2, 0); 
    
    local tabButton, borderframe;
    for i=1, 10 do
        tabButton = _G["GuildFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
    
    GuildXPFrameLevelText:ClearAllPoints();
    GuildXPFrameLevelText:SetPoint("TOP", GuildFrame, "TOP", 0, -37);
     
    GuildPerksContainerScrollBarTrack:Hide();
    
    CleanUI_CleanGuildRosterFrame();
    CleanUI_CleanGuildNewsFrame();
    CleanUI_CleanGuildRewardsFrame();
    CleanUI_CleanGuildInfoFrame();
    CleanUI_CleanGuildMemberDetailFrame();
end

function CleanUI_CleanGuildRosterFrame()    
    GuildRosterContainerScrollBarBG:Hide();
    GuildRosterContainerScrollBarTop:Hide();
    GuildRosterContainerScrollBarBottom:Hide();
    GuildRosterContainerScrollBarMiddle:Hide();
    
    CleanUI_ChangeDropDown(GuildRosterViewDropdown);
    
    local rosterColumnButton;
    for i=1, 10 do
        rosterColumnButton = _G["GuildRosterColumnButton"..i];
        if (not rosterColumnButton) then
            break;
        end
        
        CleanUI_ChangeFilterButton(rosterColumnButton, true);
    end
end

function CleanUI_CleanGuildNewsFrame()  
    CleanUI_RemoveTextures(GuildNewsFrame);
    CleanUI_RemoveTextures(GuildNewsFiltersFrame);
    CleanUI_SetBackdrop(GuildNewsFiltersFrame);  
        
    CleanUI_ChangeCloseButton(GuildNewsFiltersFrame);
    
    GuildNewsContainerScrollBarTrack:Hide();
end

function CleanUI_CleanGuildRewardsFrame()  
    CleanUI_RemoveTextures(GuildRewardsFrame);
    
    GuildRewardsContainerScrollBarBG:Hide();
    GuildRewardsContainerScrollBarTop:Hide();
    GuildRewardsContainerScrollBarBottom:Hide();
    GuildRewardsContainerScrollBarMiddle:Hide();  
end

function CleanUI_CleanGuildInfoFrame()    

    -- Nachricht des Tages/Gildeninfo
    CleanUI_RemoveTextures(GuildTextEditFrame);
    CleanUI_SetBackdrop(GuildTextEditFrame);  
    CleanUI_RemoveTextures(GuildTextEditContainer);
    CleanUI_SetBackdrop(GuildTextEditContainer);  
        
    CleanUI_ChangeCloseButton(GuildTextEditFrame);
    
    for i, child in ipairs({GuildTextEditFrame:GetChildren()}) do
        if (child and child:IsObjectType("Button")) then
            -- HACK - 2 components with same name
            if (child:GetWidth() > 50) then
                CleanUI_ChangeButton(child);
            end
        end
    end
    
    local guildInfoFrameTab;
    for i=1, 10 do
        guildInfoFrameTab = _G["GuildInfoFrameTab"..i];
        if (not guildInfoFrameTab) then
            break;
        end
        
        CleanUI_RemoveTextures(guildInfoFrameTab);
    end
    
    CleanUI_RemoveTextures(GuildInfoFrameInfo);
    GuildInfoDetailsFrameScrollBarTrack:Hide();
    
    CleanUI_ChangeButton(GuildAddMemberButton);
    CleanUI_ChangeButton(GuildViewLogButton);
    CleanUI_ChangeButton(GuildControlButton);
    CleanUI_ChangeButton(GuildRecruitmentInviteButton);
    CleanUI_ChangeButton(GuildRecruitmentMessageButton);
    CleanUI_ChangeButton(GuildRecruitmentDeclineButton);
    
    CleanUI_RemoveTextures(GuildLogFrame);
    CleanUI_SetBackdrop(GuildLogFrame);  
    CleanUI_RemoveTextures(GuildLogContainer);
    CleanUI_SetBackdrop(GuildLogContainer);
    
    --CleanUI_CreateHeaderFrame(GuildLogFrame, "CUI_GuildLogFrameHeaderBorderframe", 250, GuildLogFrameTitle);
        
    local closeButton1 = CleanUI_GetChildWithNameAtPos(GuildLogFrame, "GuildLogFrameCloseButton", 0);
    CleanUI_ChangeCloseButton(GuildLogFrame, nil, closeButton1);
    
    local closeButton2 = CleanUI_GetChildWithNameAtPos(GuildLogFrame, "GuildLogFrameCloseButton", 1);
    CleanUI_ChangeButtonRed(closeButton2);
end

function CleanUI_CleanGuildMemberDetailFrame()   
    CleanUI_RemoveTextures(GuildMemberDetailFrame);
    CleanUI_SetBackdrop(GuildMemberDetailFrame);
    
    CleanUI_ChangeDropDown(GuildMemberRankDropdown); 
    GuildMemberDetailRankText:Hide();  
    
    CleanUI_ChangeButton(GuildControlButton);
    CleanUI_ChangeButton(GuildMemberRemoveButton);
    CleanUI_ChangeButton(GuildMemberGroupInviteButton);
    
    CleanUI_SetBackdrop(GuildMemberNoteBackground);
    CleanUI_SetBackdrop(GuildMemberOfficerNoteBackground);
end

function CleanUI_CleanLookingForGuildFrame()   
    CleanUI_RemoveTextures(LookingForGuildFrame);
    CleanUI_SetBackdrop(LookingForGuildFrame);
    
    CleanUI_RemoveTextures(LookingForGuildFrameInset);
    
    LookingForGuildFramePortrait:Hide();   
         
    CleanUI_ChangeCloseButton(LookingForGuildFrame);
    
    CleanUI_CreateHeaderFrame(LookingForGuildFrame, "CUI_LookingForGuildFrameHeaderBorderframe", 250, LookingForGuildFrameTitleText);
    
    CleanUI_ChangeButton(LookingForGuildBrowseButton);
    
    CleanUI_RemoveTextures(LookingForGuildCommentInputFrame);

    CleanUI_RemoveTextures(LookingForGuildFrameTab1);
    CleanUI_RemoveTextures(LookingForGuildFrameTab2);
    CleanUI_RemoveTextures(LookingForGuildFrameTab3);
    
    LookingForGuildBrowseFrameContainerScrollBarBG:Hide();
    LookingForGuildBrowseFrameContainerScrollBarTop:Hide();
    LookingForGuildBrowseFrameContainerScrollBarBottom:Hide();
    LookingForGuildBrowseFrameContainerScrollBarMiddle:Hide();
    CleanUI_ChangeButton(LookingForGuildRequestButton);
end

function CleanUI_GuildRosterShowTooltip(index)
    local button = _G["GuildRosterContainerButton"..index];
    
    local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(button.guildIndex);
    
    GameTooltip_SetDefaultAnchor(GameTooltip, button);    
    GameTooltip:SetOwner(button, "ANCHOR_RIGHT");
    
    local classColor = RAID_CLASS_COLORS[classFileName];
    local classColorText = string.format("ff%.2x%.2x%.2x", classColor.r * 255, classColor.g * 255, classColor.b * 255);

    GameTooltip:SetText("|c"..classColorText..name.."|r ("..level..")");    
    GameTooltip:AddLine("|cffffffff"..rank.."|r");        
    if (online) then
        GameTooltip:AddLine(BN_TOAST_ONLINE);
    else
        GameTooltip:AddLine(BN_TOAST_OFFLINE);
        GameTooltip:AddLine("|cffffffff"..string.format(BNET_LAST_ONLINE_TIME, RecentTimeDate(GetGuildRosterLastOnline(button.guildIndex))).."|r");  
    end
    GameTooltip:AddLine(" "); 
    
    if (zone) then
	    GameTooltip:AddLine("|cffffffff"..ZONE..":|r "..zone);   
	    GameTooltip:AddLine(" "); 
    end
    
    GameTooltip:AddLine(note);       
    if (note) then
        GameTooltip:AddLine(officernote);
    end
    GameTooltip:AddLine(" "); 
    
    GameTooltip:AddLine(ACHIEVEMENTS..":");     
    GameTooltip:AddLine("|cffffffff"..ACHIEVEMENT_POINTS..":|r "..achievementPoints);  
    GameTooltip:AddLine("|cffffffff"..RANK..":|r "..achievementRank);
        
    GameTooltip:Show();
end

function CleanUI_GuildRosterHideTooltip()
    GameTooltip:FadeOut();
end
