CUI_MAILLOGS_MAX = 200;

local sendItems = {};

local postal = false;

-- saved variables per character
CleanUIMailLog = {};

function CleanUI_InitMail()
    CleanUI_CreateLogFrame();
    
    CleanUI_MailHookEvents();
    CleanUI_RegisterMailEvents();
    
    CleanUI_CleanMailFrame();
    CleanUI_CleanOpenMailFrame();
    
    CleanUI_CleanPostal();
end

function CleanUI_MailHookEvents()
    hooksecurefunc("SendMailFrame_Update", CleanUI_SendMailFrame_Update);
    hooksecurefunc("OpenMailFrame_UpdateButtonPositions", CleanUI_OpenMailFrame_UpdateButtonPositions);
    hooksecurefunc("MailFrameTab_OnClick", CleanUI_MailFrameTab_OnClick);
    hooksecurefunc("SendMailFrame_SendMail", CleanUI_SendMailFrame_SendMail);
end
    
function CleanUI_RegisterMailEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    frame:RegisterEvent("MAIL_SEND_INFO_UPDATE"); 
    frame:RegisterEvent("MAIL_SEND_SUCCESS"); 
    
    frame:HookScript("OnEvent", CleanUI_MailOnEvent); 
end

function CleanUI_MailOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Postal" ) then
            CleanUI_CleanPostal();
        end
    end
    if (event == "MAIL_SEND_INFO_UPDATE") then
        CleanUI_MailSendInfoUpdate();
    end
    if (event == "MAIL_SEND_SUCCESS") then
        sendItems = {};
    end
end

function CleanUI_CleanMailFrame()
    CleanUI_RemoveTextures(MailFrame);  
    CleanUI_SetBackdrop(MailFrame);   
    
    MailFramePortrait:Hide(); 
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["MailFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
        
    CleanUI_ChangeCloseButton(MailFrame);
    
    local headerBorder = CleanUI_CreateHeaderFrame(MailFrame, "CUI_MailFrameHeaderBorderframe", 200, MailFrameTitleText);
       
    InboxTitleText:ClearAllPoints(); 
    InboxTitleText:SetPoint("CENTER", headerBorder, "CENTER", 0, 0);
    InboxTitleText:SetJustifyH("CENTER");
    
    InboxTitleText:SetTextColor(0.3, 0.7, 1.0);
       
    SendMailTitleText:ClearAllPoints(); 
    SendMailTitleText:SetPoint("CENTER", headerBorder, "CENTER", 0, 0);
    SendMailTitleText:SetJustifyH("CENTER");
    
    SendMailTitleText:SetTextColor(0.3, 0.7, 1.0);
    
    CUIMailLogScrollFrameHeader:ClearAllPoints(); 
    CUIMailLogScrollFrameHeader:SetPoint("CENTER", headerBorder, "CENTER", 0, 0);
    CUIMailLogScrollFrameHeader:SetJustifyH("CENTER");
    
    CUIMailLogScrollFrameHeader:SetTextColor(0.3, 0.7, 1.0);
    
    CleanUI_RemoveTextures(MailFrameInset); 
     
    CleanUI_RemoveTextures(InboxFrame);  
    
    local mailItem, button;
    for i=1, 7 do
        mailItem = _G["MailItem"..i];
        button = _G["MailItem"..i.."Button"];
        if (mailItem) then
            CleanUI_RemoveTextures(mailItem);
            CleanUI_RemoveTextures(button);
        end
    end
    
    CleanUI_ChangeButton(CUIMailLogResetButton); 
    
    -- SendMailFrame
    CleanUI_RemoveTextures(SendMailFrame);
    CleanUI_RemoveTextures(SendMailScrollFrame);
    SendStationeryBackgroundLeft:Hide();
    SendStationeryBackgroundRight:Hide();
    
    CleanUI_ChangeEditBox(SendMailNameEditBox, 28, -12, -12);
    CleanUI_ChangeEditBox(SendMailSubjectEditBox, 28, -12, -12);
    
    CleanUI_ChangeEditBox(SendMailMoneyGold, 28, -12, -12);
    CleanUI_ChangeEditBox(SendMailMoneySilver, 28, -12, -22);
    CleanUI_ChangeEditBox(SendMailMoneyCopper, 28, -12, -22);
    
    CleanUI_RemoveTextures(SendMailMoneyInset);
    SendMailMoneyBg:Hide();
    
    CleanUI_ChangeButton(SendMailMailButton); 
    CleanUI_ChangeButtonRed(SendMailCancelButton); 
    
    local button, background;
    for i=1, 16 do
    	button = _G["SendMailAttachment"..i];
    	CleanUI_RemoveTextures(button);
    	
	    local background = _G["SendMailAttachment"..i.."Background"];
	    if (not background) then
	        background = button:CreateTexture("SendMailAttachment"..i.."Background", "BACKGROUND");
	        background:SetAllPoints(button);
	        background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\unspecified");
	    end
    end
    
    local background = SendMailScrollFrame:CreateTexture("CUI_SendMailFrameBackground", "BORDER");
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", SendMailScrollFrame, "TOPLEFT", 0, 0);
    background:SetPoint("BOTTOMRIGHT", SendMailScrollFrame, "BOTTOMRIGHT", 5, -5);
    
    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");
end

function CleanUI_CleanPostal()
    if (postal) then
        return;
    end
    
    if (not IsAddOnLoaded("Postal")) then
        return;
    end
    
    CleanUI_ChangeButton(PostalSelectOpenButton);  
    CleanUI_ChangeButton(PostalSelectReturnButton);  
    CleanUI_ChangeButton(PostalOpenAllButton);  
    
    Postal_ModuleMenuButton:ClearAllPoints(); 
    Postal_ModuleMenuButton:SetPoint("RIGHT", MailFrameCloseButton, "LEFT", 0, 0);

    postal = true;
end

function CleanUI_CleanOpenMailFrame()
    CleanUI_RemoveTextures(OpenMailFrame);  
    CleanUI_SetBackdrop(OpenMailFrame);   
    
    OpenMailFrameIcon:Hide(); 
        
    CleanUI_ChangeCloseButton(OpenMailFrame);
    
    CleanUI_CreateHeaderFrame(OpenMailFrame, "CUI_OpenMailFrameHeaderBorderframe", 200, OpenMailTitleText);
    
    CleanUI_ChangeButton(OpenMailReportSpamButton); 
    
    CleanUI_ChangeButton(OpenMailReplyButton); 
    CleanUI_ChangeButton(OpenMailDeleteButton); 
    CleanUI_ChangeButtonRed(OpenMailCancelButton); 
    
    CleanUI_RemoveTextures(OpenMailFrameInset);
    CleanUI_RemoveTextures(OpenMailScrollFrame);
    OpenStationeryBackgroundLeft:Hide();
    OpenStationeryBackgroundRight:Hide();
    
    local background = OpenMailScrollFrame:CreateTexture("CUI_OpenMailFrameBackground", "BORDER");
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", OpenMailScrollFrame, "TOPLEFT", 0, 0);
    background:SetPoint("BOTTOMRIGHT", OpenMailScrollFrame, "BOTTOMRIGHT", 5, -5);
    
    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");
end

function CleanUI_MailSendInfoUpdate()
    local itemName, itemLink, itemRarity, itemTexture, stackCount, itemQuality;
    for i=1, ATTACHMENTS_MAX_SEND do
        itemName, itemTexture, stackCount, itemQuality = GetSendMailItem(i);
        if (itemName) then
            itemLink = GetSendMailItemLink(i);
            _, _, itemRarity = GetItemInfo(itemLink);
            sendItems[itemName] = {texture=itemTexture, count=stackCount, rarity=itemRarity};
        end
    end
end
    
function CleanUI_CreateLogFrame()
    PanelTemplates_SetNumTabs(MailFrame, 3);
    
    local button = CreateFrame("Button", "MailFrameTab3", MailFrame, "FriendsFrameTabTemplate");
    button:SetAttribute("id", 3);
    button:SetText(GUILD_BANK_LOG);
    button:ClearAllPoints();
    button:SetPoint("LEFT", MailFrameTab2, "RIGHT", -8, 0);
    button:HookScript("OnClick", function() MailFrameTab_OnClick(self, 3) end);
    
    HybridScrollFrame_OnLoad(CUIMailLogScrollFrame);
    CUIMailLogScrollFrame.update = CleanUI_UpdateMailLog;
    HybridScrollFrame_CreateButtons(CUIMailLogScrollFrame, "CUIMailLogEntryTemplate");
    
    local resetButton = CreateFrame("Button", "CUIMailLogResetButton", CUIMailLogScrollFrame, "UIPanelButtonTemplate");
    resetButton:SetWidth(120);
    resetButton:SetHeight(25);
    resetButton:SetPoint("TOPRIGHT", MailFrame, "TOPRIGHT", -21, -30);
    resetButton:SetText("Reset");
    resetButton:SetScript("OnClick", CleanUI_MailLogReset);
    resetButton:SetFrameLevel(resetButton:GetFrameLevel() + 1);
    
    local header = CUIMailLogScrollFrame:CreateFontString("CUIMailLogScrollFrameHeader");
    header:SetFontObject("GameFontNormal");
    header:SetWidth(100);
    header:SetHeight(25);
    header:SetText("Log"); 
end

function CleanUI_MailLogReset()
    CleanUIMailLog = {};
    CleanUI_UpdateMailLog();
end

function CleanUI_SendMailFrame_SendMail(self)

    local data = {};
    
    local sendTo = SendMailNameEditBox:GetText();
    local sendSubject = SendMailSubjectEditBox:GetText();
    local sendBody = SendMailBodyEditBox:GetText();
    data.to = sendTo;
    data.subject = sendSubject;
    data.body = sendBody;
    
    data.money = MoneyInputFrame_GetCopper(SendMailMoney);
    
    if ( SendMailSendMoneyButton:GetChecked() ) then
        data.moneytype = SEND_MONEY;
    else
        data.moneytype = COD;
    end
    
    data.items = sendItems;
    
    data.time = time();
    data.expires = time() + 60*60*24*30;
    
    tinsert(CleanUIMailLog, 1, data);
end

function CleanUI_MailFrameTab_OnClick(self, tabID)
    if ( not tabID ) then
        tabID = self:GetID();
    end

    if ( tabID == 3 ) then
        ButtonFrameTemplate_HideButtonBar(MailFrame)
        MailFrameInset:SetPoint("TOPLEFT", 4, -58);
        InboxFrame:Hide();
        SendMailFrame:Hide();
        SetSendMailShowing(false);
        CUIMailLogScrollFrame:Show();
        CleanUI_UpdateMailLog();
    else
        CUIMailLogScrollFrame:Hide();
    end
end

function CleanUI_UpdateMailLog()  
    local buttons = CUIMailLogScrollFrame.buttons;
    
    local numEntries = #CleanUIMailLog;
    local numButtons = #buttons;
            
    local scrollOffset = HybridScrollFrame_GetOffset(CUIMailLogScrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;
    
    local entry, logIndex, data, diffExpires;
    local dayInSeconds = 24*60*60;
    local now = time(); 
    
    for i=1, numButtons do    
        logIndex = i + scrollOffset;
        entry = buttons[i];        
        data = CleanUIMailLog[logIndex];
        
        if ( logIndex <= numEntries ) then
            entry:Show();
            
            diffExpires = data.expires - now;
            
            -- weniger als ein Tag
            if (diffExpires < 0) then
                entry.time:SetText("|cffaaaaaa"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            elseif (diffExpires < dayInSeconds) then
                entry.time:SetText("|cffff0000"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            else
                entry.time:SetText("|cffffffff"..date("%m/%d/%y %H:%M:%S", data.time).."|r");
            end
            
            entry.subject:SetText("|cffffffff"..data.subject.."|r");
            entry.data = data;
        else
            entry:Hide();
        end
        
        displayedHeight = displayedHeight + buttonHeight;
    end
    
    HybridScrollFrame_Update(CUIMailLogScrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_MailLog_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    CleanUIAddons_MailLog_CreateTooltip(GameTooltip, self.data);
end

function CleanUI_MailLog_OnLeave(self)
    GameTooltip:FadeOut();
end

function CleanUIAddons_MailLog_CreateTooltip(tooltip, data)

    tooltip:SetText("|cffffffff"..data.subject.."|r");
    
    tooltip:AddDoubleLine(MAIL_TO_LABEL, "|cffffffff"..data.to.."|r", nil, nil, nil, nil, nil, nil);
    tooltip:AddLine("|cffaaaaaa"..date("%m/%d/%y %H:%M:%S", data.time).."|r", nil, nil, nil, 1);  
    
    tooltip:AddLine(" ", nil, nil, nil, 1);  
    
    local now = time(); 
    local diffExpires = data.expires - now;
    if (diffExpires > 0) then
        tooltip:AddDoubleLine(TIME_UNTIL_RETURNED, "|cffffffff"..SecondsToTime(diffExpires).."|r", nil, nil, nil, nil, nil, nil);
    end
    
    tooltip:AddLine(" ", nil, nil, nil, 1);    
    tooltip:AddLine("|cffffffff"..data.body.."|r", nil, nil, nil, 1);
    
    if (data.money > 0) then
        tooltip:AddLine(" ", nil, nil, nil, 1);  
        local amount = GetCoinTextureString(data.money);  
        tooltip:AddDoubleLine(data.moneytype, "|cffffffff"..amount.."|r", nil, nil, nil, nil, nil, nil);
    end
    
    tooltip:AddLine(" ", nil, nil, nil, 1);
    
    local hasItems = nil, r, g, b, hex, color;
    if (data.items) then
        for itemName, itemData in pairs(data.items) do
            r, g, b, hex = GetItemQualityColor(itemData.rarity or -1);
            color = cui_rgb2hex(r, g, b);
            tooltip:AddDoubleLine("|cff"..color..itemName.."|r", "|cffffffff  "..itemData.count.."|r", nil, nil, nil, nil, nil, nil);
            hasItems = true;
        end
    end
    
    if (not hasItems) then 
        tooltip:AddLine(NO_ATTACHMENTS, nil, nil, nil, 1);
    end
    
    tooltip:Show();
end

function CleanUI_SendMailFrame_Update()
	for i=1, ATTACHMENTS_MAX_SEND do
		local itemLink = GetSendMailItemLink(i);
		
		local button = _G["SendMailAttachment"..i];
		local icon = _G[button:GetName().."Icon"];
		
		if (not icon) then
            icon = button:CreateTexture(button:GetName().."Icon", "BORDER");
        end
        
        icon:ClearAllPoints();
        icon:SetPoint("TOPLEFT", -2, 2);
        icon:SetPoint("BOTTOMRIGHT", 2, -2);
        
		local border = button:GetNormalTexture();
	
        if (border) then
            border:ClearAllPoints();
            border:SetPoint("TOPLEFT", -16, 16);
            border:SetPoint("BOTTOMRIGHT", 16, -16);
        end
    
		local itemName, itemTexture, stackCount, quality = GetSendMailItem(i);
		
		icon:SetTexture(itemTexture);
	    CleanUI_SetItemButtonBorderColor(itemLink, button);
	    
	    _G["SendMailAttachment"..i.."Background"]:Show();
	end
end

function CleanUI_OpenMailFrame_UpdateButtonPositions(letterIsTakeable, textCreated, stationeryIcon, money)
	-- items
	for i=1, ATTACHMENTS_MAX_RECEIVE do
		local name, itemTexture, count, quality, canUse = GetInboxItem(InboxFrame.openMailID, i);
		local itemLink = GetInboxItemLink(InboxFrame.openMailID, i);
		
		local attachmentButton = _G["OpenMailAttachmentButton"..i];
	    CleanUI_SetItemButtonBorderColor(itemLink, attachmentButton);
    end
end