function CleanUI_InitMerchantFrame()
    CleanUI_MerchantFrameHookEvents();
    CleanUI_RegisterMerchantFrameEvents();
    
    -- DungeonBrowser stylen
    CleanUI_CleanMerchantFrame();
end
    
function CleanUI_RegisterMerchantFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_MerchantFrameOnEvent); 
end

function CleanUI_MerchantFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        --if ( addonName == "*" ) then
        --end
    end
end

function CleanUI_MerchantFrameHookEvents()
    hooksecurefunc("MerchantFrame_UpdateMerchantInfo", CleanUI_MerchantFrame_UpdateMerchantInfo);
end

function CleanUI_MerchantFrame_UpdateMerchantInfo()
    local link, texture;
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
    local r, g, b, hex;
    
    local numMerchantItems = GetMerchantNumItems();
    
	for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
		local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i);

		if ( index <= numMerchantItems ) then
	        link = GetMerchantItemLink(index);
	        texture = _G["MerchantItem"..i.."ItemButtonNormalTexture"];
	        if (link) then
	            
	            itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(link); 
	        
	            r, g, b, hex = GetItemQualityColor(itemRarity);
	            
	            if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY == 1) then
	                r, g, b = 0.5, 0.5, 0.5;
	            end
	            
	            texture:SetVertexColor(r, g, b);
	        else
	            texture:SetVertexColor(0.5, 0.5, 0.5);
	        end
	    end
    end
end

function CleanUI_CleanMerchantFrame()
    CleanUI_RemoveTextures(MerchantFrame);    
    CleanUI_SetBackdrop(MerchantFrame);  
    
    MerchantFramePortrait:Hide();    
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["MerchantFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton, 1);
    end
        
    CleanUI_ChangeCloseButton(MerchantFrame);
    
    CleanUI_CreateHeaderFrame(MerchantFrame, "CUI_MerchantFrameHeaderBorderframe", 250, MerchantNameText);
    
    CleanUI_RemoveTextures(MerchantFrameInset);
    
    CleanUI_RemoveTextures(MerchantFrameLootFilter);
    
    CleanUI_RemoveTextures(MerchantMoneyInset);
    MerchantMoneyBg:Hide();  
    
    CleanUI_RemoveTextures(MerchantMoneyFrame);
    CleanUI_RemoveTextures(MerchantExtraCurrencyInset);
    CleanUI_RemoveTextures(MerchantExtraCurrencyBg);
    
    local item;
    for i=1, 12 do
        item = _G["MerchantItem"..i];
        if (not item) then
            break;
        end    
        
        CleanUI_RemoveTextures(item);
        
        _G["MerchantItem"..i.."ItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border"); 
    end
    
    CleanUI_RemoveTextures(MerchantBuyBackItem);
    _G["MerchantBuyBackItemItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
    _G["MerchantBuyBackItemItemButtonNormalTexture"]:SetVertexColor(0.5, 0.5, 0.5);
    
    CleanUI_RemoveTextures(MerchantPrevPageButton); 
    MerchantPrevPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");  
    MerchantPrevPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");  
    MerchantPrevPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled");   
    MerchantPrevPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight");   
    
    CleanUI_RemoveTextures(MerchantNextPageButton); 
    MerchantNextPageButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");  
    MerchantNextPageButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");  
    MerchantNextPageButton:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");   
    MerchantNextPageButton:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight");  
    
    CleanUI_RemoveTextures(MerchantPrevPageButton);        
    CleanUI_ChangeBackButton(MerchantPrevPageButton);
        
    CleanUI_RemoveTextures(MerchantNextPageButton);       
    CleanUI_ChangeNextButton(MerchantNextPageButton); 
end