
local ldb = LibStub:GetLibrary("LibDataBroker-1.1");

local dataobj = ldb:NewDataObject(LUA_ERROR, {
    type = "launcher",
    icon = "Interface\\DialogFrame\\UI-Dialog-Icon-AlertNew",
    OnClick = function(self, button)
        if button == "LeftButton" then
            if (ScriptErrorsFrame) then
                ScriptErrorsFrame:Show();
            end
        end
    end,
    OnTooltipShow = function(tooltip)
        tooltip:AddLine(LUA_ERROR);
    end
});

function CleanUI_InitPopups()
    CleanUI_PopupsHookEvents();
    CleanUI_RegisterPopupsEvents();
    
    CleanUI_ChangeGameMenu(); 
    CleanUI_ChangeStaticPopups();
    CleanUI_ChangeReadyCheckFrame();
    CleanUI_ChangeBonusRollFrame();
    CleanUI_ChangeStackSplitFrame();
end
    
function CleanUI_RegisterPopupsEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:HookScript("OnEvent", CleanUI_PopupsOnEvent); 
end

function CleanUI_PopupsOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then   
        local addonName = ...;
        if (addonName == "Blizzard_DebugTools") then
            CleanUI_ChangeScriptErrorsFrame();
        end
    end
end

function CleanUI_PopupsHookEvents()
    hooksecurefunc("GameMenuFrame_UpdateVisibleButtons", CleanUI_GameMenuFrame_UpdateVisibleButtons);  
end
    
function CleanUI_ChangeGameMenu()    
    for i, child in ipairs({GameMenuFrame:GetRegions()}) do
        child:Hide();
    end
    
    CleanUI_RemoveTextures(GameMenuFrame);
    CleanUI_SetBackdrop(GameMenuFrame);
    GameMenuFrame:SetSize(185, 405);
    
    CleanUI_GameMenuChangeGraphics(GameMenuFrame);
    
    CleanUI_CreateHeaderFrame(GameMenuFrame, "CUI_GameMenuFrameHeaderBorderframe", 120, nil, MAINMENU_BUTTON);
    
	local title = GetAddOnMetadata("CleanUI", "Title");
	local version = GetAddOnMetadata("CleanUI", "Version");
    
    local info = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK", "CUIGameMenuInfoTemplate");
    info:SetText("powered by |c333399ff"..title.."|r ("..version..")");
    info:SetJustifyH("RIGHT");
    info:SetSize(180, 15);
    info:ClearAllPoints(); 
    info:SetPoint("BOTTOMRIGHT", GameMenuFrame, "BOTTOMRIGHT", -5, 0);
end
    
function CleanUI_GameMenuFrame_UpdateVisibleButtons()
    GameMenuFrame:SetSize(185, 405);
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -1);
end
    
function CleanUI_ChangeStaticPopups()   
    local frame;

    for i=1, 4, 1 do
        frame = _G["StaticPopup"..i];
        CleanUI_RemoveTextures(frame);
        CleanUI_SetBackdrop(frame);    
        CleanUI_GameMenuChangeGraphics(frame);
        _G["StaticPopup"..i.."ItemFrame"]:SetSize(37, 37);
        
        CleanUI_ChangeCloseButton(frame);
        _G["StaticPopup"..i.."CloseButton"]:SetBackdrop(nil);
    end 

    CleanUI_ChangeCloseButton(ItemRefTooltip, "ItemRefCloseButton");
    
    CleanUI_ChangeCloseButton(LootHistoryFrame, nil, LootHistoryFrame.CloseButton);
    
    -- BasicScriptErrors
    BasicScriptErrors:SetScale(0.65);

    CleanUI_RemoveTextures(BasicScriptErrors);
    CleanUI_SetBackdrop(BasicScriptErrors); 
     
    CleanUI_ChangeButton(BasicScriptErrorsButton); 
    BasicScriptErrorsButton:SetHeight(22);
end
    
function CleanUI_ChangeReadyCheckFrame()
    CleanUI_RemoveTextures(ReadyCheckFrame);
    CleanUI_RemoveTextures(ReadyCheckListenerFrame);
    CleanUI_SetBackdrop(ReadyCheckListenerFrame);
    
    ReadyCheckPortrait:Hide();
    
    ReadyCheckFrameText:ClearAllPoints();
    ReadyCheckFrameText:SetPoint("TOP", ReadyCheckFrame, "TOP", 0, -15);
    
    CleanUI_ChangeButton(ReadyCheckFrameYesButton);  
    CleanUI_ChangeButton(ReadyCheckFrameNoButton);  

    ReadyCheckFrameYesButton:ClearAllPoints();
    ReadyCheckFrameYesButton:SetPoint("BOTTOMLEFT", ReadyCheckFrame, "BOTTOMLEFT", 30, 20);
    ReadyCheckFrameNoButton:ClearAllPoints();
    ReadyCheckFrameNoButton:SetPoint("BOTTOMRIGHT", ReadyCheckFrame, "BOTTOMRIGHT", -30, 20);
end
    
function CleanUI_ChangeStackSplitFrame()
    CleanUI_RemoveTextures(StackSplitFrame);
    CleanUI_SetBackdrop(StackSplitFrame);
    
    CleanUI_ChangeButton(StackSplitOkayButton); 
    CleanUI_ChangeButton(StackSplitCancelButton); 
    
    StackSplitOkayButton:SetWidth(82);
    StackSplitOkayButton:ClearAllPoints();
    StackSplitOkayButton:SetPoint("BOTTOMLEFT", StackSplitFrame, "BOTTOMLEFT", 5, 10);
    
    StackSplitCancelButton:SetWidth(82);
    StackSplitCancelButton:ClearAllPoints();
    StackSplitCancelButton:SetPoint("BOTTOMRIGHT", StackSplitFrame, "BOTTOMRIGHT", -5, 10);
end
    
function CleanUI_ChangeScriptErrorsFrame()
    ScriptErrorsFrame:SetScale(0.65);
    
    tinsert(UISpecialFrames, ScriptErrorsFrame:GetName());

    CleanUI_RemoveTextures(ScriptErrorsFrame);
    CleanUI_SetBackdrop(ScriptErrorsFrame);
    
    CleanUI_ChangeCloseButton(ScriptErrorsFrame, "ScriptErrorsFrameClose");
    
    local header = CleanUI_CreateHeaderFrame(ScriptErrorsFrame, "CUI_ScriptErrorsFrameHeaderBorderframe", 150, ScriptErrorsFrame.title, nil, 1.0, 0.2, 0.2);
    ScriptErrorsFrameTitleButton:ClearAllPoints();
    ScriptErrorsFrameTitleButton:SetAllPoints(header);
    
    CleanUI_ChangeButton(ScriptErrorsFrame.previous);  
    CleanUI_ChangeButton(ScriptErrorsFrame.next);  
    CleanUI_ChangeButtonRed(ScriptErrorsFrame.close);  
    
    ScriptErrorsFrame:ClearAllPoints();
    ScriptErrorsFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 20, 195);
   
    ScriptErrorsFrame.previous:ClearAllPoints();
    ScriptErrorsFrame.previous:SetPoint("BOTTOMLEFT", ScriptErrorsFrame, "BOTTOMLEFT", 12, 8);
    ScriptErrorsFrame.next:ClearAllPoints();
    ScriptErrorsFrame.next:SetPoint("BOTTOMLEFT", ScriptErrorsFrame, "BOTTOMLEFT", 112, 8);
    ScriptErrorsFrame.close:ClearAllPoints();
    ScriptErrorsFrame.close:SetPoint("BOTTOMRIGHT", ScriptErrorsFrame, "BOTTOMRIGHT", -12, 8);
    ScriptErrorsFrame.indexLabel:ClearAllPoints();
    ScriptErrorsFrame.indexLabel:SetPoint("BOTTOMLEFT", ScriptErrorsFrame, "BOTTOMLEFT", 208, 12);
end
    
function CleanUI_ChangeBonusRollFrame()
    CleanUI_RemoveTextures(BonusRollFrame);
    CleanUI_SetBackdrop(BonusRollFrame);
    BonusRollFrame.PromptFrame.Timer:SetFrameStrata("TOOLTIP");
end
    
function CleanUI_GameMenuChangeGraphics(frame)    

    local editBoxLeft, editBoxMid, editBoxRight;

    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then            
            if (child:IsObjectType("Button")) then
            
                if (child.Left) then
                    child.Left:Hide();
                end
                
                if (child.Right) then
                    child.Right:Hide();
                end
                
                if (child.Middle) then
                    child.Middle:Hide();
                end
                
                child:SetSize(160, 25);
            
            	if (child:GetName() == "GameMenuButtonLogout" or child:GetName() == "GameMenuButtonQuit") then
	                CleanUI_ChangeButtonRed(child);
	            else
	                CleanUI_ChangeButton(child);
	            end
            end
            
            if (child:IsObjectType("EditBox")) then
                editBoxLeft = _G[child:GetName().."Left"];
                editBoxMid = _G[child:GetName().."Mid"];
                editBoxRight = _G[child:GetName().."Right"];
                
                if (editBoxLeft) then
                    editBoxLeft:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-left");
                    editBoxLeft:SetTexCoord(0, 1, 0, 1);
                    editBoxLeft:SetSize(28, 28);
                end
                
                if (editBoxMid) then
                    editBoxMid:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-center");
                    editBoxMid:SetTexCoord(0, 1, 0, 1);
                    editBoxMid:SetHeight(28);
                end
                
                if (editBoxRight) then
                    editBoxRight:SetTexture("Interface\\AddOns\\CleanUI\\skins\\chat\\editbox-right");
                    editBoxRight:SetTexCoord(0, 1, 0, 1);
                    editBoxRight:SetSize(28, 28);
                end                
            end
        end
    end
end