function CleanUI_InitTalentFrame()  
    CleanUI_RegisterTalentFrameEvents();
end
    
function CleanUI_RegisterTalentFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED");
    
    frame:HookScript("OnEvent", CleanUI_TalentFrameOnEvent); 
end

function CleanUI_TalentFrameHookEvents()
    --hooksecurefunc("", );
end

function CleanUI_GlyphFrameHookEvents()
    hooksecurefunc("GlyphFrame_Update", CleanUI_GlyphFrame_Update);   

end

function CleanUI_TalentFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_TalentUI" ) then  
            CleanUI_TalentFrameHookEvents();
            CleanUI_CleanTalentFrame();
        end
        if ( addonName == "Blizzard_GlyphUI" ) then
            CleanUI_GlyphFrameHookEvents();
            CleanUI_CleanGlyphFrame();
        end
    end
end
    
function CleanUI_CleanTalentFrame()
    CleanUI_RemoveTextures(PlayerTalentFrame);  
    CleanUI_SetBackdrop(PlayerTalentFrame);
    
    PlayerTalentFramePortrait:Hide(); 
    
    CleanUI_RemoveTextures(PlayerTalentFrameInset);  
        
    CleanUI_ChangeCloseButton(PlayerTalentFrame);

    CleanUI_CreateHeaderFrame(PlayerTalentFrame, "CUI_PlayerTalentFrameHeaderBorderframe", 300, PlayerTalentFrameTitleText);
    
    local tabButton;
    for i=1, 10 do
        tabButton = _G["PlayerTalentFrameTab"..i];
        if (not tabButton) then
            break;
        end
        
        CleanUI_ChangeTab(tabButton);
    end
    
    CleanUI_TalentFrameChangeSideTab(PlayerSpecTab1);
    CleanUI_TalentFrameChangeSideTab(PlayerSpecTab2); 
    
    CleanUI_ChangeButton(PlayerTalentFrameActivateButton); 
    
    CleanUI_CleanTalentFrameSpecialization(); 
    CleanUI_CleanTalentFrameTalents(); 
end
    
function CleanUI_CleanTalentFrameSpecialization()
    CleanUI_RemoveTextures(PlayerTalentFrameSpecialization); 
    
    CleanUI_ChangeButton(PlayerTalentFrameSpecializationLearnButton); 
    CleanUI_ChangeButton(PlayerTalentFramePetSpecializationLearnButton); 
    PlayerTalentFrameSpecializationTutorialButton:Hide();
    
    CleanUI_ReplaceAllTextures("bluemenu-", PlayerTalentFrameSpecialization, nil);
end
    
function CleanUI_CleanTalentFrameTalents()
    CleanUI_RemoveTextures(PlayerTalentFrameTalents); 
    
	PlayerTalentFrameTalentsClearInfoFrameIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
    CleanUI_ChangeButton(PlayerTalentFrameTalentsLearnButton);     
    PlayerTalentFrameTalentsTutorialButton:Hide();
end
    
function CleanUI_CleanGlyphFrame()
    CleanUI_RemoveTextures(GlyphFrame); 
    
	GlyphFrameClearInfoFrameIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
	
	CleanUI_ChangeDropDown(GlyphFrameFilterDropDown);
	CleanUI_ChangeEditBox(GlyphFrameSearchBox, 28, -10, 5);
	
	CleanUI_RemoveTextures(GlyphFrameSideInset);
	
	GlyphFrameScrollFrameScrollBarBG:Hide();
	GlyphFrameScrollFrameScrollBarTop:Hide();
	GlyphFrameScrollFrameScrollBarBottom:Hide();
	GlyphFrameScrollFrameScrollBarMiddle:Hide();
	
	for i=1, 6 do
		_G["GlyphFrameGlyph"..i]:SetScale(0.8);
	end
	
	GlyphFrameGlyph2:ClearAllPoints();
	GlyphFrameGlyph2:SetPoint("TOPLEFT", GlyphFrame, "TOPLEFT", 75, -20);
	GlyphFrameGlyph4:ClearAllPoints();
	GlyphFrameGlyph4:SetPoint("TOP", GlyphFrameGlyph2, "BOTTOM", 0, -10);
	GlyphFrameGlyph6:ClearAllPoints();
	GlyphFrameGlyph6:SetPoint("TOP", GlyphFrameGlyph4, "BOTTOM", 0, -10);
	
	GlyphFrameGlyph1:ClearAllPoints();
	GlyphFrameGlyph1:SetPoint("TOP", GlyphFrameGlyph6, "BOTTOM", 0, -20);
	GlyphFrameGlyph3:ClearAllPoints();
	GlyphFrameGlyph3:SetPoint("TOP", GlyphFrameGlyph1, "BOTTOM", 0, -10);
	GlyphFrameGlyph5:ClearAllPoints();
	GlyphFrameGlyph5:SetPoint("TOP", GlyphFrameGlyph3, "BOTTOM", 0, -10);
	
	for i=1, 6 do
		CleanUI_GlyphFrame_UpdateInfoForIndex(i);
	end
end

function CleanUI_GlyphFrame_UpdateInfoForIndex(index)	
	local refLink = GetGlyphLink(index, PlayerTalentFrame.talentGroup);
	local name, glyphType, isKnown, icon, glyphID, link, subText;
	
	local numGlyphs = GetNumGlyphs();
	for i = 1, numGlyphs do
		name, glyphType, isKnown, icon, glyphID, link, subText = GetGlyphInfo(i);
		if (refLink == link) then
			CleanUI_GlyphFrame_CreateOrUpdateTextInfo(index, name, subText)
		end
	end
end

function CleanUI_GlyphFrame_CreateOrUpdateTextInfo(index, name, subText)
	local parent = _G["GlyphFrameGlyph"..index];
	
	local header = _G["GlyphFrameGlyph"..index.."Header"];
	if (not header) then
		header = parent:CreateFontString("GlyphFrameGlyph"..index.."Header", "ARTWORK");
		header:SetFontObject("GameFontNormalLargeLeft");
    	header:SetJustifyH("LEFT");
	    header:SetWidth(350);
	    header:SetHeight(15);
	    header:SetTextColor(1.0, 1.0, 1.0, 1.0);
		header:SetPoint("TOPLEFT", parent, "TOPRIGHT", 35, -18);
    end	
    
	header:SetText(name);
	
	local info = _G["GlyphFrameGlyph"..index.."Info"];
	if (not info) then
		info = parent:CreateFontString("GlyphFrameGlyph"..index.."Info", "ARTWORK");
		info:SetFontObject("GameFontNormalSmall");
    	info:SetJustifyH("LEFT");
	    info:SetWidth(350);
	    info:SetHeight(15);
	    info:SetTextColor(1.0, 1.0, 0.6, 1.0);
		info:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 0, 0);
    end	
    
	info:SetText(subText);
end

function CleanUI_GlyphFrame_Update(self)
	GlyphFrame.specIcon:Hide();
	
	for i=1, 6 do
		CleanUI_GlyphFrame_UpdateInfoForIndex(i);
	end
end

function CleanUI_TalentFrameChangeSideTab(tabButton)    
    local name = tabButton:GetName();
    
    _G[name.."Background"]:Hide();
    
    local borderframe;
    
    if (name) then
	    borderframe = _G[name.."Borderframe"];
	    if (not borderframe) then
	        borderframe = CreateFrame("Frame", name.."Borderframe", tabButton);
	    end
    else
    	borderframe = CreateFrame("Frame");
    	borderframe:SetParent(tabButton);
    end
    
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -4, 4);
    borderframe:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 3, -4);
    
    borderframe:SetFrameStrata("LOW");
    
    borderframe:SetBackdrop( { 
        bgFile = CUI_TAB_BACKGROUND_RIGHT, 
        edgeFile = border, tile = true, tileSize = 64, edgeSize = 16, 
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    }); 
    
    tabButton:GetNormalTexture():SetTexCoord(0.08, 0.92, 0.08, 0.92);
    
    local checkedTexture = tabButton:GetCheckedTexture();
    checkedTexture:ClearAllPoints();
    checkedTexture:SetPoint("TOPLEFT", borderframe, "TOPLEFT", 0, 0);
    checkedTexture:SetPoint("BOTTOMRIGHT", borderframe, "BOTTOMRIGHT", 0, 0);
    tabButton:SetCheckedTexture("Interface\\AddOns\\CleanUI\\skins\\highlight");
end
