function CleanUI_InitTradeFrame()
    CleanUI_TradeFrameHookEvents();
    CleanUI_RegisterTradeFrameEvents();
    
    -- TradeFrame stylen
    CleanUI_CleanTradeFrame();
end
    
function CleanUI_RegisterTradeFrameEvents()
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("TRADE_SHOW"); 
    frame:RegisterEvent("TRADE_UPDATE"); 
    
    frame:RegisterEvent("TRADE_TARGET_ITEM_CHANGED"); 
    frame:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED"); 
    
    frame:HookScript("OnEvent", CleanUI_TradeFrameOnEvent); 
end

function CleanUI_TradeFrameOnEvent(self, event, ...)
    local arg1, arg2 = ...;
    if ( event == "TRADE_TARGET_ITEM_CHANGED" ) then
        CleanUI_TradeFrame_UpdateTargetItem(arg1);
    end
    if ( event == "TRADE_PLAYER_ITEM_CHANGED" ) then
        CleanUI_TradeFrame_UpdatePlayerItem(arg1);
    end
    if ( event == "TRADE_SHOW" or event == "TRADE_UPDATE" ) then
        CleanUI_TradeFrame_ReinitAllItems();
    end
end

function CleanUI_TradeFrameHookEvents()
    --hooksecurefunc("orig", Hook);
end

function CleanUI_CleanTradeFrame()
    CleanUI_RemoveTextures(TradeFrame);
    CleanUI_RemoveTextures(TradeFrameInset);
      
    local borderframePlayer = _G["CUI_TradeFramePlayerBorderframe"];
    if (not borderframePlayer) then
        borderframePlayer = CreateFrame("Frame", "CUI_TradeFramePlayerBorderframe", TradeFrame);
        
        CleanUI_SetBackdrop(borderframePlayer);

        borderframePlayer:SetSize(10, 10);        

        borderframePlayer:ClearAllPoints();
        borderframePlayer:SetPoint("TOPLEFT", TradeFrame, "TOPLEFT", 0, 0);
        borderframePlayer:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOM", 0, 0);
        
        borderframePlayer:SetFrameStrata("LOW");
    end
      
    local borderframeRecipient = _G["CUI_TradeFrameRecipientBorderframe"];
    if (not borderframeRecipient) then
        borderframeRecipient = CreateFrame("Frame", "CUI_TradeFrameRecipientBorderframe", TradeFrame);
        
        CleanUI_SetBackdrop(borderframeRecipient);

        borderframeRecipient:SetSize(10, 10);        

        borderframeRecipient:ClearAllPoints();
        borderframeRecipient:SetPoint("TOPLEFT", TradeFrame, "TOP", 0, 0);
        borderframeRecipient:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 0);
        
        borderframeRecipient:SetFrameStrata("LOW");
    end
    
    TradeFramePlayerPortrait:Hide();
    TradeFrameRecipientPortrait:Hide();
     
    CleanUI_CreateHeaderFrame(borderframePlayer, "CUI_TradeFramePlayerHeaderBorderframe", 110, TradeFramePlayerNameText);
    CleanUI_CreateHeaderFrame(borderframeRecipient, "CUI_TradeFrameRecipientHeaderBorderframe", 110, TradeFrameRecipientNameText);

    CleanUI_ChangeCloseButton(TradeFrame);
    
    CleanUI_RemoveTextures(TradePlayerItemsInset);
    CleanUI_RemoveTextures(TradeRecipientItemsInset);
    
    CleanUI_RemoveTextures(TradePlayerEnchantInset);
    CleanUI_RemoveTextures(TradeRecipientEnchantInset);
    
    CleanUI_RemoveTextures(TradeRecipientMoneyInset);
    CleanUI_RemoveTextures(TradeRecipientMoneyBg);

    CleanUI_ChangeButton(TradeFrameTradeButton);  
    CleanUI_ChangeButtonRed(TradeFrameCancelButton);  
    
    TradeHighlightPlayer:ClearAllPoints();
    TradeHighlightPlayer:SetPoint("TOPLEFT", TradeFrame, "TOPLEFT", 5, -57);
    TradeHighlightPlayer:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOM", 0, 93);
    
    TradeHighlightRecipient:ClearAllPoints();
    TradeHighlightRecipient:SetPoint("TOPLEFT", TradeFrame, "TOP", 5, -57);
    TradeHighlightRecipient:SetPoint("BOTTOMRIGHT", TradeFrame, "BOTTOMRIGHT", 0, 93);
    
    CleanUI_RemoveTextures(TradePlayerInputMoneyInset);
    
    CleanUI_ChangeEditBox(TradePlayerInputMoneyFrameGold, 28, -10, 5);
    CleanUI_ChangeEditBox(TradePlayerInputMoneyFrameSilver, 28, -10, -5);
    CleanUI_ChangeEditBox(TradePlayerInputMoneyFrameCopper, 28, -10, -5);
    
    for i=1, MAX_TRADE_ITEMS do
        _G["TradePlayerItem"..i.."ItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
        _G["TradeRecipientItem"..i.."ItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
        _G["TradePlayerItem"..i.."ItemButtonNormalTexture"]:SetVertexColor(0.5, 0.5, 0.5);
        _G["TradeRecipientItem"..i.."ItemButtonNormalTexture"]:SetVertexColor(0.5, 0.5, 0.5);
        
        _G["TradePlayerItem"..i.."SlotTexture"]:Hide();
        _G["TradeRecipientItem"..i.."SlotTexture"]:Hide();
        
        _G["TradePlayerItem"..i.."NameFrame"]:Hide();
        _G["TradeRecipientItem"..i.."NameFrame"]:Hide();
    end
end

function CleanUI_TradeFrame_ReinitAllItems()
    for i=1, MAX_TRADE_ITEMS, 1 do
        CleanUI_TradeFrame_UpdatePlayerItem(i);
        CleanUI_TradeFrame_UpdateTargetItem(i);
    end
end

function CleanUI_TradeFrame_UpdatePlayerItem(id)
    local name, texture, numItems, quality, isUsable, enchantment = GetTradePlayerItemInfo(id);
    local button = _G["TradePlayerItem"..id];
    CleanUI_TradeFrame_UpdateItem(button, name);
end

function CleanUI_TradeFrame_UpdateTargetItem(id)
    local name, texture, numItems, quality, isUsable, enchantment = GetTradeTargetItemInfo(id);
    local button = _G["TradeRecipientItem"..id];
    CleanUI_TradeFrame_UpdateItem(button, name);
end

function CleanUI_TradeFrame_UpdateItem(button, name)
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;

    texture = _G[button:GetName().."ItemButtonNormalTexture"];
    
    local r, g, b = 0.5, 0.5, 0.5;
    
    if (name) then
        itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(name);
        
        texture:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border"); 
    
        if (itemRarity) then
            r, g, b = GetItemQualityColor(itemRarity);
            
            if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY == 1) then
                r, g, b = 0.5, 0.5, 0.5;
            end
        end
    end
        
    texture:SetVertexColor(r, g, b);
end













