local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local UNIT_FRAME_WIDTH = 232;
local UNIT_FRAME_HEIGHT = 55;

local PLAYER_BAROFFSET = 56;

local BORDER_SIZE = 4;

local PORTRAIT_HEIGHT = UNIT_FRAME_HEIGHT - BORDER_SIZE * 2;
local PORTRAIT_WIDTH = PORTRAIT_HEIGHT;
local TARGET_PORTRAIT_OFFSET = UNIT_FRAME_WIDTH - UNIT_FRAME_HEIGHT + BORDER_SIZE;

local BARFRAME_WIDTH = 181;
local BARS_WIDTH = BARFRAME_WIDTH - 9;

local HEALTHBAR_YPOS_OFFSET = -5;
local HEALTHBAR_HEIGHT = 30;

local MANABAR_HEIGHT = 16;
local ALTERNATE_MANABAR_HEIGHT = 6;

local PET_YPOS_OFFSET = 51;

-- portrait camera
local camera = 0;
local portraitZoom = 1;

-- no animations
PlayerFrame_AnimateOut = function (self) CleanUI_DoNothing(); end;
PlayerFrame_UpdateArt = function (self) CleanUI_DoNothing(); end;
TargetFrame_CheckClassification = function (self) CleanUI_DoNothing(); end;
TotemFrame_AdjustPetFrame = function (self) CleanUI_DoNothing(); end;

function CleanUI_DoNothing()
end

function CleanUI_InitUnitFrames()
    if (CleanUIData.CHANGE_UNITFRAMES ~= 1) then
        return;
    end
    
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI");
    end

    CleanUI_InitUnitFrameUI();
    
    CleanUI_ColorizeRuneFrame();
    CleanUI_MoveComboFrameIcons();
    
    CleanUI_MakeFramesMovable();
    
    CleanUI_MovePlayerIcons();
    CleanUI_MovePartyIcons();
    CleanUI_MovePetIcons();
    CleanUI_MoveTargetIcons();
    CleanUI_MoveFocusIcons();
    CleanUI_CheckClassColors();    
    
    CleanUI_InitUnitTextures();
    CleanUI_UpdateShapeshiftForm();
    CleanUI_InitPortrait3DModels();
    CleanUI_InitPortraitClassIcons();
    CleanUI_SetPortraitTexture(PlayerPortrait, "player");
    
    if (CleanUIData.CHANGE_BOSSFRAMES == 1) then 
        CleanUI_BossTargetIcons(_G["Boss1TargetFrame"]);
        CleanUI_BossTargetIcons(_G["Boss2TargetFrame"]);    
        CleanUI_BossTargetIcons(_G["Boss3TargetFrame"]);
        CleanUI_BossTargetIcons(_G["Boss4TargetFrame"]);
        CleanUI_BossTargetIcons(_G["Boss5TargetFrame"]);
    end
  
    CleanUI_SetBackdrop(PartyMemberBuffTooltip);
    
    cui_unitframes_loaded = true;
    
    CleanUI_UnitFramesHookEvents();
    CleanUI_RegisterUnitFramesEvents();
    
    CleanUI_UpdateLootSpecialization();
end
    
function CleanUI_RegisterUnitFramesEvents()         
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED"); 
    
    frame:RegisterEvent("GROUP_ROSTER_UPDATE");  
    frame:RegisterEvent("PARTY_MEMBERS_CHANGED");  
    frame:RegisterEvent("RAID_ROSTER_UPDATE");    
    frame:RegisterEvent("UNIT_AURA");  
    frame:RegisterEvent("UNIT_COMBAT");  
    frame:RegisterEvent("UNIT_PORTRAIT_UPDATE");  
    frame:RegisterEvent("UPDATE_SHAPESHIFT_FORM");  
    frame:RegisterEvent("UPDATE_SHAPESHIFT_USABLE");  
    frame:RegisterEvent("PLAYER_REGEN_ENABLED");  
    frame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");  
    frame:RegisterEvent("PLAYER_TARGET_CHANGED");  
    frame:RegisterEvent("PLAYER_FOCUS_CHANGED");  
    
    frame:HookScript("OnEvent", CleanUI_UnitFramesOnEvent); 
    hooksecurefunc("TargetFrame_CheckFaction", CleanUI_TargetFrame_CheckFaction);  
end

function CleanUI_UnitFramesOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_ArenaUI" ) then
            CleanUI_CleanArenaFrames();
        end
    end
    if (event == "GROUP_ROSTER_UPDATE" or event == "PARTY_MEMBERS_CHANGED" or event == "RAID_ROSTER_UPDATE") then
        CleanUI_CheckClassColors();
        CleanUI_InitUnitTextures();
        CleanUI_RebindUnitFrames();
    end
    if (event == "UNIT_AURA" or event == "UNIT_COMBAT") then
        CleanUI_ColorizeRuneFrame();
        CleanUI_UpdateShapeshiftForm();
    end
    if (event == "UNIT_PORTRAIT_UPDATE") then
        local unit = ...;
 	    CleanUI_UpdatePortrait(unit);
    end
    if (event == "UPDATE_SHAPESHIFT_FORM" or event == "UPDATE_SHAPESHIFT_USABLE") then
        CleanUI_UpdateShapeshiftForm();
    end
    if (event == "PLAYER_REGEN_ENABLED") then
        CleanUI_RebindUnitFrames();
    end
    if (event == "ACTIVE_TALENT_GROUP_CHANGED") then
        CleanUI_PlayerFrame_UpdateArt();
    end
    if (event == "PLAYER_TARGET_CHANGED") then
        CleanUI_UpdateTargetInfo();
    end
    if (event == "PLAYER_FOCUS_CHANGED") then
        CleanUI_UpdateFocusInfo();
    end    
    
	CleanUI_InitUnitDrawLayer();
end

function CleanUI_RebindUnitFrames()
    if (InCombatLockdown()) then
        return;
    end
	
	if (UnitExists("pet")) then
	    UnitFrame_SetUnit(PetFrame, "pet", PetFrameHealthBar, PetFrameManaBar);
	    PetFrame:Show();
	else
	    PetFrame:Hide();
	end
    
    if (IsInRaid()) then
        for i = 1, 4 do
            _G["PartyMemberFrame"..i]:Hide();
            _G["PartyMemberFrame"..i.."PetFrame"]:Hide();
        end
    else
        for i = 1, 4 do
            if (UnitExists("party"..i)) then
                UnitFrame_SetUnit(_G["PartyMemberFrame"..i], "party"..i, _G["PartyMemberFrame"..i.."HealthBar"], _G["PartyMemberFrame"..i.."ManaBar"]);
                _G["PartyMemberFrame"..i]:Show();
            else
                _G["PartyMemberFrame"..i]:Hide();
            end
            if (UnitExists("partypet"..i)) then
                UnitFrame_SetUnit(_G["PartyMemberFrame"..i.."PetFrame"], "partypet"..i, _G["PartyMemberFrame"..i.."PetFrameHealthBar"], nil);
                _G["PartyMemberFrame"..i.."PetFrame"]:Show();
            else
                _G["PartyMemberFrame"..i.."PetFrame"]:Hide();
            end
        end
    end
end

function CleanUI_UnitFramesHookEvents()
    hooksecurefunc("UnitFrame_UpdateTooltip", CleanUI_UnitFrame_UpdateTooltip);
    hooksecurefunc("PlayerFrame_ToPlayerArt", CleanUI_PlayerFrame_UpdateArt);
    hooksecurefunc("PlayerFrame_ToVehicleArt", CleanUI_PlayerFrame_UpdateArt);
    hooksecurefunc("PartyMemberFrame_ToPlayerArt", CleanUI_PartyMemberFrame_UpdateArt);
    hooksecurefunc("PartyMemberFrame_ToVehicleArt", CleanUI_PartyMemberFrame_UpdateArt);
    hooksecurefunc("PartyMemberFrame_UpdatePet", CleanUI_PartyMemberFrame_UpdatePet);
    hooksecurefunc("SetPortraitTexture", CleanUI_SetPortraitTexture);
    hooksecurefunc("BossTargetFrame_OnLoad", CleanUI_BossTargetFrame_OnLoad);
    hooksecurefunc("TargetFrame_UpdateAuras", CleanUI_TargetFrame_UpdateAuras);
    
    hooksecurefunc("ShardBar_Update", CleanUI_ShardBar_Update);
    
    hooksecurefunc("UnitPopup_ShowMenu", CleanUI_UnitPopup_AppendUnitFramePositions);
    hooksecurefunc("UnitPopup_OnClick", CleanUI_UnitPopup_OnClick); 
    
	hooksecurefunc("MonkHarmonyBar_Update", CleanUI_MonkHarmonyBar_Update); 
	
	hooksecurefunc("PlayerFrame_ResetUserPlacedPosition", CleanUIConfig_LoadDefaultUnitFramePositions); 
	hooksecurefunc("TargetFrame_ResetUserPlacedPosition", CleanUIConfig_LoadDefaultUnitFramePositions); 
	
	hooksecurefunc("SetLootSpecialization", CleanUI_SetLootSpecialization); 	
end

function CleanUI_UpdateTargetInfo()
    if (not cui_unitframes_loaded) then
        return;
    end

    local targetClassification = UnitClassification("target");
    
    if (targetClassification == "worldboss") then
        CUI_TargetFrameWorldBossTexture:Show();
        CUI_TargetFrameEliteTexture:Hide();
        CUI_TargetFrameRareTexture:Hide();
        TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
    elseif (targetClassification == "rareelite" or targetClassification == "elite") then
        CUI_TargetFrameWorldBossTexture:Hide();
        CUI_TargetFrameEliteTexture:Show();
        CUI_TargetFrameRareTexture:Hide();
        TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
    elseif (targetClassification == "rare") then
        CUI_TargetFrameWorldBossTexture:Hide();
        CUI_TargetFrameEliteTexture:Hide();
        CUI_TargetFrameRareTexture:Show();
        TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
        CUI_TargetManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0);
    else
        CUI_TargetFrameWorldBossTexture:Hide();
        CUI_TargetFrameEliteTexture:Hide();
        CUI_TargetFrameRareTexture:Hide();
        TargetFrameTextureFrameHealthBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
        TargetFrameTextureFrameManaBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
        CUI_TargetHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
        CUI_TargetManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    end
    
    if (CUI_TargetInfo) then
        local creatureType = UnitCreatureType("target");
        local creatureFamily = UnitCreatureFamily("target");
        local localizedClass, englishClass = UnitClass("target");
        
        local classColor = RAID_CLASS_COLORS[englishClass];

        if (localizedClass == UnitName("target")) then
            localizedClass = nil;
        end
        
        local targetinfo = "";
         
        if (creatureType) then
            targetinfo = creatureType;
        end
        
        if (localizedClass and localizedClass ~= "") then
            local classInfo = format("|cff%.2x%.2x%.2x%s|r", classColor.r * 255, classColor.g * 255, classColor.b * 255, localizedClass);
            targetinfo = targetinfo..", "..classInfo;
        end
        
        if (creatureFamily and creatureFamily ~= "") then
            targetinfo = targetinfo..", "..creatureFamily;
        end
        
        CUI_TargetInfo:SetText(targetinfo);
    end
end

function CleanUI_UpdateFocusInfo()    
    if (not cui_unitframes_loaded) then
        return;
    end

    local focusClassification = UnitClassification("focus");
    
    if (focusClassification == "worldboss") then
        CUI_FocusFrameWorldBossTexture:Show();
        CUI_FocusFrameEliteTexture:Hide();
        CUI_FocusFrameRareTexture:Hide();
        FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
    elseif (focusClassification == "rareelite" or focusClassification == "elite") then
        CUI_FocusFrameWorldBossTexture:Hide();
        CUI_FocusFrameEliteTexture:Show();
        CUI_FocusFrameRareTexture:Hide();
        FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
    elseif (focusClassification == "rare") then
        CUI_FocusFrameWorldBossTexture:Hide();
        CUI_FocusFrameEliteTexture:Hide();
        CUI_FocusFrameRareTexture:Show();
        FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusHealthBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
        CUI_FocusManaBarPercent:SetTextColor(1.0, 0.9, 0.0, 1.0); 
    else
        CUI_FocusFrameWorldBossTexture:Hide();
        CUI_FocusFrameEliteTexture:Hide();
        CUI_FocusFrameRareTexture:Hide();
        FocusFrameTextureFrameHealthBarText:SetTextColor(1.0, 1.0, 1.0, 1.0); 
        FocusFrameTextureFrameManaBarText:SetTextColor(1.0, 1.0, 1.0, 1.0);
        CUI_FocusHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
        CUI_FocusManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    end
    
    if (CUI_FocusInfo) then
        local creatureType = UnitCreatureType("focus");
        local creatureFamily = UnitCreatureFamily("focus");
        local localizedClass, englishClass = UnitClass("focus");
        
        local classColor = RAID_CLASS_COLORS[englishClass];
        
        if (localizedClass == UnitName("focus")) then
            localizedClass = nil;
        end
         
        local focusinfo = "";
         
        if (creatureType) then
            focusinfo = creatureType;
        end
        
        if (localizedClass and localizedClass ~= "") then
            local classInfo = format("|cff%.2x%.2x%.2x%s|r", classColor.r * 255, classColor.g * 255, classColor.b * 255, localizedClass);
            focusinfo = focusinfo..", "..classInfo;
        end
        
        if (creatureFamily and creatureFamily ~= "") then
            focusinfo = focusinfo..", "..creatureFamily;
        end
                
        CUI_FocusInfo:SetText(focusinfo);
    end
end

local UPDATE_WAIT = 0.1;
local _WaitTimeSinceLast = 0;
function CleanUI_UnitFramesOnUpdate(self, elapsed)
    CleanUI_UpdatePaladinPowerBar(elapsed);
    
    if (_WaitTimeSinceLast <= 0) then
        CleanUI_UnitFramesUpdate(self, elapsed);
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_UnitFramesUpdate(self, elapsed)
    CleanUI_UpdateBackdrop();
end

function CleanUI_TargetFrame_CheckFaction(self)
	if (not self or not self.tappedInfo) then
	    return;
	end

	if ( not UnitPlayerControlled(self.unit) and UnitIsTapped(self.unit) and not UnitIsTappedByPlayer(self.unit) and not UnitIsTappedByAllThreatList(self.unit) ) then
		self.tappedInfo:SetText("tapped");
	else
		self.tappedInfo:SetText("");
	end
end

function CleanUI_UpdateBackdrop()
    local playerborder = _G["CUI_PlayerFrameBorder1"];
    if (playerborder) then
        local isSelfCast = false;
        
        local modifiedClick = GetModifiedClick("SELFCAST");
        if (modifiedClick == "ALT" and IsAltKeyDown()) then
            isSelfCast = true;                
        elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
            isSelfCast = true;                
        elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
            isSelfCast = true;                
        end
        
        if (isSelfCast) then
            playerborder:SetBackdropBorderColor(1.0, 0.0, 0.0, 1.0);
        else
            playerborder:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
        end
    end

	local focusborder = _G["CUI_FocusFrameBorder2"];
    if (focusborder) then
        local isFocusCast = false;
        
        local modifiedClick = GetModifiedClick("FOCUSCAST");
        if (modifiedClick == "ALT" and IsAltKeyDown()) then
            isFocusCast = true;                
        elseif (modifiedClick == "CTRL" and IsControlKeyDown()) then
            isFocusCast = true;                
        elseif (modifiedClick == "SHIFT" and IsShiftKeyDown()) then
            isFocusCast = true;                
        end
        
        if (isFocusCast) then
            focusborder:SetBackdropBorderColor(1.0, 0.0, 0.0, 1.0);
        else
            focusborder:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
        end
    end
end

function CleanUI_TargetFrame_UpdateAuras(self)  
    -- Masque
    if (MASQUE and buttongroup) then 	
        for i=1, MAX_TARGET_BUFFS do
            local button = _G[self:GetName().."Buff"..i];
            
            if (button) then                
                buttongroup:RemoveButton(button);
                buttongroup:AddButton(button);
            end
        end
        
        for i=1, MAX_TARGET_DEBUFFS do
            local button = _G[self:GetName().."Debuff"..i];
            
            if (button) then                
                buttongroup:RemoveButton(button);
                buttongroup:AddButton(button);
            end
        end
    end
    
    -- Stealable
    for i=1, MAX_TARGET_BUFFS do	
		frameName = self:GetName().."Buff"..i;
        frameStealable = _G[frameName.."Stealable"];
        
        if (frameStealable) then        
            frameStealable:ClearAllPoints();
            frameStealable:SetPoint("TOPLEFT", _G[frameName], "TOPLEFT", -4, 4);
            frameStealable:SetPoint("BOTTOMRIGHT", _G[frameName], "BOTTOMRIGHT", 4, -4);
        end
    end
end

function CleanUI_MakeFramesMovable()
    PlayerFrame:SetMovable(true);
    TargetFrame:SetMovable(true);
    FocusFrame:SetMovable(true);
    
    PlayerFrame:HookScript("OnMouseDown", CleanUI_UnitFrame_OnMouseDown);
	PlayerFrame:HookScript("OnMouseUp", CleanUI_UnitFrame_OnMouseUp);
    TargetFrame:HookScript("OnMouseDown", CleanUI_UnitFrame_OnMouseDown);
	TargetFrame:HookScript("OnMouseUp", CleanUI_UnitFrame_OnMouseUp);
    FocusFrame:HookScript("OnMouseDown", CleanUI_UnitFrame_OnMouseDown);
	FocusFrame:HookScript("OnMouseUp", CleanUI_UnitFrame_OnMouseUp);
	
	if (not CleanUIData.unitFramePositions) then
        CleanUIData.unitFramePositions = {};
        CleanUIData.unitFramePositions["PlayerFrame"] = nil;
        CleanUIData.unitFramePositions["TargetFrame"] = nil;
        CleanUIData.unitFramePositions["FocusFrame"] = nil;
	end
end

function CleanUI_LoadFramePosition(self, def_x, def_y)

    if (InCombatLockdown()) then
        return;
    end

    local name = self:GetName();
    
    local position = CleanUIData.unitFramePositions[name];
    
    if (position) then
        local point = position.p;
        local relativeTo = _G[position.rt];
        local relativePoint = position.rp;
        local x = position.x;
        local y = position.y;
        
        if (not point) then
            point = "TOPLEFT";
        end
        
        if (not relativeTo) then
            relativeTo = UIParent;
        end
        
        if (not relativePoint) then
            relativePoint = "TOPLEFT";
        end
        
        self:ClearAllPoints();
        self:SetPoint(point, relativeTo, relativePoint, x, y);
    else
        self:ClearAllPoints();
        self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", def_x, def_y);
        
        CleanUI_SaveFramePosition(self, "TOPLEFT", UIParent, "TOPLEFT", def_x, def_y)
    end
end

function CleanUI_SaveFramePosition(self, point, relativeTo, relativePoint, posx, posy)

    local name = self:GetName();
    
    if (not relativeTo) then
        relativeTo = UIParent;
    end
    
    CleanUIData.unitFramePositions[name] = { p = point, rt = relativeTo:GetName(), rp = relativePoint, x = posx, y = posy };
    
    if (InCombatLockdown() == nil) then
        self:ClearAllPoints();
        self:SetPoint(point, relativeTo, relativePoint, posx, posy);
    end
end

function CleanUI_UnitFrame_OnMouseDown(self, button)
    if (InCombatLockdown()) then
        self:StopMovingOrSizing();
        return;
    end

    if (button == "RightButton") then
        if (IsControlKeyDown()) then
            self:StartMoving();
        end
    end
end

function CleanUI_UnitFrame_OnMouseUp(self, button)
    self:StopMovingOrSizing();

    if (InCombatLockdown()) then
        return;
    end
    
    if (button == "RightButton") then        
        local point, relativeTo, relativePoint, x, y = self:GetPoint(); 
               
	    x = ceil(x);
	    y = ceil(y);
    
        CleanUI_SaveFramePosition(self, point, relativeTo, relativePoint, x, y);
    end
end

function CleanUI_InitPortrait3DModels()
    local playerModelFrame = CreateFrame("PlayerModel", "CUI_PlayerModelFrame", PlayerFrame);
    playerModelFrame:SetWidth(PORTRAIT_WIDTH);
    playerModelFrame:SetHeight(PORTRAIT_HEIGHT);
    playerModelFrame:ClearAllPoints();
    playerModelFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 4, -4);
    playerModelFrame:SetFrameStrata("BACKGROUND");
    playerModelFrame:SetScript("OnShow", CleanUI_InitPortraitCamera);
    playerModelFrame:EnableMouse(false);
	
	local targetModelFrame = CreateFrame("PlayerModel", "CUI_TargetModelFrame", TargetFrame);
    targetModelFrame:SetWidth(PORTRAIT_WIDTH);
    targetModelFrame:SetHeight(PORTRAIT_HEIGHT);
    targetModelFrame:ClearAllPoints();
    targetModelFrame:SetPoint("TOPLEFT", TargetFrame, "TOPLEFT", TARGET_PORTRAIT_OFFSET, -4);
    targetModelFrame:SetFrameStrata("BACKGROUND");
    targetModelFrame:SetScript("OnShow", CleanUI_InitPortraitCamera);
    targetModelFrame:EnableMouse(false); 
	
	local focusModelFrame = CreateFrame("PlayerModel", "CUI_FocusModelFrame", FocusFrame);
    focusModelFrame:SetWidth(PORTRAIT_WIDTH);
    focusModelFrame:SetHeight(PORTRAIT_HEIGHT);
    focusModelFrame:ClearAllPoints();
    focusModelFrame:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", TARGET_PORTRAIT_OFFSET, -4);
    focusModelFrame:SetFrameStrata("BACKGROUND");
    focusModelFrame:SetScript("OnShow", CleanUI_InitPortraitCamera);
    targetModelFrame:EnableMouse(false); 
	
    for i=1, 4, 1 do
    
        local partyFrame = _G["PartyMemberFrame"..i];
            
        local partyModelFrame = CreateFrame("PlayerModel", "CUI_Party"..i.."ModelFrame", partyFrame);
        partyModelFrame:SetWidth(40);
        partyModelFrame:SetHeight(40);
        partyModelFrame:ClearAllPoints();
        partyModelFrame:SetPoint("TOPLEFT", partyFrame, "TOPLEFT", 4, -4);
        partyModelFrame:SetFrameStrata("LOW");
        partyModelFrame:SetScript("OnShow", CleanUI_InitPortraitCamera);
        partyModelFrame:EnableMouse(false); 
        
        local help = _G["CUI_Party"..i.."BGFrame"];
        if (not help) then
            help = CreateFrame("Frame", "CUI_Party"..i.."BGFrame", partyFrame);
            CleanUI_SetBackdropWithoutBorder(help);
        end
        help:SetWidth(48);
        help:SetHeight(48);
        help:ClearAllPoints();
        help:SetPoint("TOPLEFT", partyFrame, "TOPLEFT", 0, 0);
    end
end

function CleanUI_InitPortraitCamera(self)
	if (self) then
		self:SetCamera(camera);
		self:SetPortraitZoom(portraitZoom);
	end
end

function CleanUI_InitPortraitClassIcons()
    local playerModelFrame = CreateFrame("Frame", "CUI_PlayerClassIcon", PlayerFrame);
    playerModelFrame:SetWidth(PORTRAIT_WIDTH);
    playerModelFrame:SetHeight(PORTRAIT_HEIGHT);
    playerModelFrame:ClearAllPoints();
    playerModelFrame:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 4, -4);
    playerModelFrame:SetFrameStrata("BACKGROUND");
    
	local playerClassIcon = playerModelFrame:CreateTexture(playerModelFrame:GetName().."Icon", "ARTWORK");
    playerClassIcon:ClearAllPoints();
    playerClassIcon:SetPoint("TOPLEFT", 0, 0);
    playerClassIcon:SetPoint("BOTTOMRIGHT", 0, 0);
    
	
    local targetModelFrame = CreateFrame("Frame", "CUI_TargetClassIcon", TargetFrame);
    targetModelFrame:SetWidth(PORTRAIT_WIDTH);
    targetModelFrame:SetHeight(PORTRAIT_HEIGHT);
    targetModelFrame:ClearAllPoints();
    targetModelFrame:SetPoint("TOPLEFT", TargetFrame, "TOPLEFT", TARGET_PORTRAIT_OFFSET, -4);
    targetModelFrame:SetFrameStrata("BACKGROUND");
    
	local targetClassIcon = targetModelFrame:CreateTexture(targetModelFrame:GetName().."Icon", "ARTWORK");
    targetClassIcon:ClearAllPoints();
    targetClassIcon:SetPoint("TOPLEFT", 0, 0);
    targetClassIcon:SetPoint("BOTTOMRIGHT", 0, 0);
    
	
    local focusModelFrame = CreateFrame("Frame", "CUI_FocusClassIcon", FocusFrame);
    focusModelFrame:SetWidth(PORTRAIT_WIDTH);
    focusModelFrame:SetHeight(PORTRAIT_HEIGHT);
    focusModelFrame:ClearAllPoints();
    focusModelFrame:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", TARGET_PORTRAIT_OFFSET, -4);
    focusModelFrame:SetFrameStrata("BACKGROUND");
    
	local focusClassIcon = focusModelFrame:CreateTexture(focusModelFrame:GetName().."Icon", "ARTWORK");
    focusClassIcon:ClearAllPoints();
    focusClassIcon:SetPoint("TOPLEFT", 0, 0);
    focusClassIcon:SetPoint("BOTTOMRIGHT", 0, 0);
	
    for i=1, 4, 1 do
    
        local partyFrame = _G["PartyMemberFrame"..i];
	
        local partyModelFrame = CreateFrame("Frame", "CUI_Party"..i.."ClassIcon", partyFrame);
        partyModelFrame:SetWidth(40);
        partyModelFrame:SetHeight(40);
        partyModelFrame:ClearAllPoints();
        partyModelFrame:SetPoint("TOPLEFT", partyFrame, "TOPLEFT", 4, -4);
        partyModelFrame:SetFrameStrata("LOW");
        
        local partyClassIcon = partyModelFrame:CreateTexture(partyModelFrame:GetName().."Icon", "ARTWORK");
        partyClassIcon:ClearAllPoints();
        partyClassIcon:SetPoint("TOPLEFT", 0, 0);
        partyClassIcon:SetPoint("BOTTOMRIGHT", 0, 0);
    end
end

function CleanUI_InitPortraitType(modelFrame, classiconFrame, unit)
    if (CleanUIData.PORTRAIT_STYLE == 1) then       -- 3D portrait
        if (modelFrame) then
            modelFrame:Show();
            
            modelFrame:ClearModel();
            modelFrame:SetUnit(unit);
            modelFrame:SetCamera(camera);
			modelFrame:SetPortraitZoom(portraitZoom);
        end
        if (classiconFrame) then
            classiconFrame:Hide();
        end
    elseif (CleanUIData.PORTRAIT_STYLE == 2) then   -- class icon
        if (classiconFrame) then
            classiconFrame:Show();
            
            local _, classToken = UnitClass(unit);
            if (classToken) then
                local classicon = _G[classiconFrame:GetName().."Icon"];
                local coords = CLASS_BUTTONS[classToken];
                classicon:SetTexture("Interface\\WorldStateFrame\\Icons-Classes");
                classicon:SetTexCoord(coords[1], coords[2], coords[3], coords[4]);
            end
        end
        if (modelFrame) then
            modelFrame:Hide();
        end
    end
end

function CleanUI_UpdateAllPortraits()
    CleanUI_UpdatePortrait("player");
    CleanUI_UpdatePortrait("target");
    CleanUI_UpdatePortrait("focus");
    CleanUI_UpdatePortrait("party1");
    CleanUI_UpdatePortrait("party2");
    CleanUI_UpdatePortrait("party3");
    CleanUI_UpdatePortrait("party4");
end

function CleanUI_UpdatePortrait(unit)
    if (unit == "player") then
        CleanUI_InitPortraitType(_G["CUI_PlayerModelFrame"], _G["CUI_PlayerClassIcon"], unit);
    elseif (unit == "target") then
        CleanUI_InitPortraitType(_G["CUI_TargetModelFrame"], _G["CUI_TargetClassIcon"], unit);
    elseif (unit == "focus") then
        CleanUI_InitPortraitType(_G["CUI_FocusModelFrame"], _G["CUI_FocusClassIcon"], unit);
    elseif (unit == "party1") then
        CleanUI_InitPortraitType(_G["CUI_Party1ModelFrame"], _G["CUI_Party1ClassIcon"], unit);
    elseif (unit == "party2") then
        CleanUI_InitPortraitType(_G["CUI_Party2ModelFrame"], _G["CUI_Party2ClassIcon"], unit);
    elseif (unit == "party3") then
        CleanUI_InitPortraitType(_G["CUI_Party3ModelFrame"], _G["CUI_Party3ClassIcon"], unit);
    elseif (unit == "party4") then
        CleanUI_InitPortraitType(_G["CUI_Party4ModelFrame"], _G["CUI_Party4ClassIcon"], unit);
    end
end

function CleanUI_SetPortraitTexture(portraitframe, unit)
    if (portraitframe) then
 	    if (portraitframe:GetName() == "PlayerPortrait") then
	        CleanUI_InitPortraitType(_G["CUI_PlayerModelFrame"], _G["CUI_PlayerClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "TargetFramePortrait") then
	        CleanUI_InitPortraitType(_G["CUI_TargetModelFrame"], _G["CUI_TargetClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "FocusFramePortrait") then
	        CleanUI_InitPortraitType(_G["CUI_FocusModelFrame"], _G["CUI_FocusClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "PartyMemberFrame1Portrait") then
	        CleanUI_InitPortraitType(_G["CUI_Party1ModelFrame"], _G["CUI_Party1ClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "PartyMemberFrame2Portrait") then
	        CleanUI_InitPortraitType(_G["CUI_Party2ModelFrame"], _G["CUI_Party2ClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "PartyMemberFrame3Portrait") then
	        CleanUI_InitPortraitType(_G["CUI_Party3ModelFrame"], _G["CUI_Party3ClassIcon"], unit);
 	    elseif (portraitframe:GetName() == "PartyMemberFrame4Portrait") then
	        CleanUI_InitPortraitType(_G["CUI_Party4ModelFrame"], _G["CUI_Party4ClassIcon"], unit);
 	    end
 	end
end

function CleanUI_CheckClassColors()

    CleanUI_SetClassColors("player", PlayerName);
    
    for i=1, 4, 1 do
        CleanUI_SetClassColors("party"..i, _G["PartyMemberFrame"..i.."Name"]);
    end
end

function CleanUI_SetClassColors(unit, text)
    local _, class = UnitClass(unit);
    
    local classTextColor;
	if (class) then
		classTextColor = RAID_CLASS_COLORS[class];
	else
		classTextColor = NORMAL_FONT_COLOR;
	end
    text:SetTextColor(classTextColor.r, classTextColor.g, classTextColor.b);
end

function CleanUI_InitUnitFrameUI()
	-- Fokus entfernen
	local removeFocusButton = CreateFrame("Button", "CUI_RemoveFocusButton", FocusFrame, "CleanUIFocusButtonTemplate");
	removeFocusButton:SetScale(1);
    removeFocusButton:ClearAllPoints();
    removeFocusButton:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", 230, 2);
	removeFocusButton:SetAttribute("type", "macro");
	removeFocusButton:SetAttribute("macrotext", "/clearfocus");

	removeFocusButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
	removeFocusButton:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
	removeFocusButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
	
	local removeFocusButtonIcon = _G["CUI_RemoveFocusButtonIcon"];
	removeFocusButtonIcon:SetTexture("Interface\\AddOns\\CleanUI\\skins\\clearfocus");
	
	_G["CUI_RemoveFocusButtonBorder"]:Hide();
	_G["CUI_RemoveFocusButtonNormalTexture"]:Hide();
		
	-- Party
    for i=1, 4, 1 do       
        -- party pet manabar
        local partyPetManaBar = CreateFrame("StatusBar", "CUI_Party"..i.."PetManaBar", _G["PartyMemberFrame"..i.."PetFrame"], "CleanUIPartyPetManaBarTemplate");
        partyPetManaBar:SetWidth(75);    
        partyPetManaBar:SetHeight(14);
        partyPetManaBar:ClearAllPoints();
        partyPetManaBar:SetPoint("TOP", _G["PartyMemberFrame"..i.."PetFrameHealthBar"], "BOTTOM", 0, 0);
        partyPetManaBar:SetFrameStrata("LOW");   
        partyPetManaBar:EnableMouse(false); 
        
        --party pet name
        local partyPetName = _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:CreateFontString("CUI_Party"..i.."PetName", "ARTWORK");
        partyPetName:SetFontObject("GameFontNormalSmall");
        partyPetName:SetWidth(66);
        partyPetName:SetHeight(10);
        partyPetName:ClearAllPoints(); 
        partyPetName:SetPoint("CENTER", _G["PartyMemberFrame"..i.."PetFrame"], "CENTER", 0, 15);
        partyPetName:SetJustifyH("CENTER");
        partyPetName:SetText("");
    end
    
    local targetHealthBarPercent = TargetFrame:CreateFontString("CUI_TargetHealthBarPercent", "OVERLAY");
	targetHealthBarPercent:SetFontObject("SystemFont_Outline_Small");
    targetHealthBarPercent:SetWidth(50);
    targetHealthBarPercent:SetHeight(10);
    targetHealthBarPercent:ClearAllPoints(); 
    targetHealthBarPercent:SetPoint("BOTTOMRIGHT", TargetFrameHealthBar, "BOTTOMRIGHT", 0, 5);
    targetHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    targetHealthBarPercent:SetJustifyH("RIGHT");
    targetHealthBarPercent:SetText("");
    
    local targetManaBarPercent = TargetFrame:CreateFontString("CUI_TargetManaBarPercent", "OVERLAY");
	targetManaBarPercent:SetFontObject("SystemFont_Outline_Small");
    targetManaBarPercent:SetWidth(50);
    targetManaBarPercent:SetHeight(10);
    targetManaBarPercent:ClearAllPoints(); 
    targetManaBarPercent:SetPoint("RIGHT", TargetFrameManaBar, "RIGHT", 0, 0);
    targetManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    targetManaBarPercent:SetJustifyH("RIGHT");
    targetManaBarPercent:SetText("");
    
    local targetInfo = TargetFrame:CreateFontString("CUI_TargetInfo", "ARTWORK");
	targetInfo:SetFontObject("GameFontNormalSmall");
    targetInfo:SetWidth(200);
    targetInfo:SetHeight(10);
    targetInfo:SetTextColor(1.0, 1.0, 1.0, 1.0);
    targetInfo:SetText("");
    
    local focusHealthBarPercent = FocusFrame:CreateFontString("CUI_FocusHealthBarPercent", "OVERLAY");
	focusHealthBarPercent:SetFontObject("SystemFont_Outline_Small");
    focusHealthBarPercent:SetWidth(50);
    focusHealthBarPercent:SetHeight(10);
    focusHealthBarPercent:ClearAllPoints(); 
    focusHealthBarPercent:SetPoint("BOTTOMRIGHT", FocusFrameHealthBar, "BOTTOMRIGHT", 0, 5);
    focusHealthBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    focusHealthBarPercent:SetJustifyH("RIGHT");
    focusHealthBarPercent:SetText("");
    
    local focusManaBarPercent = FocusFrame:CreateFontString("CUI_FocusManaBarPercent", "OVERLAY");
	focusManaBarPercent:SetFontObject("SystemFont_Outline_Small");
    focusManaBarPercent:SetWidth(50);
    focusManaBarPercent:SetHeight(10);
    focusManaBarPercent:ClearAllPoints(); 
    focusManaBarPercent:SetPoint("RIGHT", FocusFrameManaBar, "RIGHT", 0, 0);
    focusManaBarPercent:SetTextColor(1.0, 1.0, 1.0, 1.0);
    focusManaBarPercent:SetJustifyH("RIGHT");
    focusManaBarPercent:SetText("");
    
    local focusInfo = FocusFrame:CreateFontString("CUI_FocusInfo", "ARTWORK");
	focusInfo:SetFontObject("GameFontNormalSmall");
    focusInfo:SetWidth(200);
    focusInfo:SetHeight(10);
    focusInfo:SetTextColor(1.0, 1.0, 1.0, 1.0);
    focusInfo:SetText("");
    
    TargetFrame.tappedInfo = TargetFrame:CreateFontString("TargetFrametappedInfo", "OVERLAY");
	TargetFrame.tappedInfo:SetFontObject("NumberFontNormalHuge");
	TargetFrame.tappedInfo:SetTextHeight(20);
    TargetFrame.tappedInfo:SetWidth(200);
    TargetFrame.tappedInfo:SetHeight(100);
    TargetFrame.tappedInfo:SetTextColor(1.0, 0.2, 0.2);
    TargetFrame.tappedInfo:ClearAllPoints(); 
    TargetFrame.tappedInfo:SetPoint("CENTER", TargetFrame, "CENTER", 90, 15);
    TargetFrame.tappedInfo:SetText("");
    
    FocusFrame.tappedInfo = FocusFrame:CreateFontString("FocusFrametappedInfo", "OVERLAY");
	FocusFrame.tappedInfo:SetFontObject("NumberFontNormalHuge");
	FocusFrame.tappedInfo:SetTextHeight(20);
    FocusFrame.tappedInfo:SetWidth(200);
    FocusFrame.tappedInfo:SetHeight(100);
    FocusFrame.tappedInfo:SetTextColor(1.0, 0.2, 0.2);
    FocusFrame.tappedInfo:ClearAllPoints(); 
    FocusFrame.tappedInfo:SetPoint("CENTER", FocusFrame, "CENTER", 90, 15);
    FocusFrame.tappedInfo:SetText("");
    
    local DRAGON_X = 167;
    local DRAGON_Y = 16;
    local DRAGON_SIZE = 90;
    
    local targetWorldBossTexture = TargetFrame:CreateTexture("CUI_TargetFrameWorldBossTexture", "OVERLAY");
    targetWorldBossTexture:ClearAllPoints(); 
    targetWorldBossTexture:SetPoint("TOPLEFT", TargetFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    targetWorldBossTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\red_dragon");
    targetWorldBossTexture:SetWidth(DRAGON_SIZE);
    targetWorldBossTexture:SetHeight(DRAGON_SIZE);
    targetWorldBossTexture:Hide();
    
    local targetEliteTexture = TargetFrame:CreateTexture("CUI_TargetFrameEliteTexture", "OVERLAY");
    targetEliteTexture:ClearAllPoints(); 
    targetEliteTexture:SetPoint("TOPLEFT", TargetFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    targetEliteTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\gold_dragon");
    targetEliteTexture:SetWidth(DRAGON_SIZE);
    targetEliteTexture:SetHeight(DRAGON_SIZE);
    targetEliteTexture:Hide();
    
    local targetRareTexture = TargetFrame:CreateTexture("CUI_TargetFrameRareTexture", "OVERLAY");
    targetRareTexture:ClearAllPoints(); 
    targetRareTexture:SetPoint("TOPLEFT", TargetFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    targetRareTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\silver_dragon");
    targetRareTexture:SetWidth(DRAGON_SIZE);
    targetRareTexture:SetHeight(DRAGON_SIZE);
    targetRareTexture:Hide();
    
    local focusWorldBossTexture = FocusFrame:CreateTexture("CUI_FocusFrameWorldBossTexture", "OVERLAY");
    focusWorldBossTexture:ClearAllPoints(); 
    focusWorldBossTexture:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    focusWorldBossTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\red_dragon");
    focusWorldBossTexture:SetWidth(DRAGON_SIZE);
    focusWorldBossTexture:SetHeight(DRAGON_SIZE);
    focusWorldBossTexture:Hide();
    
    local focusEliteTexture = FocusFrame:CreateTexture("CUI_FocusFrameEliteTexture", "OVERLAY");
    focusEliteTexture:ClearAllPoints(); 
    focusEliteTexture:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    focusEliteTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\gold_dragon");
    focusEliteTexture:SetWidth(DRAGON_SIZE);
    focusEliteTexture:SetHeight(DRAGON_SIZE);
    focusEliteTexture:Hide();
    
    local focusRareTexture = FocusFrame:CreateTexture("CUI_FocusFrameRareTexture", "OVERLAY");
    focusRareTexture:ClearAllPoints(); 
    focusRareTexture:SetPoint("TOPLEFT", FocusFrame, "TOPLEFT", DRAGON_X, DRAGON_Y);
    focusRareTexture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\silver_dragon");
    focusRareTexture:SetWidth(DRAGON_SIZE);
    focusRareTexture:SetHeight(DRAGON_SIZE);
    focusRareTexture:Hide();
    
    for i = 1, MAX_BOSS_FRAMES do
        local texture = _G["Boss"..i.."TargetFrame"]:CreateTexture("CUI_Boss"..i.."TargetFrameEliteTexture", "OVERLAY");
        texture:ClearAllPoints(); 
        texture:SetPoint("TOPLEFT", _G["Boss"..i.."TargetFrame"], "TOPLEFT", 58, 18);
        texture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\dragons\\gold_dragon");
        texture:SetWidth(110);
        texture:SetHeight(105);
    end
end

function CleanUI_UnitFrame_UpdateTooltip(self)

    local name, id = GameTooltip:GetUnit();
    
    if (name == UnitName("player")) then
        cui_party_xp[name] = {level=UnitLevel("player"), xp=UnitXP("player"), xpmax=UnitXPMax("player"), xpexhaustion=GetXPExhaustion()};
    end
    
    if (cui_party_xp[name] == nil or cui_party_xp[name].xp == nil or cui_party_xp[name].xpmax == nil) then
        return;
    end
    
    local level =  tonumber(cui_party_xp[name].level);
    
    if (level < MAX_PLAYER_LEVEL) then
        local xp =  tonumber(cui_party_xp[name].xp);
        local xpmax =  tonumber(cui_party_xp[name].xpmax);
        local pxp = 100 * xp / xpmax;
            
        GameTooltip:AddLine(" ");
        GameTooltip:AddLine("|c333399ffXP:|r |cffffffff"..xp.."/"..xpmax.."  (|r"..format("%d", pxp).." %|cffffffff)|r");
            
        GameTooltip:AppendText("");
    end
end

function CleanUI_ColorizeRuneFrame()
    RuneFrame:SetBackdropBorderColor(1.0, 1.0, 1.0, 1.0);
    
    local _, class = UnitClass("player");
	
	if ( class == "DEATHKNIGHT" ) then    
        for index = 1,25 do
            local _, _, _, _, _, _, _, _, _, _, spellId = UnitAura("player", index);
            if (spellId == 48266) then    -- Frost: 48266
                RuneFrame:SetBackdropColor(0.0, 0.3, 0.7, 1.0);
                RuneFrame:SetBackdropBorderColor(0.0, 0.5, 1.0, 1.0);
                return;
            elseif (spellId == 48263) then        -- Blut: 48263
                RuneFrame:SetBackdropColor(0.7, 0.0, 0.0, 1.0);
                RuneFrame:SetBackdropBorderColor(1.0, 0.0, 0.0, 1.0);
                return;
            elseif (spellId == 48265) then    -- Unheilig: 48265
                RuneFrame:SetBackdropColor(0.0, 0.3, 0.0, 1.0);
                RuneFrame:SetBackdropBorderColor(0.0, 0.5, 0.0, 1.0);
                return;
            end
        end
    end
end

function CleanUI_SetWhiteStatusTexture(textureframe, parent)
    if (not parent) then 
        return;
    end
    
    textureframe:SetTexture("Interface\\AddOns\\CleanUI\\skins\\CleanUI-UnitStatus-White");
    textureframe:SetTexCoord(0, 1, 0, 1);
    textureframe:ClearAllPoints();
    textureframe:SetPoint("TOPLEFT", parent, "TOPLEFT", 3, -3);
    textureframe:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -3, 3);
end

function CleanUI_SetRedStatusTexture(textureframe, parent)
    if (not parent) then 
        return;
    end
    
    textureframe:SetTexture("Interface\\AddOns\\CleanUI\\skins\\CleanUI-UnitStatus-Red");
    textureframe:SetTexCoord(0, 1, 0, 1);
    textureframe:ClearAllPoints();
    textureframe:SetPoint("TOPLEFT", parent, "TOPLEFT", 3, -3);
    textureframe:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", -3, 3);
end

function CleanUI_MoveComboFrameIcons()
    CleanUI_SetBackdrop(ComboFrame);	
    
    ComboFrame:SetWidth(118);
    ComboFrame:SetHeight(32);
    ComboFrame:ClearAllPoints();
    ComboFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 4);
    
    local width = 22;
    local height = 24;
		
	local combopoint, highlight;
	local parent = ComboFrame;
    local xoffset = 4;
    local ref = "LEFT";
	
	for i=1, MAX_COMBO_POINTS do
        comboPoint = _G["ComboPoint"..i];
        highlight = _G["ComboPoint"..i.."Highlight"];
		
	    comboPoint:SetWidth(width);
	    comboPoint:SetHeight(height);
	    comboPoint:ClearAllPoints();
        comboPoint:SetPoint("LEFT", parent, ref, xoffset, 0);
        parent = comboPoint;
        xoffset = 0;
        ref = "RIGHT";
        
	    highlight:ClearAllPoints();
        highlight:SetPoint("TOPLEFT", comboPoint, "TOPLEFT", 0, 0);
        highlight:SetPoint("BOTTOMRIGHT", comboPoint, "BOTTOMRIGHT", 0, 0);
	
        for i, child in ipairs({comboPoint:GetRegions()}) do
            if (not child:GetName()) then
                child:SetTexture();
            end
        end
    end
    
    CleanUI_UpdateComboPointTexture(CleanUIData.CUI_STATUSBAR);
end

function CleanUI_UpdateComboPointTexture(texture) 
    local highlight;
    for i=1, MAX_COMBO_POINTS do
        highlight = _G["ComboPoint"..i.."Highlight"];
        highlight:SetTexture(texture);
        highlight:SetTexCoord(0, 1, 0, 1);
        
        if (i == 1) then
            highlight:SetVertexColor(0.25, 0.75, 0.25, 1.0);
        elseif (i == 2) then
            highlight:SetVertexColor(1.00, 1.00, 0.00, 1.0);
        elseif (i == 3) then
            highlight:SetVertexColor(1.00, 0.75, 0.25, 1.0);
        elseif (i == 4) then
            highlight:SetVertexColor(1.00, 0.50, 0.25, 1.0);
        elseif (i == 5) then
            highlight:SetVertexColor(1.00, 0.10, 0.10, 1.0);
        end
    end
end

function CleanUI_InitUnitTextures() 
	-- Player
    PlayerFrameTexture:Hide();
    PlayerFrameVehicleTexture:Hide();
    PlayerFrameBackground:Hide();
    PlayerPortrait:Hide();
    PlayerAttackBackground:Hide();
    
    PlayerFrameTexture:SetTexture(nil);
    PlayerFrameVehicleTexture:SetTexture(nil);
    PlayerFrameBackground:SetTexture(nil);
    PlayerPortrait:SetTexture(nil);
    PlayerAttackBackground:SetTexture(nil);
    
    CleanUI_SetWhiteStatusTexture(PlayerStatusTexture, CUI_PlayerFrameBorder2);
    CleanUI_SetRedStatusTexture(PlayerFrameFlash, CUI_PlayerFrameBorder2);
    
	local _, class = UnitClass("player");
	
	if (RuneFrame) then
	    RuneFrame:SetWidth(120);
	    RuneFrame:SetHeight(32);
        RuneFrame:ClearAllPoints();
	    RuneFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 4);
	end
	
	if (TotemFrame) then
	    TotemFrame:SetWidth(120);
	    TotemFrame:SetHeight(32);
        TotemFrame:ClearAllPoints();
	    TotemFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 4);
	end
    
    if (ShardBarFrame) then
        ShardBarFrame:SetWidth(120);
        ShardBarFrame:SetHeight(32);        
        ShardBarFrame:ClearAllPoints();
        ShardBarFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 14, 2);
        
        CleanUI_ShardBar_Update(ShardBarFrame, "SOUL_SHARDS");
    end
    
    if (BurningEmbersBarFrame) then
        BurningEmbersBarFrame:SetScale(0.85);         
        BurningEmbersBarFrame:ClearAllPoints();
        BurningEmbersBarFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 5, 2);
    end
    
    if (DemonicFuryBarFrame) then
        DemonicFuryBarFrame:SetScale(0.8);        
        DemonicFuryBarFrame:ClearAllPoints();
        DemonicFuryBarFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 12, 2);
    end
	
	if (EclipseBarFrame) then         
        EclipseBarFrame:SetWidth(120);
        EclipseBarFrame:SetHeight(32);   
        EclipseBarFrame:ClearAllPoints();
        EclipseBarFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 2);
        
        EclipseBarFrameBar:SetWidth(130);
        EclipseBarFrameSunBar:SetWidth(130);
        EclipseBarFrameMoonBar:SetWidth(130);
        
        EclipseBarFrameDarkSun:ClearAllPoints();
        EclipseBarFrameDarkSun:SetPoint("RIGHT", EclipseBarFrame, "RIGHT", -3, 0);        
        EclipseBarFrameSun:ClearAllPoints();
        EclipseBarFrameSun:SetPoint("RIGHT", EclipseBarFrame, "RIGHT", -3, 0);
        
        EclipseBarFrameDarkMoon:ClearAllPoints();
        EclipseBarFrameDarkMoon:SetPoint("LEFT", EclipseBarFrame, "LEFT", 3, 0);        
        EclipseBarFrameMoon:ClearAllPoints();
        EclipseBarFrameMoon:SetPoint("LEFT", EclipseBarFrame, "LEFT", 3, 0);
    end
    
    if (MonkHarmonyBar) then   
        CleanUI_RemoveTextures(MonkHarmonyBar);
        MonkHarmonyBar:SetWidth(120);
        MonkHarmonyBar:SetHeight(32);   
        MonkHarmonyBar:ClearAllPoints();
        MonkHarmonyBar:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 4);  
        CleanUI_SetBackdrop(MonkHarmonyBar);
        
        CleanUI_MonkHarmonyBar_Update();
    end
    
    if (MonkStaggerBar) then
    	CleanUI_RemoveTextures(MonkStaggerBar);
    	MonkStaggerBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR); 
    end
    
    if (PaladinPowerBar) then     
        PaladinPowerBar:SetWidth(120);
        PaladinPowerBar:SetHeight(36);      
        PaladinPowerBar:ClearAllPoints();
        PaladinPowerBar:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 0, 2);
    end
    
    if (PriestBarFrame) then     
    	CleanUI_RemoveTextures(PriestBarFrame);     
        PriestBarFrame:ClearAllPoints();
        PriestBarFrame:SetPoint("TOPRIGHT", PlayerFrame, "BOTTOMRIGHT", 22, 5);
    end
    
    -- Pet
    PetFrameTexture:Hide();
    PetPortrait:Hide();  
    PetFrameTexture:SetTexture(nil);
    PetPortrait:SetTexture(nil);  
    
    CleanUI_SetWhiteStatusTexture(PetFrameFlash, PetFrame);
    CleanUI_SetRedStatusTexture(PetAttackModeTexture, PetFrame);
    
    -- Target
    TargetFrameBackground:Hide();
    TargetFrameNameBackground:Hide();
    TargetFrameTextureFrameTexture:Hide();
    TargetFramePortrait:Hide();
    TargetFrameBackground:SetTexture(nil);
    TargetFrameNameBackground:SetTexture(nil);
    TargetFrameTextureFrameTexture:SetTexture(nil);
    TargetFramePortrait:SetTexture(nil);
    
    CleanUI_SetWhiteStatusTexture(TargetFrameFlash, CUI_TargetFrameBorder1);
	
    TargetFrameToTPortrait:Hide();
    TargetFrameToTBackground:Hide();
    TargetFrameToTTextureFrameTexture:Hide();
    TargetFrameToTPortrait:SetTexture(nil);
    TargetFrameToTBackground:SetTexture(nil);
    TargetFrameToTTextureFrameTexture:SetTexture(nil);
    
    -- Focus
    FocusFrameBackground:Hide();
    FocusFrameNameBackground:Hide();
    FocusFrameTextureFrameTexture:Hide();
    FocusFramePortrait:Hide();
    FocusFrameBackground:SetTexture(nil);
    FocusFrameNameBackground:SetTexture(nil);
    FocusFrameTextureFrameTexture:SetTexture(nil);
    FocusFramePortrait:SetTexture(nil);
    
    CleanUI_SetWhiteStatusTexture(FocusFrameFlash, CUI_FocusFrameBorder1);
	
    FocusFrameToTPortrait:Hide();
    FocusFrameToTBackground:Hide();
    FocusFrameToTTextureFrameTexture:Hide();
    FocusFrameToTPortrait:SetTexture(nil);
    FocusFrameToTBackground:SetTexture(nil);
    FocusFrameToTTextureFrameTexture:SetTexture(nil);
    
    -- Party
	for i=1, 4, 1 do    
        _G["PartyMemberFrame"..i.."Portrait"]:Hide();
        _G["PartyMemberFrame"..i.."Texture"]:Hide();
        _G["PartyMemberFrame"..i.."VehicleTexture"]:Hide();
        _G["PartyMemberFrame"..i.."Background"]:Hide();
    
        _G["PartyMemberFrame"..i.."PetFramePortrait"]:Hide();
        _G["PartyMemberFrame"..i.."PetFrameTexture"]:Hide();
        
        CleanUI_SetRedStatusTexture(_G["PartyMemberFrame"..i.."PetFrameFlash"], _G["PartyMemberFrame"..i.."PetFrame"]);       
	end	
        
    CleanUI_SetRedStatusTexture(_G["PartyMemberFrame1Flash"], CUI_PartyFrameBorder21);        
    CleanUI_SetWhiteStatusTexture(_G["PartyMemberFrame1Status"], CUI_PartyFrameBorder21);
    CleanUI_SetRedStatusTexture(_G["PartyMemberFrame2Flash"], CUI_PartyFrameBorder22);        
    CleanUI_SetWhiteStatusTexture(_G["PartyMemberFrame2Status"], CUI_PartyFrameBorder22);
    CleanUI_SetRedStatusTexture(_G["PartyMemberFrame3Flash"], CUI_PartyFrameBorder23);        
    CleanUI_SetWhiteStatusTexture(_G["PartyMemberFrame3Status"], CUI_PartyFrameBorder23);
    CleanUI_SetRedStatusTexture(_G["PartyMemberFrame4Flash"], CUI_PartyFrameBorder24);        
    CleanUI_SetWhiteStatusTexture(_G["PartyMemberFrame4Status"], CUI_PartyFrameBorder24);
    
    CleanUI_SetWhiteStatusTexture(Boss1TargetFrameFlash, CUI_Boss1TargetFrameBorderFrame);
    CleanUI_SetWhiteStatusTexture(Boss2TargetFrameFlash, CUI_Boss2TargetFrameBorderFrame);
    CleanUI_SetWhiteStatusTexture(Boss3TargetFrameFlash, CUI_Boss3TargetFrameBorderFrame);
    CleanUI_SetWhiteStatusTexture(Boss4TargetFrameFlash, CUI_Boss4TargetFrameBorderFrame);
    CleanUI_SetWhiteStatusTexture(Boss5TargetFrameFlash, CUI_Boss5TargetFrameBorderFrame);
    
	CleanUI_InitUnitDrawLayer();
end

function CleanUI_ShardBar_Update(self, powerType)
    if ( powerType and powerType ~= "SOUL_SHARDS" ) then
        return;
    end

    if ( self.shardCount == 3 ) then
        self.shard1:SetPoint("TOPLEFT", 0, 0);
        self.shard2:SetPoint("TOPLEFT", self.shard1, "TOPLEFT", 33, 0);
        self.shard3:SetPoint("TOPLEFT", self.shard2, "TOPLEFT", 33, 0);
    else
        self.shard1:SetPoint("TOPLEFT", -10, 0);
        self.shard2:SetPoint("TOPLEFT", self.shard1, "TOPLEFT", 27, 0);
        self.shard3:SetPoint("TOPLEFT", self.shard2, "TOPLEFT", 27, 0);
        self.shard4:SetPoint("TOPLEFT", self.shard3, "TOPLEFT", 27, 0);
    end
end

function CleanUI_InitUnitDrawLayer()
    PlayerStatusTexture:SetDrawLayer("OVERLAY");
    PlayerFrameFlash:SetDrawLayer("ARTWORK");
    PetFrameFlash:SetDrawLayer("ARTWORK");
    PetAttackModeTexture:SetDrawLayer("OVERLAY");
    TargetFrameFlash:SetDrawLayer("ARTWORK");
    FocusFrameFlash:SetDrawLayer("ARTWORK");
    
    Boss1TargetFrameFlash:SetDrawLayer("ARTWORK");
    Boss2TargetFrameFlash:SetDrawLayer("ARTWORK");
    Boss3TargetFrameFlash:SetDrawLayer("ARTWORK");
    Boss4TargetFrameFlash:SetDrawLayer("ARTWORK");
    Boss5TargetFrameFlash:SetDrawLayer("ARTWORK");
    
	for i=1, 4, 1 do
        _G["PartyMemberFrame"..i.."Flash"]:SetDrawLayer("OVERLAY");
        _G["PartyMemberFrame"..i.."Status"]:SetDrawLayer("OVERLAY");
        _G["PartyMemberFrame"..i.."PetFrameFlash"]:SetDrawLayer("OVERLAY");
	end
end

function CleanUI_MonkHarmonyBar_Update()  
    local chiMax = UnitPowerMax("player", SPELL_POWER_CHI);
    
    local energy, texture;
    for i=1, 5 do
        energy = _G["MonkHarmonyBarLightEnergy"..i];
        energy:ClearAllPoints();
        CleanUI_RemoveTextures(energy);
        
        texture = _G["MonkHarmonyBarLightEnergy"..i.."Glow"];
        
        texture:SetTexture("Interface\\AddOns\\CleanUI\\skins\\monk\\"..CleanUIData.MONK_CHI_TEXTURE);
        texture:SetTexCoord(0, 1, 0, 1);
    end
        
    if (chiMax == 5) then
        MonkHarmonyBarLightEnergy1:SetPoint("LEFT", MonkHarmonyBar, "LEFT", 8, 0);
        MonkHarmonyBarLightEnergy2:SetPoint("LEFT", MonkHarmonyBarLightEnergy1, "RIGHT", 3, 0);
        MonkHarmonyBarLightEnergy3:SetPoint("LEFT", MonkHarmonyBarLightEnergy2, "RIGHT", 3, 0);
        MonkHarmonyBarLightEnergy4:SetPoint("LEFT", MonkHarmonyBarLightEnergy3, "RIGHT", 3, 0);
        MonkHarmonyBarLightEnergy5:SetPoint("LEFT", MonkHarmonyBarLightEnergy4, "RIGHT", 3, 0);
    else
        MonkHarmonyBarLightEnergy1:SetPoint("LEFT", MonkHarmonyBar, "LEFT", 12, 0);
        MonkHarmonyBarLightEnergy2:SetPoint("LEFT", MonkHarmonyBarLightEnergy1, "RIGHT", 7, 0);
        MonkHarmonyBarLightEnergy3:SetPoint("LEFT", MonkHarmonyBarLightEnergy2, "RIGHT", 7, 0);
        MonkHarmonyBarLightEnergy4:SetPoint("LEFT", MonkHarmonyBarLightEnergy3, "RIGHT", 7, 0);
    end
end

function CleanUI_MovePartyIcons()
    for i = 1, 4 do
        CleanUI_PartyMemberFrame_UpdateArt(_G["PartyMemberFrame"..i]);
    end
end

function CleanUI_PlayerFrame_UpdateArt(self)
    CleanUI_StartDelay(3, CleanUI_MovePlayerIcons);
end

function CleanUI_PartyMemberFrame_UpdateArt(self)
 	local i = self:GetID();
 	local unit = "party"..i;
	
	if (UnitExists(unit)) then
	    CleanUI_MovePartyDefaultIcons(i);
    end
    
    if (InCombatLockdown() == nil) then
        _G["PartyMemberFrame1"]:ClearAllPoints();
        _G["PartyMemberFrame1"]:SetPoint("TOPLEFT", _G["CompactRaidFrameManager"], "TOPRIGHT", 7, -15);
        
        if (i > 1) then
            _G["PartyMemberFrame"..i]:ClearAllPoints();
            _G["PartyMemberFrame"..i]:SetPoint("TOPLEFT", _G["PartyMemberFrame"..(i-1)], "BOTTOMLEFT", 0, -25);
        end
    end
end

function CleanUI_MovePlayerIcons()    
    CleanUI_LoadFramePosition(PlayerFrame, 5, -10);
    
    if (InCombatLockdown() == nil) then
        PlayerFrame:SetWidth(UNIT_FRAME_WIDTH);
        PlayerFrame:SetHeight(UNIT_FRAME_HEIGHT);
    
        PlayerFrame:SetHitRectInsets(0, 0, 0, 0);
    end
    
	local borderframe1 = _G["CUI_PlayerFrameBorder1"];
    if (not borderframe1) then
        borderframe1 = CreateFrame("Frame", "CUI_PlayerFrameBorder1", PlayerFrame);
	end
	borderframe1:SetPoint("TOPLEFT", 0, 0);
    borderframe1:SetWidth(UNIT_FRAME_HEIGHT);
    borderframe1:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe1:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe1);
    
	local borderframe2 = _G["CUI_PlayerFrameBorder2"];
    if (not borderframe2) then
        borderframe2 = CreateFrame("Frame", "CUI_PlayerFrameBorder2", PlayerFrame);
	end
	borderframe2:SetPoint("TOPRIGHT", 0, 0);
    borderframe2:SetWidth(BARFRAME_WIDTH);
    borderframe2:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe2:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe2);
    
    CleanUI_SetWhiteStatusTexture(PlayerStatusTexture, CUI_PlayerFrameBorder2);
    CleanUI_SetRedStatusTexture(PlayerFrameFlash, CUI_PlayerFrameBorder2);
    
    PlayerFrameTexture:Hide();
    PlayerFrameVehicleTexture:Hide();
    PlayerFrameBackground:Hide();
    PlayerPortrait:Hide();
    
    PlayerFrameTexture:SetTexture(nil);
    PlayerFrameVehicleTexture:SetTexture(nil);
    PlayerFrameBackground:SetTexture(nil);
    PlayerPortrait:SetTexture(nil);
    
    PlayerName:ClearAllPoints();
	PlayerName:SetPoint("TOP", borderframe2, "TOP", 0, -5);
    PlayerName:SetWidth(BARS_WIDTH);
    PlayerName:SetJustifyH("CENTER");
	
    PlayerFrameReadyCheck:ClearAllPoints();
	PlayerFrameReadyCheck:SetPoint("CENTER", borderframe1, "CENTER", 0, 0);
	
    PlayerLeaderIcon:ClearAllPoints();
	PlayerLeaderIcon:SetPoint("TOPLEFT", borderframe1, "TOPLEFT", 3, -1);
	
    PlayerMasterIcon:ClearAllPoints();
	PlayerMasterIcon:SetPoint("TOPLEFT", borderframe1, "TOPLEFT", 3, -12);
	
    PlayerGuideIcon:SetSize(14, 14);
    PlayerGuideIcon:ClearAllPoints();
	PlayerGuideIcon:SetPoint("BOTTOMRIGHT", borderframe1, "BOTTOMRIGHT", -3, 3);
	
	PlayerFrameRoleIcon:SetSize(14, 14);
    PlayerFrameRoleIcon:ClearAllPoints();
	PlayerFrameRoleIcon:SetPoint("BOTTOMRIGHT", borderframe1, "BOTTOMRIGHT", -3, 3);
	
    PlayerLevelText:ClearAllPoints();
	PlayerLevelText:SetPoint("BOTTOMLEFT", borderframe1, "BOTTOMLEFT", 4, 4);
	
    PlayerRestIcon:ClearAllPoints();
	PlayerRestIcon:SetPoint("CENTER", borderframe1, "RIGHT", 0, -2);
	
    PlayerRestGlow:ClearAllPoints();
	PlayerRestGlow:SetPoint("CENTER", borderframe1, "RIGHT", 0, -4);
    
    PlayerAttackIcon:ClearAllPoints();
	PlayerAttackIcon:SetPoint("CENTER", borderframe1, "RIGHT", 0, -4);
	PlayerAttackIcon:SetTexCoord(0.5, 1.0, 0, 0.49);
    
    PlayerAttackGlow:ClearAllPoints();
	PlayerAttackGlow:SetPoint("CENTER", borderframe1, "RIGHT", 0, -4);
    
    PlayerAttackBackground:ClearAllPoints();
	PlayerAttackBackground:SetPoint("TOPLEFT", PlayerFrame, "TOPLEFT", 2, 1000000); -- out of screen
    
    PlayerPVPIcon:ClearAllPoints();
    PlayerPVPIcon:SetPoint("CENTER", PlayerFrame, "RIGHT", 4, -5);
    PlayerPVPIcon:SetWidth(36);    
    PlayerPVPIcon:SetHeight(36);
    
    PlayerPVPTimerText:ClearAllPoints();
	PlayerPVPTimerText:SetPoint("CENTER", PlayerPVPIcon, "CENTER", -8, -7);
    PlayerPVPTimerText:SetDrawLayer("OVERLAY"); 
	
	PlayerHitIndicator:ClearAllPoints();
	PlayerHitIndicator:SetPoint("CENTER", borderframe1, "CENTER", 0, 0);
	
	-- non shaman totem
    local button = _G["TotemFrameTotem1"];
    button:ClearAllPoints();
    button:SetPoint("CENTER", PlayerFrame, "RIGHT", 22, 0);

	-- class specific
	local _, class = UnitClass("player");	
	if ( class == "DEATHKNIGHT") then	    
	    CleanUI_SetBackdrop(RuneFrame);
        
        RuneButtonIndividual1:SetPoint("LEFT", RuneFrame, "LEFT", 9, 0);
        RuneButtonIndividual2:SetPoint("LEFT", RuneButtonIndividual1, "RIGHT", 3, 0);
        RuneButtonIndividual3:SetPoint("LEFT", RuneButtonIndividual2, "RIGHT", 6, 0);
        RuneButtonIndividual4:SetPoint("LEFT", RuneButtonIndividual3, "RIGHT", 3, 0);
        RuneButtonIndividual5:SetPoint("LEFT", RuneButtonIndividual4, "RIGHT", 6, 0);  
        RuneButtonIndividual6:SetPoint("LEFT", RuneButtonIndividual5, "RIGHT", 3, 0);
        
        for i = 1, 6 do
            _G["RuneButtonIndividual"..i]:SetScale(0.8);
        end

	elseif ( class == "SHAMAN" ) then	    
	    CleanUI_SetBackdrop(TotemFrame);
        
        TotemFrameTotem1:SetPoint("TOPLEFT", TotemFrame, "TOPLEFT", 6, -3);
        TotemFrameTotem2:SetPoint("LEFT", TotemFrameTotem1, "RIGHT", -1, 0);
        TotemFrameTotem3:SetPoint("LEFT", TotemFrameTotem2, "RIGHT", -1, 0);
        TotemFrameTotem4:SetPoint("LEFT", TotemFrameTotem3, "RIGHT", -1, 0);      
        
        for i = 1, 4 do
            local button = _G["TotemFrameTotem"..i];
            button:SetScale(0.75);
            
            local duration = _G["TotemFrameTotem"..i.."Duration"];
            duration:ClearAllPoints();
            duration:SetPoint("CENTER", button, "CENTER", 0, 0);
        end
	elseif ( class == "WARLOCK" ) then      
	    
        local primaryTree = GetSpecialization(false, false, index);    
        if (primaryTree == 1) then
        end       
	elseif ( class == "PALADIN" ) then                 
        PaladinPowerBar:SetAlpha(0.0);
	    
	    CleanUI_SetBackdrop(PaladinPowerBar);
        
        CleanUI_SetWhiteStatusTexture(PaladinPowerBarGlowBGTexture, PaladinPowerBar);
        PaladinPowerBarGlowBGTexture:SetDrawLayer("ARTWORK");
        
        PaladinPowerBarBankBG:Hide();
        PaladinPowerBarBG:Hide();
            
        PaladinPowerBarRune1:ClearAllPoints();
        PaladinPowerBarRune1:SetPoint("LEFT", PaladinPowerBar, "LEFT", 12, 0);
	end
	
	PlayerFrameGroupIndicatorLeft:Hide();
	PlayerFrameGroupIndicatorRight:Hide();
	PlayerFrameGroupIndicatorMiddle:Hide();
    
    PlayerFrameGroupIndicator:SetFrameStrata("HIGH");
    PlayerFrameGroupIndicator:ClearAllPoints();
	PlayerFrameGroupIndicator:SetPoint("TOPLEFT", borderframe1, "TOPLEFT", -10, 15);
    
    PlayerFrameAlternateManaBarBackground:Hide();
    PlayerFrameAlternateManaBarBorder:Hide();
    PlayerFrameAlternateManaBarLeftBorder:Hide();
    PlayerFrameAlternateManaBarRightBorder:Hide();
    CleanUI_RemoveTextures(PlayerFrameAlternateManaBar);
    PlayerFrameAlternateManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    
    CleanUI_UpdateShapeshiftForm();
end

local PALADIN_POWERBAR_HIDE_WAIT = 1.5;
local _palaWaitTimeSinceLast = 0;
function CleanUI_UpdatePaladinPowerBar(elapsed)

    local alpha1 = PaladinPowerBarRune1:GetAlpha();
    
    if (alpha1 == 0.0) then
        if (_palaWaitTimeSinceLast > 0) then
            _palaWaitTimeSinceLast = _palaWaitTimeSinceLast - elapsed;
            if (_palaWaitTimeSinceLast < 0) then
                PaladinPowerBar:SetAlpha(0.0);
            end
	    end
    else
        PaladinPowerBar:SetAlpha(1.0);
        _palaWaitTimeSinceLast = PALADIN_POWERBAR_HIDE_WAIT;
    end
end

function CleanUI_UpdateShapeshiftForm()
    
    PlayerFrameHealthBar:SetWidth(BARS_WIDTH);
    PlayerFrameHealthBar:SetHeight(HEALTHBAR_HEIGHT);  
    PlayerFrameHealthBar:ClearAllPoints();
    PlayerFrameHealthBar:SetPoint("TOPLEFT", PLAYER_BAROFFSET, HEALTHBAR_YPOS_OFFSET);
    PlayerFrameHealthBarText:ClearAllPoints();
    PlayerFrameHealthBarText:SetPoint("BOTTOM", PlayerFrameHealthBar, "BOTTOM", 0, 5);
    
    PlayerFrameManaBar:SetWidth(BARS_WIDTH); 
    PlayerFrameManaBar:ClearAllPoints();
    PlayerFrameManaBar:SetPoint("TOP", PlayerFrameHealthBar, "BOTTOM", 0, 0);
    PlayerFrameManaBarText:ClearAllPoints();
    PlayerFrameManaBarText:SetPoint("CENTER", PlayerFrameManaBar, "CENTER", 0, 0);
    
    --PlayerFrameAlternateManaBar:Hide();
        
    PlayerFrameAlternateManaBar:SetWidth(BARS_WIDTH);    
    PlayerFrameAlternateManaBar:SetHeight(ALTERNATE_MANABAR_HEIGHT);
    PlayerFrameAlternateManaBar:ClearAllPoints();
    PlayerFrameAlternateManaBar:SetPoint("TOP", PlayerFrameManaBar, "BOTTOM", 0, 0);
    PlayerFrameAlternateManaBar:SetStatusBarColor(PowerBarColor["MANA"].r, PowerBarColor["MANA"].g, PowerBarColor["MANA"].b);
        
    MonkStaggerBar:SetWidth(BARS_WIDTH);    
    MonkStaggerBar:SetHeight(ALTERNATE_MANABAR_HEIGHT);
    MonkStaggerBar:ClearAllPoints();
    MonkStaggerBar:SetPoint("TOP", PlayerFrameManaBar, "BOTTOM", 0, 0);
    
	if ( PlayerFrameAlternateManaBar:IsVisible() ) then   
        PlayerFrameManaBar:SetHeight(MANABAR_HEIGHT - ALTERNATE_MANABAR_HEIGHT);
    elseif ( MonkStaggerBar:IsVisible() ) then   
        PlayerFrameManaBar:SetHeight(MANABAR_HEIGHT - ALTERNATE_MANABAR_HEIGHT);
    else
        PlayerFrameManaBar:SetHeight(MANABAR_HEIGHT);
    end
end

function CleanUI_MovePetIcons()
    
    if (InCombatLockdown() == nil) then
        PetFrame:SetWidth(116);
        PetFrame:SetHeight(48);
        PetFrame:SetHitRectInsets(0, 0, 0, 0);
    
        PetFrame:ClearAllPoints();
        PetFrame:SetPoint("TOPLEFT", 0, -PET_YPOS_OFFSET);
    end
    
    CleanUI_SetBackdrop(PetFrame);
    
    PetFrameTexture:Hide();
    PetPortrait:Hide();
    
    PetName:ClearAllPoints();
	PetName:SetPoint("CENTER", 0, 15);
    
    PetFrameHealthBar:SetWidth(107);
    PetFrameHealthBar:SetHeight(26);  
    PetFrameHealthBar:ClearAllPoints();
    PetFrameHealthBar:SetPoint("TOP", PetFrame, "TOP", 0, -4);
    PetFrameHealthBar:EnableMouse(false); 
    PetFrameHealthBarText:SetPoint("CENTER", 0, 2); 
    PetFrameHealthBarText:Show();
    
    PetFrameManaBar:SetWidth(107);    
    PetFrameManaBar:SetHeight(14);
    PetFrameManaBar:ClearAllPoints();
    PetFrameManaBar:SetPoint("TOP", PetFrame, "TOP", 0, -30);
    PetFrameManaBar:EnableMouse(false); 
    PetFrameManaBarText:SetPoint("CENTER", 0, -13); 
    
    PetHitIndicator:ClearAllPoints();
    PetHitIndicator:SetPoint("CENTER", PetFrame, "CENTER", 0, 0);
end

function CleanUI_MovePartyDefaultIcons(i)
	
    if (InCombatLockdown() == nil) then
        _G["PartyMemberFrame"..i]:SetWidth(170);	
        _G["PartyMemberFrame"..i]:SetHeight(48);
    end
    
	local borderframe1 = _G["CUI_PartyFrameBorder1"..i];
    if (not borderframe1) then
        borderframe1 = CreateFrame("Frame", "CUI_PartyFrameBorder1"..i, _G["PartyMemberFrame"..i]);
	end
	borderframe1:SetPoint("TOPLEFT", 0, 0);
    borderframe1:SetWidth(48);
    borderframe1:SetHeight(48);
    borderframe1:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe1);
    
	local borderframe2 = _G["CUI_PartyFrameBorder2"..i];
    if (not borderframe2) then
        borderframe2 = CreateFrame("Frame", "CUI_PartyFrameBorder2"..i, _G["PartyMemberFrame"..i]);
	end
	borderframe2:SetPoint("TOPRIGHT", 0, 0);
    borderframe2:SetWidth(126);
    borderframe2:SetHeight(48);
    borderframe2:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe2);
    
    _G["PartyMemberFrame"..i.."Portrait"]:Hide();
    _G["PartyMemberFrame"..i.."Texture"]:Hide();
    _G["PartyMemberFrame"..i.."VehicleTexture"]:Hide();
    _G["PartyMemberFrame"..i.."Background"]:Hide();
    
    _G["PartyMemberFrame"..i.."Name"]:ClearAllPoints();
	_G["PartyMemberFrame"..i.."Name"]:SetPoint("CENTER", borderframe2, "CENTER", 0, 15);
    
    _G["PartyMemberFrame"..i.."RoleIcon"]:SetSize(14, 14);
    _G["PartyMemberFrame"..i.."RoleIcon"]:ClearAllPoints();
    _G["PartyMemberFrame"..i.."RoleIcon"]:SetPoint("BOTTOMRIGHT", borderframe1, "BOTTOMRIGHT", -3, 3);
    
    _G["PartyMemberFrame"..i.."GuideIcon"]:ClearAllPoints();
	_G["PartyMemberFrame"..i.."GuideIcon"]:SetPoint("BOTTOMRIGHT", borderframe1, "BOTTOMRIGHT", -3, 3);
    
    _G["PartyMemberFrame"..i.."MasterIcon"]:ClearAllPoints();
	_G["PartyMemberFrame"..i.."MasterIcon"]:SetPoint("TOPLEFT", borderframe1, "TOPLEFT", 3, -12);
    
    _G["PartyMemberFrame"..i.."LeaderIcon"]:ClearAllPoints();
	_G["PartyMemberFrame"..i.."LeaderIcon"]:SetPoint("TOPLEFT", borderframe1, "TOPLEFT", 3, -1);
    
    _G["PartyMemberFrame"..i.."PVPIcon"]:ClearAllPoints();
	_G["PartyMemberFrame"..i.."PVPIcon"]:SetPoint("CENTER", _G["PartyMemberFrame"..i], "RIGHT", -3, -1);
    
    _G["PartyMemberFrame"..i.."HealthBar"]:SetWidth(117);
    _G["PartyMemberFrame"..i.."HealthBar"]:SetHeight(26);  
    _G["PartyMemberFrame"..i.."HealthBar"]:ClearAllPoints();
    _G["PartyMemberFrame"..i.."HealthBar"]:SetPoint("TOP", borderframe2, "TOP", 0, -4);
    _G["PartyMemberFrame"..i.."HealthBar"]:SetFrameStrata("LOW");
    _G["PartyMemberFrame"..i.."HealthBarText"]:SetPoint("BOTTOM", _G["PartyMemberFrame"..i.."HealthBar"], "BOTTOM", 0, 5);
    
    _G["PartyMemberFrame"..i.."ManaBar"]:SetWidth(117);    
    _G["PartyMemberFrame"..i.."ManaBar"]:SetHeight(14);
    _G["PartyMemberFrame"..i.."ManaBar"]:ClearAllPoints();
    _G["PartyMemberFrame"..i.."ManaBar"]:SetPoint("TOP", _G["PartyMemberFrame"..i.."HealthBar"], "BOTTOM", 0, 0);
    _G["PartyMemberFrame"..i.."ManaBar"]:SetFrameStrata("LOW");
    _G["PartyMemberFrame"..i.."ManaBarText"]:SetPoint("CENTER", _G["PartyMemberFrame"..i.."ManaBar"], "CENTER", 0, 0);
    
    _G["PartyMemberFrame"..i.."Debuff1"]:SetPoint("TOPLEFT", 59, -50);
    
    CleanUI_MovePartyDefaultPetIcons(i);
end

function CleanUI_PartyMemberFrame_UpdatePet(self, id)
    if (InCombatLockdown()) then
        return;
    end

	if ( not id ) then
		id = self:GetID();
	end
	
    _G["PartyMemberFrame"..id.."PetFrame"]:ClearAllPoints();
	_G["PartyMemberFrame"..id.."PetFrame"]:SetPoint("TOPLEFT", _G["PartyMemberFrame"..id], "TOPRIGHT", -2, 0);
    _G["PartyMemberFrame"..id.."PetFrame"]:SetWidth(84);
    _G["PartyMemberFrame"..id.."PetFrame"]:SetHeight(48);  
end

function CleanUI_MovePartyDefaultPetIcons(i)
    CleanUI_SetBackdrop(_G["PartyMemberFrame"..i.."PetFrame"]);
    
    _G["PartyMemberFrame"..i.."PetFramePortrait"]:Hide();
    _G["PartyMemberFrame"..i.."PetFrameTexture"]:Hide();
   
    CleanUI_PartyMemberFrame_UpdatePet(nil, i);
    
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetWidth(75);
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetHeight(26);  
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:ClearAllPoints();
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetPoint("TOP", 0, -4);
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetFrameStrata("LOW");      
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetBackdropColor(0.0, 0.0, 0.0, 1.0);  
    _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:EnableMouse(false); 

    
    _G["PartyMemberFrame"..i.."PetFrameDebuff1"]:SetPoint("TOPLEFT", 7, -50);
    
    local pet = "partypet"..i;
    
    if (UnitExists(pet)) then
        local petname = UnitName(pet);
        
        if (petname and _G["CUI_Party"..i.."PetName"]) then
            _G["CUI_Party"..i.."PetName"]:SetText(petname);
            _G["CUI_Party"..i.."PetName"]:Show();
        end
    else  
        if (_G["CUI_Party"..i.."PetName"]) then
            _G["CUI_Party"..i.."PetName"]:SetText("");  
            _G["CUI_Party"..i.."PetName"]:Hide(); 
        end
    end
end

function CleanUI_MoveTargetIcons()
  
    if (InCombatLockdown() == nil) then
        TargetFrame:SetWidth(UNIT_FRAME_WIDTH);
        TargetFrame:SetHeight(90);
        TargetFrame:SetHitRectInsets(165, 0, 0, 37);
    end
        
    CleanUI_LoadFramePosition(TargetFrame, 300, -10);
    
    TargetFrame.powerBarAlt:ClearAllPoints(); 
    TargetFrame.powerBarAlt:SetPoint("LEFT", TargetFrame, "RIGHT", 0, 5);
    
	local borderframe1 = _G["CUI_TargetFrameBorder1"];
    if (not borderframe1) then
        borderframe1 = CreateFrame("Frame", "CUI_TargetFrameBorder1", TargetFrame);
	end
	borderframe1:SetPoint("TOPLEFT", 0, 0);
    borderframe1:SetWidth(BARFRAME_WIDTH);
    borderframe1:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe1:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe1);
    
	local borderframe2 = _G["CUI_TargetFrameBorder2"];
    if (not borderframe2) then
        borderframe2 = CreateFrame("Frame", "CUI_TargetFrameBorder2", TargetFrame);
	end
	borderframe2:SetPoint("TOPLEFT", borderframe1, "TOPRIGHT", -4, 0);
    borderframe2:SetWidth(UNIT_FRAME_HEIGHT);
    borderframe2:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe2:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe2);
    
    TargetFrameBackground:Hide();
    TargetFrameNameBackground:Hide();
    TargetFrameTextureFrameTexture:Hide();
    TargetFramePortrait:Hide();
    
    TargetFrameBackground:SetTexture(nil);
    TargetFrameNameBackground:SetTexture(nil);
    TargetFrameTextureFrameTexture:SetTexture(nil);
    TargetFramePortrait:SetTexture(nil);
    
    TargetFrameTextureFrameName:ClearAllPoints();
	TargetFrameTextureFrameName:SetPoint("TOP", borderframe1, "TOP", 0, -5);
    TargetFrameTextureFrameName:SetWidth(BARS_WIDTH);
    TargetFrameTextureFrameName:SetJustifyH("CENTER");
	
    CUI_TargetInfo:ClearAllPoints(); 
    CUI_TargetInfo:SetPoint("BOTTOM", borderframe1, "TOP", 0, -1);
	
    TargetFrameNumericalThreat:SetWidth(70);
    TargetFrameNumericalThreat:ClearAllPoints(); 
    TargetFrameNumericalThreat:SetPoint("TOP", borderframe2, "TOP", 0, -3);
	
	for i, child in ipairs({TargetFrameNumericalThreat:GetRegions()}) do
	    if (not child:GetName()) then
	        child:SetTexture();
	    end
	end
    
    TargetFrameHealthBar:SetWidth(BARS_WIDTH + 1);
    TargetFrameHealthBar:SetHeight(HEALTHBAR_HEIGHT); 
    TargetFrameHealthBar:ClearAllPoints(); 
    TargetFrameHealthBar:SetPoint("TOPLEFT", 4, HEALTHBAR_YPOS_OFFSET);    
    TargetFrameTextureFrameHealthBarText:ClearAllPoints(); 
    TargetFrameTextureFrameHealthBarText:SetPoint("BOTTOM", TargetFrameHealthBar, "BOTTOM", 0, 5);
    
    TargetFrameManaBar:SetWidth(BARS_WIDTH + 1);    
    TargetFrameManaBar:SetHeight(MANABAR_HEIGHT);
    TargetFrameManaBar:ClearAllPoints(); 
    TargetFrameManaBar:SetPoint("TOP", TargetFrameHealthBar, "BOTTOM", 0, 0);
    
    TargetFrameTextureFrameManaBarText:ClearAllPoints(); 
    TargetFrameTextureFrameManaBarText:SetPoint("CENTER", TargetFrameManaBar, "CENTER", 0, 0);
    
    TargetFrameTextureFrameLeaderIcon:ClearAllPoints(); 
    TargetFrameTextureFrameLeaderIcon:SetPoint("TOPLEFT", borderframe2, "TOPLEFT", 3, -3);
    
    TargetFrameTextureFrameLevelText:ClearAllPoints(); 
    TargetFrameTextureFrameLevelText:SetPoint("BOTTOMRIGHT", borderframe2, "BOTTOMRIGHT", -4, 4);
    
    TargetFrameTextureFrameHighLevelTexture:ClearAllPoints(); 
    TargetFrameTextureFrameHighLevelTexture:SetPoint("BOTTOMRIGHT", borderframe2, "BOTTOMRIGHT", -4, 4);
    
    TargetFrameTextureFrameRaidTargetIcon:SetWidth(18);    
    TargetFrameTextureFrameRaidTargetIcon:SetHeight(18);
    TargetFrameTextureFrameRaidTargetIcon:ClearAllPoints(); 
    TargetFrameTextureFrameRaidTargetIcon:SetPoint("TOPRIGHT", borderframe2, "TOPRIGHT", -4, 4);
    
    TargetFrameTextureFramePVPIcon:ClearAllPoints(); 
    TargetFrameTextureFramePVPIcon:SetPoint("CENTER", borderframe2, "RIGHT", 4, -5);
    TargetFrameTextureFramePVPIcon:SetWidth(36);    
    TargetFrameTextureFramePVPIcon:SetHeight(36);
    
    TargetFrameTextureFramePetBattleIcon:ClearAllPoints(); 
    TargetFrameTextureFramePetBattleIcon:SetPoint("CENTER", borderframe2, "RIGHT", 0, 0);
    
    
    TargetFrameTextureFrameQuestIcon:ClearAllPoints(); 
    TargetFrameTextureFrameQuestIcon:SetPoint("CENTER", borderframe1, "RIGHT", 0, 0);
    
    TargetFrameTextureFrameDeadText:ClearAllPoints(); 
    TargetFrameTextureFrameDeadText:SetPoint("CENTER", TargetFrameHealthBar, "CENTER", 0, 0);
    
    CleanUI_MoveTargetOfTargetIcons();
    CleanUI_SetSpellbarAspect(TargetFrameSpellBar);
end

function CleanUI_MoveTargetOfTargetIcons()
    if (InCombatLockdown() == nil) then
        TargetFrameToT:SetWidth(112);
        TargetFrameToT:SetHeight(48);
        TargetFrameToT:SetHitRectInsets(0, 0, 0, 0);
        
        TargetFrameToT:ClearAllPoints();
        TargetFrameToT:SetPoint("TOPLEFT", 120, -PET_YPOS_OFFSET);
    end
    
    CleanUI_SetBackdrop(TargetFrameToT);
	
    TargetFrameToTPortrait:Hide();
    TargetFrameToTBackground:Hide();
    TargetFrameToTTextureFrameTexture:Hide();

    TargetFrameToTTextureFrameName:ClearAllPoints();
	TargetFrameToTTextureFrameName:SetPoint("CENTER", 0, 15);
	TargetFrameToTTextureFrameName:SetJustifyH("CENTER");
    
    TargetFrameToTHealthBar:SetWidth(103);
    TargetFrameToTHealthBar:SetHeight(26);  
    TargetFrameToTHealthBar:ClearAllPoints();
    TargetFrameToTHealthBar:SetPoint("TOP", 0, -4);
    TargetFrameToTHealthBar:EnableMouse(false);
    
    TargetFrameToTManaBar:SetWidth(103);    
    TargetFrameToTManaBar:SetHeight(14);
    TargetFrameToTManaBar:ClearAllPoints();
    TargetFrameToTManaBar:SetPoint("TOP", 0, -30);
    TargetFrameToTManaBar:EnableMouse(false);
    
    TargetFrameToTTextureFrameDeadText:ClearAllPoints(); 
    TargetFrameToTTextureFrameDeadText:SetPoint("CENTER", TargetFrameToTHealthBar, "CENTER", 0, 0);
end

function CleanUI_MoveFocusIcons()
    local fullsize = GetCVarBool("fullSizeFocusFrame");
    
    if (InCombatLockdown() == nil) then
        FocusFrame:SetWidth(UNIT_FRAME_WIDTH);
        FocusFrame:SetHeight(90);
        FocusFrame:SetHitRectInsets(165, 0, 0, 37);
    end
    
    CleanUI_LoadFramePosition(FocusFrame, 560, -10);
    
    FocusFrame.powerBarAlt:ClearAllPoints(); 
    FocusFrame.powerBarAlt:SetPoint("LEFT", FocusFrame, "RIGHT", 0, 5);
    
	local borderframe1 = _G["CUI_FocusFrameBorder1"];
    if (not borderframe1) then
        borderframe1 = CreateFrame("Frame", "CUI_FocusFrameBorder1", FocusFrame);
	end
	borderframe1:SetPoint("TOPLEFT", 0, 0);
    borderframe1:SetWidth(BARFRAME_WIDTH);
    borderframe1:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe1:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe1);
    
	local borderframe2 = _G["CUI_FocusFrameBorder2"];
    if (not borderframe2) then
        borderframe2 = CreateFrame("Frame", "CUI_FocusFrameBorder2", FocusFrame);
	end
	borderframe2:SetPoint("TOPLEFT", borderframe1, "TOPRIGHT", -4, 0);
    borderframe2:SetWidth(UNIT_FRAME_HEIGHT);
    borderframe2:SetHeight(UNIT_FRAME_HEIGHT);
    borderframe2:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe2);
    
    FocusFrameBackground:Hide();
    FocusFrameNameBackground:Hide();
    FocusFrameTextureFrameTexture:Hide();
    FocusFramePortrait:Hide();
    
    FocusFrameBackground:SetTexture(nil);
    FocusFrameNameBackground:SetTexture(nil);
    FocusFrameTextureFrameTexture:SetTexture(nil);
    FocusFramePortrait:SetTexture(nil);
    
    FocusFrameTextureFrameName:ClearAllPoints();
    FocusFrameTextureFrameName:SetPoint("TOP", borderframe1, "TOP", 0, -5);
    FocusFrameTextureFrameName:SetWidth(BARS_WIDTH);
    FocusFrameTextureFrameName:SetJustifyH("CENTER");
	
    CUI_FocusInfo:ClearAllPoints(); 
    CUI_FocusInfo:SetPoint("BOTTOM", borderframe1, "TOP", 0, -1);
    
    FocusFrameNumericalThreat:SetWidth(70);
    FocusFrameNumericalThreat:ClearAllPoints(); 
    FocusFrameNumericalThreat:SetPoint("TOP", borderframe2, "TOP", 0, -3);
	
	for i, child in ipairs({FocusFrameNumericalThreat:GetRegions()}) do
	    if (not child:GetName()) then
	        child:SetTexture();
	    end
	end
	
    FocusFrameHealthBar:SetWidth(BARS_WIDTH + 1);
    FocusFrameHealthBar:SetHeight(HEALTHBAR_HEIGHT); 
    FocusFrameHealthBar:ClearAllPoints(); 
    FocusFrameHealthBar:SetPoint("TOPLEFT", 4, HEALTHBAR_YPOS_OFFSET);    
    FocusFrameTextureFrameHealthBarText:ClearAllPoints(); 
    FocusFrameTextureFrameHealthBarText:SetPoint("BOTTOM", FocusFrameHealthBar, "BOTTOM", 0, 5);
    
    FocusFrameManaBar:SetWidth(BARS_WIDTH + 1);    
    FocusFrameManaBar:SetHeight(MANABAR_HEIGHT);
    FocusFrameManaBar:ClearAllPoints(); 
    FocusFrameManaBar:SetPoint("TOP", FocusFrameHealthBar, "BOTTOM", 0, 0);
    
    FocusFrameTextureFrameManaBarText:ClearAllPoints(); 
    FocusFrameTextureFrameManaBarText:SetPoint("CENTER", FocusFrameManaBar, "CENTER", 0, 0);
    
    FocusFrameTextureFrameLeaderIcon:ClearAllPoints(); 
    FocusFrameTextureFrameLeaderIcon:SetPoint("TOPLEFT", borderframe2, "TOPLEFT", 3, -3);
    
    FocusFrameTextureFrameLevelText:ClearAllPoints(); 
    FocusFrameTextureFrameLevelText:SetPoint("BOTTOMRIGHT", borderframe2, "BOTTOMRIGHT", -4, 4);
    
    FocusFrameTextureFrameHighLevelTexture:ClearAllPoints(); 
    FocusFrameTextureFrameHighLevelTexture:SetPoint("BOTTOMRIGHT", borderframe2, "BOTTOMRIGHT", -4, 4);
    
    FocusFrameTextureFrameRaidTargetIcon:SetWidth(18);    
    FocusFrameTextureFrameRaidTargetIcon:SetHeight(18);
    FocusFrameTextureFrameRaidTargetIcon:ClearAllPoints(); 
    FocusFrameTextureFrameRaidTargetIcon:SetPoint("TOPRIGHT", borderframe2, "TOPRIGHT", -4, 4);
    
    FocusFrameTextureFramePVPIcon:ClearAllPoints(); 
    FocusFrameTextureFramePVPIcon:SetPoint("CENTER", borderframe2, "RIGHT", 4, -5);
    FocusFrameTextureFramePVPIcon:SetWidth(36);    
    FocusFrameTextureFramePVPIcon:SetHeight(36);
    
    FocusFrameTextureFramePetBattleIcon:ClearAllPoints(); 
    FocusFrameTextureFramePetBattleIcon:SetPoint("CENTER", borderframe2, "RIGHT", 0, 0);
    
    FocusFrameTextureFrameQuestIcon:ClearAllPoints(); 
    FocusFrameTextureFrameQuestIcon:SetPoint("CENTER", borderframe1, "RIGHT", 0, 0);
    
    FocusFrameTextureFrameDeadText:ClearAllPoints(); 
    FocusFrameTextureFrameDeadText:SetPoint("CENTER", FocusFrameHealthBar, "CENTER", 0, 0);
    
    CleanUI_MoveTargetOfFocusIcons();
    CleanUI_SetSpellbarAspect(FocusFrameSpellBar);
end

function CleanUI_MoveTargetOfFocusIcons()	
    local fullsize = GetCVarBool("fullSizeFocusFrame");

    if (InCombatLockdown() == nil) then
        FocusFrameToT:SetWidth(112);
        FocusFrameToT:SetHeight(48);
        FocusFrameToT:SetHitRectInsets(0, 0, 0, 0);
    end
    
    CleanUI_SetBackdrop(FocusFrameToT);
	
    FocusFrameToTPortrait:Hide();
    FocusFrameToTBackground:Hide();
    FocusFrameToTTextureFrameTexture:Hide();
    
    FocusFrameToTTextureFrameName:ClearAllPoints();
	FocusFrameToTTextureFrameName:SetPoint("CENTER", 0, 15);
	FocusFrameToTTextureFrameName:SetJustifyH("CENTER");
    
    FocusFrameToTHealthBar:SetWidth(103);
    FocusFrameToTHealthBar:SetHeight(26);  
    FocusFrameToTHealthBar:ClearAllPoints();
    FocusFrameToTHealthBar:SetPoint("TOP", 0, -4);
    FocusFrameToTHealthBar:EnableMouse(false);

    FocusFrameToTManaBar:SetWidth(103);    
    FocusFrameToTManaBar:SetHeight(14);
    FocusFrameToTManaBar:ClearAllPoints();
    FocusFrameToTManaBar:SetPoint("TOP", 0, -30);
    FocusFrameToTManaBar:EnableMouse(false);
    
    FocusFrameToTTextureFrameDeadText:ClearAllPoints(); 
    FocusFrameToTTextureFrameDeadText:SetPoint("CENTER", FocusFrameToTHealthBar, "CENTER", 0, 0);
    
    if (InCombatLockdown() == nil) then
        if (fullsize) then
            FocusFrameToT:SetScale(1.0);
        
            FocusFrameToT:ClearAllPoints();
            FocusFrameToT:SetPoint("TOPLEFT", 120, -PET_YPOS_OFFSET);
        else
            FocusFrameToT:SetScale(0.9);
        
            FocusFrameToT:ClearAllPoints();
            FocusFrameToT:SetPoint("TOPLEFT", 95, -66);
        end
    end
end

function CleanUI_BossTargetFrame_OnLoad(self, unit, event)
    if (CleanUIData.CHANGE_BOSSFRAMES == 1) then 
        CleanUI_BossTargetIcons(self);
    end
end

function CleanUI_BossTargetIcons(frame)
    local borderframe = _G["CUI_"..frame:GetName().."BorderFrame"];
    
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUI_"..frame:GetName().."BorderFrame", frame);
    end
    
    borderframe:SetWidth(138);
    borderframe:SetHeight(65);
    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", 0, 0);
    borderframe:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe);
    
    CleanUI_SetWhiteStatusTexture(_G[frame:GetName().."Flash"], borderframe);
    _G[frame:GetName().."Flash"]:SetDrawLayer("OVERLAY");
        
    _G[frame:GetName().."Background"]:Hide();
    _G[frame:GetName().."NameBackground"]:Hide();
    _G[frame:GetName().."TextureFrameTexture"]:Hide();
    _G[frame:GetName().."Portrait"]:Hide();
    
    _G[frame:GetName().."TextureFrameName"]:SetWidth(130);
    _G[frame:GetName().."TextureFrameName"]:SetHeight(10);
    _G[frame:GetName().."TextureFrameName"]:ClearAllPoints();
	_G[frame:GetName().."TextureFrameName"]:SetPoint("TOP", borderframe, "TOP", 0, -7);
    
    _G[frame:GetName().."HealthBar"]:SetWidth(130);
    _G[frame:GetName().."HealthBar"]:SetHeight(40); 
    _G[frame:GetName().."HealthBar"]:ClearAllPoints(); 
    _G[frame:GetName().."HealthBar"]:SetPoint("TOPLEFT", 4, -4);
    _G[frame:GetName().."TextureFrameHealthBarText"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameHealthBarText"]:SetPoint("CENTER", _G[frame:GetName().."HealthBar"], "CENTER", 0, -5);
    
    _G[frame:GetName().."ManaBar"]:SetWidth(130);    
    _G[frame:GetName().."ManaBar"]:SetHeight(17);
    _G[frame:GetName().."ManaBar"]:ClearAllPoints(); 
    _G[frame:GetName().."ManaBar"]:SetPoint("TOPLEFT", 4, -44);
    _G[frame:GetName().."TextureFrameManaBarText"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameManaBarText"]:SetPoint("CENTER", _G[frame:GetName().."ManaBar"], "CENTER", 0, 0);
    
    _G[frame:GetName().."TextureFrameLeaderIcon"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameLeaderIcon"]:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5);
    
    _G[frame:GetName().."TextureFrameLevelText"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameLevelText"]:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -100, -5);
    
    _G[frame:GetName().."TextureFrameHighLevelTexture"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameHighLevelTexture"]:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -100, -5);
    
    _G[frame:GetName().."TextureFrameRaidTargetIcon"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameRaidTargetIcon"]:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -100, -35);
    
    _G[frame:GetName().."TextureFramePVPIcon"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFramePVPIcon"]:SetPoint("CENTER", frame, "RIGHT", 8, 2);
    
    _G[frame:GetName().."TextureFrameDeadText"]:ClearAllPoints(); 
    _G[frame:GetName().."TextureFrameDeadText"]:SetPoint("CENTER", _G[frame:GetName().."HealthBar"], "CENTER", 0, 0);
    _G[frame:GetName().."TextureFrameDeadText"]:Hide();
end

function CleanUI_CleanArenaFrames()
    local frame;
    local parent = _G["ArenaEnemyFrame1"];
    for i=2, 5 do
        frame = _G["ArenaEnemyFrame"..i];
        frame:ClearAllPoints(); 
        frame:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", 0, -30);
        parent = frame;
    end
    
    for i=1, 5 do
        frame = _G["ArenaEnemyFrame"..i];
        CleanUI_CleanArenaFrame(frame);
    end
end

function CleanUI_CleanArenaFrame(frame)
    frame:SetSize(108, 40);

    local borderframe = _G["CUI_"..frame:GetName().."BorderFrame"];
    
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUI_"..frame:GetName().."BorderFrame", frame);
    end
    
    borderframe:SetWidth(10);
    borderframe:SetHeight(10);
    borderframe:ClearAllPoints();
    borderframe:SetAllPoints(frame);
    borderframe:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe);
    
    _G[frame:GetName().."Name"]:ClearAllPoints(); 
    _G[frame:GetName().."Name"]:SetPoint("TOP", frame, "TOP", 4, -4);
    
    _G[frame:GetName().."Background"]:Hide();
    _G[frame:GetName().."ClassPortrait"]:Hide();
    _G[frame:GetName().."Texture"]:Hide();
    
    _G[frame:GetName().."HealthBar"]:SetWidth(100);
    _G[frame:GetName().."HealthBar"]:SetHeight(20); 
    _G[frame:GetName().."HealthBar"]:ClearAllPoints(); 
    _G[frame:GetName().."HealthBar"]:SetPoint("TOPLEFT", 4, -4);
    _G[frame:GetName().."HealthBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    _G[frame:GetName().."HealthBarText"]:ClearAllPoints(); 
    _G[frame:GetName().."HealthBarText"]:SetPoint("CENTER", _G[frame:GetName().."HealthBar"], "CENTER", 0, 0);
    
    _G[frame:GetName().."ManaBar"]:SetWidth(100);    
    _G[frame:GetName().."ManaBar"]:SetHeight(12);
    _G[frame:GetName().."ManaBar"]:ClearAllPoints(); 
    _G[frame:GetName().."ManaBar"]:SetPoint("TOPLEFT", 4, -24);
    _G[frame:GetName().."ManaBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    _G[frame:GetName().."ManaBarText"]:ClearAllPoints(); 
    _G[frame:GetName().."ManaBarText"]:SetPoint("CENTER", _G[frame:GetName().."ManaBar"], "CENTER", 0, 0);
    
    CleanUI_CleanArenaPetFrame(frame);
end

function CleanUI_CleanArenaPetFrame(parent)
    local frame = _G[parent:GetName().."PetFrame"];
    frame:SetSize(68, 28);
    
    frame:ClearAllPoints(); 
    frame:SetPoint("TOPRIGHT", parent, "BOTTOMRIGHT", 0, 2);

    local borderframe = _G["CUI_"..frame:GetName().."BorderFrame"];
    
    if (not borderframe) then
        borderframe = CreateFrame("Frame", "CUI_"..frame:GetName().."BorderFrame", frame);
    end
    
    borderframe:SetWidth(10);
    borderframe:SetHeight(10);
    borderframe:ClearAllPoints();
    borderframe:SetAllPoints(frame);
    borderframe:SetFrameStrata("BACKGROUND");
    
    CleanUI_SetBackdrop(borderframe);
    
    _G[frame:GetName().."Portrait"]:Hide();
    _G[frame:GetName().."Texture"]:Hide();
    
    _G[frame:GetName().."HealthBar"]:SetWidth(60);
    _G[frame:GetName().."HealthBar"]:SetHeight(12); 
    _G[frame:GetName().."HealthBar"]:ClearAllPoints(); 
    _G[frame:GetName().."HealthBar"]:SetPoint("TOPLEFT", 4, -4);
    _G[frame:GetName().."HealthBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    
    _G[frame:GetName().."ManaBar"]:SetWidth(60);    
    _G[frame:GetName().."ManaBar"]:SetHeight(8);
    _G[frame:GetName().."ManaBar"]:ClearAllPoints(); 
    _G[frame:GetName().."ManaBar"]:SetPoint("TOPLEFT", 4, -16);
    _G[frame:GetName().."ManaBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
end

function CleanUI_SetSpellbarAspect(self)

    self:SetWidth(205);
    self:SetHeight(18);

	local spellBarName = self:GetName();

	local frameText = _G[spellBarName.."Text"];
	if ( frameText ) then
		frameText:SetFontObject(SystemFont_Shadow_Small);
		frameText:ClearAllPoints();
		frameText:SetPoint("CENTER", self, "CENTER", 0, 0);
	end

	local frameBorder = _G[spellBarName.."Border"];
	if ( frameBorder ) then
		frameBorder:SetTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
	end
	
	local frameIcon = _G[spellBarName.."Icon"];
	if ( frameIcon ) then
        frameIcon:SetWidth(20);
        frameIcon:SetHeight(18);
		frameIcon:ClearAllPoints();
		frameIcon:SetPoint("RIGHT", self, "LEFT", -1, 0);
		frameIcon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    end

	local frameBorderShield = _G[spellBarName.."BorderShield"];
	if ( frameBorderShield ) then
		frameBorderShield:SetTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
	end

	local frameFlash = _G[spellBarName.."Flash"];
	if ( frameFlash ) then
		frameFlash:SetTexture("Interface\\AddOns\\CleanUI\\skins\\spellbar_flash");
		frameFlash:ClearAllPoints();
        frameFlash:SetPoint("TOPLEFT", self, "TOPLEFT", -21, 0);
        frameFlash:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0);
	end

	local frameSpark = _G[spellBarName.."Spark"];
	if ( frameSpark ) then
		frameSpark:SetWidth(40);
		frameSpark:SetHeight(40);
	end
end

function CleanUI_SetTexture(object, texture)
	object:SetNormalTexture(texture);
	object:SetPushedTexture(texture);
end

function CleanUI_SetTextureCoords(object, index)
	object:GetNormalTexture():SetTexCoord(CUI_RAIDICONS[index].left,CUI_RAIDICONS[index].right,CUI_RAIDICONS[index].top,CUI_RAIDICONS[index].bottom);
	object:GetPushedTexture():SetTexCoord(CUI_RAIDICONS[index].left,CUI_RAIDICONS[index].right,CUI_RAIDICONS[index].top,CUI_RAIDICONS[index].bottom);
end

function CleanUI_SetLootSpecialization(id)
	CleanUI_UpdateLootSpecialization();
end

function CleanUI_UpdateLootSpecialization()	
	CleanUI_StartDelay(1, CleanUI_UpdateLootSpecializationIcon);
end

function CleanUI_UpdateLootSpecializationIcon()	
	local frame = CUI_PlayerFrameBorder1;
	
	if (not frame.SpecIcon) then
        frame.SpecIcon = frame:CreateTexture(frame:GetName().."SpecIcon", BACKGROUND);
		frame.SpecIcon:SetPoint("TOPLEFT", frame, "TOPLEFT", 4, -4);
		frame.SpecIcon:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4);
		--frame.SpecIcon:SetSize(12, 12);
		
		-- FIXME Position fr "LootSpecializationIcon" suchen!
		
		frame.SpecIcon:SetTexCoord(0.08, 0.92, 0.08, 0.92);
        frame.SpecIcon:SetTexture(nil);
	end

	local specID = GetLootSpecialization();
	if ( specID and specID > 0 ) then
		local id, name, description, texture, background, role, class = GetSpecializationInfoByID(specID);
		frame.SpecIcon:SetTexture(texture);
		--frame.SpecIcon:SetTexture("Interface\\TALENTFRAME\\"..background);
		
		frame.SpecIcon:Show();
	else
		frame.SpecIcon:Hide();
	end
end

function CleanUI_RaidIconUpdate()  
    --pet
    
    local button = getglobal("CleanUIPetRaidIcon");

    if (UnitExists("pet") == 1 and _G["PetFrame"]:IsVisible()) then		
        
        if (UnitExists("pettarget") == 1) then    
            local index = GetRaidTargetIndex("pettarget");
            
            if (index == nil or index == 0) then
                CleanUI_SetTexture(button, "Interface\\AddOns\\CleanUI\\skins\\assist");
                CleanUI_SetTextureCoords(button, 9);
            else
                CleanUI_SetTexture(button, CUI_RAIDICON_FILE);
                CleanUI_SetTextureCoords(button, index);
            end
        else
            CleanUI_SetTexture(button, CUI_RAIDICON_FILE);
            CleanUI_SetTextureCoords(button, 0);
        end
    end

    -- party
	for i=1, 4, 1 do
		local button = getglobal("CleanUIParty"..i.."RaidIcon");
	
		if (UnitExists("party"..i) == 1 and _G["PartyMemberFrame"..i]:IsVisible()) then
		    
		    if (UnitExists("party"..i.."target") == 1) then		    
		        local index = GetRaidTargetIndex("party"..i.."target");
		    
                if (index == nil or index == 0) then
                    CleanUI_SetTexture(button, "Interface\\AddOns\\CleanUI\\skins\\assist");
                    CleanUI_SetTextureCoords(button, 9);
                else
                    CleanUI_SetTexture(button, CUI_RAIDICON_FILE);
                    CleanUI_SetTextureCoords(button, index);
                end
            else
                CleanUI_SetTexture(button, CUI_RAIDICON_FILE);
                CleanUI_SetTextureCoords(button, 0);
            end
		end
	end
end

function CleanUI_UnitPopup_AppendUnitFramePositions(dropdownMenu, which, unit, name, userData)

    if (unit ~= "player" or UIDROPDOWNMENU_MENU_LEVEL > 1) then
        return;
    end
        
    local info = UIDropDownMenu_CreateInfo();
    info.text = CONFIG_UNITFRAME_POSITIONS;
    info.value = "UNITFRAME_POSITIONS";
    info.notCheckable = 1;
    info.isNoRadio = 1;
    info.hasArrow = 1;
    info.nested = 1;  
    UIDropDownMenu_AddButton(info);    
    
    UnitPopupMenus["UNITFRAME_POSITIONS"] = { "UNITFRAME_POSITION1", "UNITFRAME_POSITION2",
                                              "UNITFRAME_POSITION_CUSTOM1", "UNITFRAME_POSITION_CUSTOM2", "UNITFRAME_POSITION_CUSTOM3" };    
    
    UnitPopupButtons["UNITFRAME_POSITION1"] = { text = CONFIG_DEFAULT_UNITFRAME_POSITIONS, dist = 0 };
    UnitPopupButtons["UNITFRAME_POSITION2"] = { text = CONFIG_BATTLE_UNITFRAME_POSITIONS, dist = 0 };
    UnitPopupButtons["UNITFRAME_POSITION_CUSTOM1"] = { text = CUSTOM.." 1", dist = 0 };
    UnitPopupButtons["UNITFRAME_POSITION_CUSTOM2"] = { text = CUSTOM.." 2", dist = 0 };
    UnitPopupButtons["UNITFRAME_POSITION_CUSTOM3"] = { text = CUSTOM.." 3", dist = 0 };
end

function CleanUI_UnitPopup_OnClick(self)
    if (self.value == "UNITFRAME_POSITION1") then
        CleanUIConfig_LoadDefaultUnitFramePositions();
    elseif (self.value == "UNITFRAME_POSITION2") then
        CleanUIConfig_LoadBattleUnitFramePositions();
    elseif (self.value == "UNITFRAME_POSITION_CUSTOM1") then
        CleanUI_CustomUnitFramePositionsClicked(1);
    elseif (self.value == "UNITFRAME_POSITION_CUSTOM2") then
        CleanUI_CustomUnitFramePositionsClicked(2);
    elseif (self.value == "UNITFRAME_POSITION_CUSTOM3") then
        CleanUI_CustomUnitFramePositionsClicked(3);
    end 
end

function CleanUI_CustomUnitFramePositionsClicked(index)
    if (IsControlKeyDown()) then
        CleanUIConfig_SaveCustomUnitFramePosition(index);
    else
        CleanUIConfig_LoadCustomUnitFramePosition(index);
    end
end