local PARTY_UNITS = {"player", "party1", "party2", "party3", "party4"};

local XP_TEXT_FULL = "|cffffffff%d / %d (%d%%)|r\32";
local XP_TEXT_NAME_X_XMAX = "|cffffffff%d / %d|r\32";
local XP_TEXT_NAME_PERCENT = "|cffffffff%d%%|r\32";

local XP_EXHAUSTION_TEXT = "|cffffffff%d (%d%%)|r\32";

function CleanUI_InitXpFrame()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI");
    end

    CleanUI_SetBackdrop(CleanUIXpFrame);
    CleanUIXpFrame:EnableMouse(false);
    CleanUI_CleanXpFrame();
    
    CleanUI_XpFrameHookEvents();
    CleanUI_RegisterXpFrameEvents();
end
    
function CleanUI_RegisterXpFrameEvents()         
    local frame = CreateFrame("Frame");
    
    frame:RegisterEvent("ADDON_LOADED");  
    frame:RegisterEvent("GROUP_ROSTER_UPDATE");  
    frame:RegisterEvent("PLAYER_XP_UPDATE");  
    frame:RegisterEvent("CHAT_MSG_ADDON");  
    frame:RegisterEvent("UPDATE_EXHAUSTION"); 
    
    frame:HookScript("OnEvent", CleanUI_XpFrameOnEvent); 
end

function CleanUI_XpFrameOnEvent(self, event, ...)
	if (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "CleanUI") then            
            CleanUI_SendXpUpdate();
        end
    end
    if (event == "GROUP_ROSTER_UPDATE") then
        CleanUI_UpdateXpFrame();
    end
    if (event == "PLAYER_XP_UPDATE" or event == "UPDATE_EXHAUSTION" or event == "GROUP_ROSTER_UPDATE") then
        CleanUI_UpdateXpFrame();
        CleanUI_SendXpUpdate();
    end
    if (event == "CHAT_MSG_ADDON") then
        local prefix, message, type, sender = ...;
        CleanUI_ReceiveXpUpdate(prefix, message, type, sender);
    end
end

function CleanUI_XpFrameHookEvents()    
    MainMenuExpBar:HookScript("OnMouseUp", CleanUIConfig_ToggleXpFrame);
end

function CleanUI_CleanXpFrame()        
    CleanUI_UpdateXpFrame();
    CleanUI_XpFrame_Resize();
end

function CleanUI_UpdateXpFrame()    
    if (CleanUIData.SHOW_XPFRAME == nil) then
        CleanUIData.SHOW_XPFRAME = 0;
    end

    if (CleanUIData.SHOW_XPFRAME == 1) then
        CleanUIXpFrame:Show();
    else
        CleanUIXpFrame:Hide();
        return;
    end
    
 	CleanUIXpFrame:SetClampedToScreen(true);
 	CleanUIXpFrame:SetClampRectInsets(1, -1, -1, 1);
    
    cui_party_xp[UnitName("player")] = {level=UnitLevel("player"), xp=UnitXP("player"), xpmax=UnitXPMax("player"), xpexhaustion=GetXPExhaustion()};
    
    local count = 0;

	for index, id in pairs(PARTY_UNITS) do
	
	 	local name = UnitName(id);
	 	
	 	local xpbar = _G["CleanUIXpFrame_group"..index.."xpbar"];
	 	local xpexhaustionbar = _G["CleanUIXpFrame_group"..index.."xpbarExhaustion"];
	 	local xpbarunit = _G["CleanUIXpFrame_group"..index.."xpbarUnit"];
	 	local xpbarinfo = _G["CleanUIXpFrame_group"..index.."xpbarInfo"];
                    
        xpbar:SetAttribute("xp", nil);
        xpbar:SetAttribute("xpmax", nil);
        xpbar:SetAttribute("pxp", nil);
        xpbar:SetAttribute("xpexhaustion", nil);
        
        xpbar:SetHeight(CleanUIData.XPBAR_HEIGHT);
        xpexhaustionbar:SetHeight(CleanUIData.XPBAR_HEIGHT);
	 	
	 	if (not name) then
	 	    xpbar:Hide();
	 	    xpbarinfo:SetText("");
	 	else
	 	    local _, classFileName = UnitClass(id);
	 	    local classTextColor = RAID_CLASS_COLORS[classFileName];
		    
		    if (cui_party_xp[name] == nil) then		    
		        if (CleanUIData.XPFRAME_UNKNOWN == 1) then
                    -- show unknown
                    xpbarunit:SetText(name);   
                    xpbarinfo:SetText("");                
                    xpbar:SetValue(0);
                    
                    xpbar:Show();                   
                    count = count + 1;	
                end
		    else
                local level = tonumber(cui_party_xp[name].level);
                
                if (level < MAX_PLAYER_LEVEL) then
                    local xp = tonumber(cui_party_xp[name].xp);
                    local xpmax = tonumber(cui_party_xp[name].xpmax);
                    local pxp = 100 * xp / xpmax;
                    
                    local xpexhaustion = tonumber(cui_party_xp[name].xpexhaustion);
                    local pxpexhaustion = 0;
                    
                    if (xpexhaustion) then
                        pxpexhaustion = 100 * xpexhaustion / xpmax;
                    end
                    
                    xpbar:SetAttribute("xp", xp);
                    xpbar:SetAttribute("xpmax", xpmax);
                    xpbar:SetAttribute("pxp", pxp);
                    xpbar:SetAttribute("xpexhaustion", xpexhaustion);
                    xpbar:SetAttribute("pxpexhaustion", pxpexhaustion);
                                           
                    local valueinfo = "";
                    
                    if (CleanUIData.XPFRAME_X_XMAX == 1 and CleanUIData.XPFRAME_PERCENT == 1) then
                        valueinfo = format(XP_TEXT_FULL, xp, xpmax, pxp);
                    elseif (CleanUIData.XPFRAME_X_XMAX == 1) then
                        valueinfo = format(XP_TEXT_NAME_X_XMAX, xp, xpmax);
                    elseif (CleanUIData.XPFRAME_PERCENT == 1) then
                        valueinfo = format(XP_TEXT_NAME_PERCENT, pxp);
                    end
                    
                    xpbarunit:SetText("|c333399ff"..level.."|r "..name);
                    xpbarinfo:SetText(valueinfo);                    
                    xpbar:SetMinMaxValues(0, xpmax);                 
                    xpbar:SetValue(xp);
                
                    xpbar:Show();                   
                    count = count + 1;	  
                    
                    if (xpexhaustion) then
                        xpbar:SetStatusBarColor(0.0, 0.39, 0.88);
                    else
                        xpbar:SetStatusBarColor(0.58, 0.0, 0.55);
                    end
                    
    
                    -- ok, tricky we need right to left...
                    
                    if (not xpexhaustion) then
                        xpexhaustion = 0;
                    end
                    
                    local exhaustiondiff = xpmax - xpexhaustion - xp;
                    
                    if (exhaustiondiff < 0) then
                        exhaustiondiff = 0;
                    end
                    
                    xpexhaustionbar:SetMinMaxValues(0, 1);
                    xpexhaustionbar:SetValue(1);
                    
                    local width = xpbar:GetWidth() / xpmax * exhaustiondiff;
                    
                    if (width < 0) then
                        width = 0;
                    end
                    
                    xpexhaustionbar:SetWidth(width);
                    
                    if (width > 0) then
                        xpexhaustionbar:Show();
                    else
                        xpexhaustionbar:Hide();
                    end
                else                  
		            if (CleanUIData.XPFRAME_MAXLEVEL == 1) then
                        -- show max level
                        xpbarunit:SetText(name);   
                        xpbarinfo:SetText("");                    
                        xpbar:SetValue(0);
                        xpexhaustionbar:Hide();
                    
                        xpbar:Show();                   
                        count = count + 1;	 
                    else
                        xpbarunit:SetText("");   
                        xpbarinfo:SetText("");                    
                        xpbar:SetValue(0);
                        xpexhaustionbar:Hide();
                    end  
                end
		    end    
	 	end
	 	
        xpbar:ClearAllPoints();
        xpbar:SetPoint("TOP", CleanUIXpFrame, "TOP", 1, -4 - ((count - 1) * CleanUIData.XPBAR_HEIGHT));	 	
	end
	
	CleanUIXpFrame:SetHeight(9 + count * CleanUIData.XPBAR_HEIGHT);
	
    if (count == 0) then
        CleanUIXpFrame:Hide();
    end
end

function CleanUI_XpFrame_Resize()
    CleanUIXpFrame:SetWidth(CleanUIData.XPFRAME_WIDTH);
    
	for index, id in pairs(PARTY_UNITS) do	 	
	 	local xpbar = _G["CleanUIXpFrame_group"..index.."xpbar"];
        xpbar:SetWidth(CleanUIData.XPFRAME_WIDTH - 8);
	end
end

function CleanUI_XpFrame_OnMouseDown(frame, button)
    if (button == "RightButton") then
        if (IsControlKeyDown()) then
            CleanUIXpFrame:StartMoving();
        end
    end
end

function CleanUI_XpFrame_OnMouseUp(frame, button)
    if (button == "RightButton") then
        CleanUIXpFrame:StopMovingOrSizing();
    end
end

function CleanUI_SetXpTooltip(self)
    local xp =  self:GetAttribute("xp");
    local xpmax = self:GetAttribute("xpmax");
    local pxp = self:GetAttribute("pxp");
    local xpexhaustion = self:GetAttribute("xpexhaustion");
    local pxpexhaustion = self:GetAttribute("pxpexhaustion");
    
    if (not xp) then
        return;
    end
 	
 	local erfahrung = 100;
 	
 	if (xpexhaustion) then
 	    erfahrung = 200;
 	end
                    
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    
    local xptext = format(XP_TEXT_FULL, xp, xpmax, pxp);
    
    local tooltipText = xptext..format(EXHAUST_TOOLTIP1, "", erfahrung);
    
    GameTooltip:SetText(tooltipText);
    
    GameTooltip:AddLine(" ");
    if (xpexhaustion) then
        local xpexhaustiontext = format(XP_EXHAUSTION_TEXT, xpexhaustion, pxpexhaustion);
        GameTooltip:AddLine(EXHAUSTION_LABEL..": "..xpexhaustiontext);
    else
        GameTooltip:AddLine(EXHAUST_TOOLTIP2);
    
        local exhaustionCountdown = nil;
        if (GetTimeToWellRested()) then
            exhaustionCountdown = GetTimeToWellRested() / 60;
        end
        
        if (exhaustionCountdown) then        
            GameTooltip:AddLine(" ");
            GameTooltip:AddLine(format(EXHAUST_TOOLTIP4, exhaustionCountdown));
        end
    end
		
    GameTooltip:Show();
end

function CleanUI_SendXpUpdate()   
    local name = UnitName("player");
    local level = UnitLevel("player");
    local xp = UnitXP("player");
    local xpmax = UnitXPMax("player");
    local xpexhaustion = GetXPExhaustion();
    
    if (not xpexhaustion) then
        xpexhaustion = "";
    end
    
    local message = name..";"..level..";"..xp..";"..xpmax..";"..xpexhaustion;
    SendAddonMessage("CleanUI_XP", message, "RAID");
end

function CleanUI_ReceiveXpUpdate(prefix, message, type, sender)
    if (prefix == nil or prefix ~= "CleanUI_XP") then
        return;
    end
    
    local name, _level, _xp, _xpmax, _xpexhaustion = strsplit(";", message, 5);
    
    
    if (_xpexhaustion == "") then
        _xpexhaustion = nil;
    end
    
    if (name ~= UnitName("player")) then
        cui_party_xp[name] = {level=_level, xp=_xp, xpmax=_xpmax, xpexhaustion=_xpexhaustion};
    end
    
    CleanUI_UpdateXpFrame();
end